package com.foreverwin.mesnac.anomaly.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.foreverwin.mesnac.anomaly.dto.AbnormalBillDisposeDto;
import com.foreverwin.mesnac.anomaly.dto.AbnormalBillDto;
import com.foreverwin.mesnac.anomaly.dto.NcCodesVO;
import com.foreverwin.mesnac.anomaly.dto.RouterBpmVO;
import com.foreverwin.mesnac.anomaly.mapper.AbnormalBillDisposeMapper;
import com.foreverwin.mesnac.anomaly.model.AbnormalBill;
import com.foreverwin.mesnac.anomaly.model.AbnormalBillDispose;
import com.foreverwin.mesnac.anomaly.service.AbnormalBillDisposeService;
import com.foreverwin.mesnac.anomaly.service.AbnormalBillService;
import com.foreverwin.mesnac.anomaly.service.AbnormalPlanService;
import com.foreverwin.mesnac.anomaly.service.FileUploadedService;
import com.foreverwin.mesnac.anomaly.utils.*;
import com.foreverwin.mesnac.common.constant.Constants;
import com.foreverwin.mesnac.common.dto.SfcDispatchDto;
import com.foreverwin.mesnac.common.enums.HandleEnum;
import com.foreverwin.mesnac.common.service.SfcDispatchCommonService;
import com.foreverwin.mesnac.common.util.StringUtil;
import com.foreverwin.mesnac.dispatch.model.SfcDispatch;
import com.foreverwin.mesnac.dispatch.service.SfcDispatchService;
import com.foreverwin.mesnac.meapi.dto.AbnormalRouterDto;
import com.foreverwin.mesnac.meapi.dto.RouterStepDto;
import com.foreverwin.mesnac.meapi.model.RouterStep;
import com.foreverwin.mesnac.meapi.service.NcCodeService;
import com.foreverwin.mesnac.meapi.service.RouterService;
import com.foreverwin.mesnac.meapi.service.RouterStepService;
import com.foreverwin.mesnac.meapi.util.StringUtils;
import com.foreverwin.modular.core.util.FrontPage;
import com.foreverwin.modular.core.util.R;
import com.foreverwin.modular.core.util.SpringUtil;
import com.sap.me.frame.service.CommonMethods;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 *
 * @author zjw
 * @since 2021-09-15
 */
@RestController
@RequestMapping("/Z-ABNORMAL-BILL")
public class AbnormalBillController {

    @Autowired
    public AbnormalBillService abnormalBillService;

    @Autowired
    private SfcDispatchCommonService sfcDispatchCommonService;

    @Autowired
    private FileUploadedService fileUploadedService;

    @Autowired
    private NcCodeService ncCodeService;

    @Autowired
    private RouterService routerService;

    @Autowired
    private RouterStepService routerStepService;

    @Autowired
    private AbnormalBillDisposeService abnormalBillDisposeService;

    @Autowired
    private AbnormalBillDisposeMapper abnormalBillDisposeMapper;

    @Autowired
    private AbnormalPlanService abnormalPlanService;

    /**
    * 根据id查询
    *
    * @param id 主键
    * @return
    */
    @ResponseBody
    @GetMapping("/{id:.+}")
    public R getAbnormalBillById(@PathVariable String id) {
        return R.ok( abnormalBillService.getById(id));
    }

    /**
     * 查询所有数据
     *
     * @return
     */
    @ResponseBody
    @GetMapping("")
    public R getAbnormalBillList(AbnormalBill abnormalBill){
        List<AbnormalBill> result;
        QueryWrapper<AbnormalBill> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(abnormalBill);
        result = abnormalBillService.list(queryWrapper);
        return R.ok(result);
    }

    /**
     * 分页查询数据
     *
     * @param frontPage  分页信息
     * @return
     */
    @ResponseBody
    @GetMapping("/page")
    public R page(FrontPage<AbnormalBill> frontPage, AbnormalBill abnormalBill){
        IPage result;
        QueryWrapper<AbnormalBill> queryWrapper = new QueryWrapper<>();
        String site = CommonMethods.getSite();
        abnormalBill.setSite(site);
        queryWrapper.setEntity(abnormalBill);
        if (frontPage.getGlobalQuery() != null && !"".equals(frontPage.getGlobalQuery().trim())) {
            //TODO modify global query
            queryWrapper.lambda().and(wrapper -> wrapper
                .like(AbnormalBill::getHandle, frontPage.getGlobalQuery())
                .or().like(AbnormalBill::getSite, frontPage.getGlobalQuery())
                .or().like(AbnormalBill::getAbnormalNo, frontPage.getGlobalQuery())
                .or().like(AbnormalBill::getStatus, frontPage.getGlobalQuery())
                .or().like(AbnormalBill::getCorrection, frontPage.getGlobalQuery())
                .or().like(AbnormalBill::getProgram, frontPage.getGlobalQuery())
                .or().like(AbnormalBill::getType, frontPage.getGlobalQuery())
                .or().like(AbnormalBill::getItemBo, frontPage.getGlobalQuery())
                .or().like(AbnormalBill::getSfc, frontPage.getGlobalQuery())
                .or().like(AbnormalBill::getOperation, frontPage.getGlobalQuery())
                .or().like(AbnormalBill::getWorkCenter, frontPage.getGlobalQuery())
                .or().like(AbnormalBill::getShopOrder, frontPage.getGlobalQuery())
                .or().like(AbnormalBill::getMessageType, frontPage.getGlobalQuery())
                .or().like(AbnormalBill::getResrce, frontPage.getGlobalQuery())
                .or().like(AbnormalBill::getNcCode, frontPage.getGlobalQuery())
                .or().like(AbnormalBill::getShutDown, frontPage.getGlobalQuery())
                .or().like(AbnormalBill::getResponseUser, frontPage.getGlobalQuery())
                .or().like(AbnormalBill::getPbDescription, frontPage.getGlobalQuery())
                .or().like(AbnormalBill::getNcQty, frontPage.getGlobalQuery())
                .or().like(AbnormalBill::getPbGrade, frontPage.getGlobalQuery())
                .or().like(AbnormalBill::getPbUser, frontPage.getGlobalQuery())
                .or().like(AbnormalBill::getPbQty, frontPage.getGlobalQuery())
                .or().like(AbnormalBill::getDiscover, frontPage.getGlobalQuery())
                .or().like(AbnormalBill::getInspector, frontPage.getGlobalQuery())
                .or().like(AbnormalBill::getEntityLocation, frontPage.getGlobalQuery())
                .or().like(AbnormalBill::getReportFrom, frontPage.getGlobalQuery())
                .or().like(AbnormalBill::getObjectBo, frontPage.getGlobalQuery())
                .or().like(AbnormalBill::getPbPhotoshop, frontPage.getGlobalQuery())
                .or().like(AbnormalBill::getCancelCode, frontPage.getGlobalQuery())
                .or().like(AbnormalBill::getCancelReason, frontPage.getGlobalQuery())
                .or().like(AbnormalBill::getCancelUser, frontPage.getGlobalQuery())
                .or().like(AbnormalBill::getReportSendUserGroup, frontPage.getGlobalQuery())
                .or().like(AbnormalBill::getCancelSendUserGroup, frontPage.getGlobalQuery())
                .or().like(AbnormalBill::getCreatedUser, frontPage.getGlobalQuery())
                .or().like(AbnormalBill::getModifiedUser, frontPage.getGlobalQuery())
                .or().like(AbnormalBill::getProductCategory, frontPage.getGlobalQuery())
                .or().like(AbnormalBill::getStepId, frontPage.getGlobalQuery())
                .or().like(AbnormalBill::getWorkOrder, frontPage.getGlobalQuery())
                .or().like(AbnormalBill::getItemNumber, frontPage.getGlobalQuery())
    );
        }
        result = abnormalBillService.page(frontPage.getPagePlus(), queryWrapper);
        return R.ok(result);
    }

    /**
     * 新增
     * @param abnormalBill  传递的实体
     * @return  null 失败  实体成功
     */
    @PostMapping
    public R save(@RequestBody AbnormalBill abnormalBill) {
        return R.ok(abnormalBillService.save(abnormalBill));
    }

    /**
     * 修改
     * @param abnormalBill  传递的实体
     * @return  null 失败  实体成功
     */
    @PutMapping
    public R updateById(@RequestBody AbnormalBill abnormalBill) {
        return R.ok(abnormalBillService.updateById(abnormalBill));
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     * @return 0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.DELETE, value = "/{id:.+}")
    public R removeById(@PathVariable("id") String id){
        return R.ok(abnormalBillService.removeById(id));
    }

    /**
     * 批量删除对象
     * @param ids 实体集合ID
     * @return  0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.POST, value = "/delete-batch")
    public R removeByIds(List<String> ids){
        return R.ok(abnormalBillService.removeByIds(ids));
    }

    /**
     * 拆分弹出框
     *
     * @return
     */
    @ResponseBody
    @GetMapping("getQCAbnormal")
    public R getQCAbnormal(){
        List<AbnormalBill> result;
        result = abnormalBillService.getQCAbnormal();
        return R.ok(result);
    }
    @GetMapping("getAbnormalSFCPage")
    public R getAbnormalSFCPage(FrontPage frontPage,@RequestParam(required =  false) Map paramMap){
        String globalQuery = frontPage.getGlobalQuery();
        return R.ok(abnormalBillService.getAbnormalSFCPage(frontPage.getPagePlus(),paramMap,globalQuery));
    }
    @GetMapping("getAbnormalByConditionalPage")
    public R getAbnormalByConditionalPage(FrontPage frontPage,@RequestParam(required =  false) Map paramMap){
        String globalQuery = frontPage.getGlobalQuery();
        return R.ok(abnormalBillService.getAbnormalByConditionalPage(frontPage.getPagePlus(),paramMap,globalQuery));
    }



    /**
     * 初始化
     * @param  type 异常的类型
     * @return
     */
    @GetMapping("/init")
    public R init(String type){
        return R.ok(abnormalBillService.init(type));
    }


    /**
     * 其他异常提报
     * @param abnormalBill
     * @return
     */
    @GetMapping("/anomalyReportOther")
    public R anomalyReportOther(AbnormalBill abnormalBill,
                                AbnormalBillDispose abnormalBillDispose,
                                @RequestParam List<String> dutyCauseType,
                                @RequestParam List<String> dutyType){
        return R.ok(abnormalBillService.anomalyReportOther(abnormalBill, abnormalBillDispose,dutyCauseType,dutyType));
    }

    /**
     * 质量异常提报
     * @param abnormalBill
     * @param abnormalBillDispose
     * @return
     */
    @GetMapping("/anomalyReport")
    public R anomalyReport(AbnormalBill abnormalBill,
                           AbnormalBillDispose abnormalBillDispose,
                           @RequestParam List<String> ncGroupAndNcCodes,
                           @RequestParam List<String> dutyCauseType,
                           @RequestParam List<String> dutyType){
        try{
            return R.ok(abnormalBillService.anomalyReport(abnormalBill, abnormalBillDispose,ncGroupAndNcCodes,dutyCauseType,dutyType));
        }catch(Exception e){
            return R.failed("质量异常提报失败:"+e.getMessage());
        }
    }

    /**
     * 设备异常提报
     * @param abnormalBill
     * @return
     */
    @GetMapping("/anomalyReportDevice")
    public R anomalyReportDevice(AbnormalBill abnormalBill, AbnormalBillDispose abnormalBillDispose){
        try{
            return R.ok(abnormalBillService.anomalyReportDevice(abnormalBill,abnormalBillDispose));
        }catch (Exception e){
            return R.failed("设备异常提报失败:"+e.getMessage());
        }

    }


    /**
     * 取消原因
     * @param handle
     * @param cancelBrowse
     * @return
     */
    @GetMapping("/cancelBrowse")
    public R cancelBrowse(String handle, String cancelBrowse,String code,String cancelSendUserGroup){
        try{
            return R.ok( abnormalBillService.cancelBrowse(handle,cancelBrowse,code,cancelSendUserGroup));
        }catch(Exception e){
            return R.failed("取消失败:"+e.getMessage());
        }
    }

    @GetMapping("/backLog")
    public R backLog(AbnormalBillDto abnormalBillDto){
        try{
            return R.ok(abnormalBillService.backLog(abnormalBillDto));
        }catch(Exception e){
            return R.failed("检索失败:"+e.getMessage());
        }
    }

    @GetMapping("/backLogDevice")
    public R backLogDevice(AbnormalBillDto abnormalBillDto){
        try{
            return R.ok(abnormalBillService.backLogByDeviceUser(abnormalBillDto));
        }catch(Exception e){
            return R.failed("检索失败:"+e.getMessage());
        }
    }

    @ResponseBody
    @PostMapping("/saveDevice")
    public R saveDevice(@RequestBody List<AbnormalBillDto> abnormalBillList) {
        Boolean flag;
        StringBuffer message = new StringBuffer();
        String site = CommonMethods.getSite();
        List<AbnormalBill> abnormalBills=new ArrayList<>();
        try {
            for (AbnormalBillDto ab:abnormalBillList) {
                AbnormalBill abnormalBill = new AbnormalBill();
                abnormalBill.setHandle("AbnormalBillBo:1000,"+ab.getAbnormalNo());
                abnormalBill.setUserConTime(LocalDateTime.now());
                abnormalBill.setUserIsCon("true");
                abnormalBill.setSite(site);
                abnormalBills.add(abnormalBill);
            }
            flag=abnormalBillService.deviceRepairComplete(abnormalBills);
        } catch (Exception e) {
            return R.failed(e.getMessage());
        }
        if (flag) {
            message.insert(0, "确认维修成功");

            return R.ok(message.toString());
        } else {
            message.insert(0, "确认维修失败：");
            return R.failed(message.toString());
        }

    }

    /**
     * bpm获取工艺路线
     * @param
     * @return
     */
    @GetMapping("/getRouting")
    public ResultVo getRouting(RouterBpmVO routerBpmVO, String secretKey){
        com.foreverwin.modular.core.util.CommonMethods.setSite("1000");
        com.foreverwin.modular.core.util.CommonMethods.setUser("BPM_USER");
        String site = com.foreverwin.modular.core.util.CommonMethods.getSite();
        if (!yunBpmConfig.secretKey.equals(secretKey)){
            return new ResultVo(ResultCode.FAILED,"秘钥错误");
        }
        try{
            AbnormalRouterDto abnormalRouterDto = new AbnormalRouterDto();
            abnormalRouterDto.setSite(site);
            abnormalRouterDto.setItem(routerBpmVO.getItem());
            List<AbnormalRouterDto> abnormalRouterDtoList = routerService.selectAllDisRouter(abnormalRouterDto);
            ArrayList<RouterBpmVO> routerBpmVOs = new ArrayList<>();
            for (AbnormalRouterDto routerDto : abnormalRouterDtoList) {
                RouterBpmVO bpmVO = new RouterBpmVO();
                bpmVO.setAbnormalNo(routerDto.getAbnormalNo());
                bpmVO.setRouter(routerDto.getRouter());
                bpmVO.setRouterType("C");
                bpmVO.setItemDescription(routerDto.getItemDescription());
                bpmVO.setItemRevision(routerDto.getRouterRevision());
                bpmVO.setPbDescription(routerDto.getPbDescription());
                bpmVO.setRouterRevision(routerDto.getRouterRevision());
                bpmVO.setPbDescription(routerDto.getPbDescription());
                bpmVO.setItem(routerDto.getItem());
                bpmVO.setShopOrder(routerBpmVO.getShopOrder());
                bpmVO.setSite(site);
                bpmVO.setRouterBo(routerDto.getRouterBo());
                RouterStep routerStep = new RouterStep();
                routerStep.setRouterBo(routerDto.getRouterBo());
                List<RouterStepDto> routerOperationByRouterBo = routerStepService.findRouterOperationByRouterBo(site, routerDto.getRouterBo());
                //判断当前SFC是否工序步骤标识重复
                if (StringUtil.notBlank(routerBpmVO.getSfc())){
                    List<SfcDispatch> sfcDispatchList = SpringUtil.getBean(SfcDispatchService.class).findSfcDispatch(site, routerBpmVO.getSfc(), null, null);
                    for (SfcDispatch sfcDispatch : sfcDispatchList) {
                        for (RouterStepDto routerStepDto : routerOperationByRouterBo) {
                            if (sfcDispatch.getStepId().equals(routerStepDto.getStepId())){
                                //保存操作日志
                                StringUtil.saveOperLog("BPM获取工艺路线接口,SFC："  + routerBpmVO.getSfc(),"bpmGetRouting",
                                        routerBpmVO.toString(),null,"此产品条码工序步骤标识重复：" + routerBpmVO.getSfc() + "，请重新出方案！");
                                return new ResultVo(ResultCode.FAILED,"此SFC产品条码工序步骤标识重复：" + routerBpmVO.getSfc() + "，请重新出方案！");
                            }
                        }
                    }
                }
                bpmVO.setRouterStep(routerOperationByRouterBo);
                routerBpmVOs.add(bpmVO);
            }
            return new ResultVo(ResultCode.SUCCESS,routerBpmVOs);
        }catch(Exception e){
            return new ResultVo(ResultCode.FAILED,R.failed(e.getMessage()).getMsg());
        }
    }


    /**
     * bpm售后异常提报
     * @param
     * @return
     */
    @PostMapping("/bpmAbnormalReport")
    public ResultVo bpmAbnormalReport(String secretKey, AbnormalBill abnormalBill, AbnormalBillDispose abnormalBillDispose,
                                      NcCodesVO ncCodesVO){
        com.foreverwin.modular.core.util.CommonMethods.setSite("1000");
        String site = com.foreverwin.modular.core.util.CommonMethods.getSite();
        com.foreverwin.modular.core.util.CommonMethods.setUser("BPM_USER");
        LocalDateTime now = LocalDateTime.now();
        if (!yunBpmConfig.secretKey.equals(secretKey)){
            return new ResultVo(ResultCode.FAILED,"秘钥错误");
        }
        try{
            String abnormalNo = abnormalBillService.getNextNumber("Z");
            abnormalBill.setBpmSubmit("true");
            abnormalBill.setAbnormalNo(abnormalNo);
            abnormalBill.setStatus("N");
            abnormalBill.setSite(site);
            abnormalBill.setType("Z");
            abnormalBill.setReportFrom("J");
            abnormalBill.setItemBo(HandleEnum.ITEM.getHandle(site,abnormalBill.getItemBo(),"A"));
            abnormalBill.setReportSendUserGroup("bpmUserGroup");
            abnormalBill.setSite(site);
            abnormalBill.setPbUser(abnormalBill.getPbUser());
            abnormalBill.setResponseUser(abnormalBill.getPbUser());
            abnormalBill.setResponseDateTime(now);
            abnormalBillDispose.setSite(site);
            com.foreverwin.modular.core.util.CommonMethods.setUser(abnormalBill.getPbUser());
            abnormalBillService.anomalyReport(abnormalBill, abnormalBillDispose,
                    ncCodesVO.getNcGroupAndNcCodes(), ncCodesVO.getDutyCauseType(), ncCodesVO.getDutyType());
            return new ResultVo(ResultCode.SUCCESS,abnormalBill);
        }catch(Exception e){
            return new ResultVo(ResultCode.FAILED,R.failed(e.getMessage()).getMsg());
        }
    }

    /**
     * bpm方案确认回传mes接口
     * @param
     * @return
     */
    @PostMapping("/bpmPlanConfirm")
    public ResultVo bpmPlanConfirm(String secretKey,AbnormalBill abnormalBill,AbnormalBillDispose abnormalBillDispose){
        com.foreverwin.modular.core.util.CommonMethods.setSite("1000");
        com.foreverwin.modular.core.util.CommonMethods.setUser("BPM_USER");
        String site = com.foreverwin.modular.core.util.CommonMethods.getSite();
        if (!yunBpmConfig.secretKey.equals(secretKey)){
            return new ResultVo(ResultCode.FAILED,"秘钥错误");
        }else if (!StringUtil.isBlank(abnormalBillDispose.getResolveUser())){
            com.foreverwin.modular.core.util.CommonMethods.setUser(abnormalBillDispose.getResolveUser());
        }

        try{
            abnormalBill.setHandle(HandleEnum.ABNORMAL_BILL.getHandle(site,abnormalBill.getAbnormalNo()));
            abnormalBill.setItemBo(null);
            //保存操作日志
            StringUtil.saveOperLog("BPM质量系统方案确认回调接口，单号："  + abnormalBill.getAbnormalNo(),"bpmPlanConfirm",
                    abnormalBill.toString() + abnormalBillDispose.toString(),null,null);
            abnormalBillService.programConfirm(abnormalBill,abnormalBillDispose);
            return new ResultVo(ResultCode.SUCCESS,"BPM方案确认回传成功");
        }catch(Exception e){
            return new ResultVo(ResultCode.FAILED,R.failed(e.getMessage()).getMsg());
        }
    }

    /**
     * bpm响应处理
     * @param abnormalBill
     * @return
     */
    @PostMapping("/bpmResProcessing")
    public ResultVo bpmResProcessing(String secretKey,AbnormalBill abnormalBill){
        if (!yunBpmConfig.secretKey.equals(secretKey)){
            return new ResultVo(ResultCode.FAILED,"秘钥错误");
        }else if (StringUtils.isBlank(abnormalBill.getAbnormalNo())){
            return new ResultVo(ResultCode.FAILED,"响应处理失败:异常单号为空");
        }
        com.foreverwin.modular.core.util.CommonMethods.setValue("SITE","1000");
        String site = com.foreverwin.modular.core.util.CommonMethods.getSite();
        com.foreverwin.modular.core.util.CommonMethods.setUser(abnormalBill.getResponseUser());
        try{
            AbnormalBill abnormalBillOld = abnormalBillService.getById(HandleEnum.ABNORMAL_BILL.getHandle(site,abnormalBill.getAbnormalNo()));
            //保存操作日志
            StringUtil.saveOperLog("BPM质量系统响应处理回调接口，单号："  + abnormalBill.getAbnormalNo(),"bpmResProcessing",
                    abnormalBill.toString(),null,null);
            if ("N".equals(abnormalBillOld.getStatus())){
                abnormalBillService.resProcessing(abnormalBillOld);
            }else {
                return new ResultVo(ResultCode.FAILED,"响应处理失败，该异常单号MES中已响应处理:" + abnormalBill.getAbnormalNo());
            }
            return new ResultVo(ResultCode.SUCCESS,"BPM响应处理成功");
        }catch(Exception e){
            return new ResultVo(ResultCode.FAILED,"响应处理失败:" + e.getMessage());
        }
    }

    /**
     * bpm判责提交
     * @param abnormalBill
     * @param abnormalBillDispose
     * @return
     */
    @PostMapping("/bpmDutyConfirm")
    public ResultVo bpmDutyConfirm(String secretKey,AbnormalBill abnormalBill,
                                   AbnormalBillDispose abnormalBillDispose,NcCodesVO ncCodesVO){
        com.foreverwin.modular.core.util.CommonMethods.setValue("SITE","1000");
        com.foreverwin.modular.core.util.CommonMethods.setUser(abnormalBillDispose.getDutyUser());
        String site = com.foreverwin.modular.core.util.CommonMethods.getSite();
        if (!yunBpmConfig.secretKey.equals(secretKey)){
            return new ResultVo(ResultCode.FAILED,"秘钥错误");
        }
        try{
            AbnormalBill abnormalBillOld = abnormalBillService.getById(HandleEnum.ABNORMAL_BILL.getHandle(site,abnormalBill.getAbnormalNo()));
            //保存操作日志
            StringUtil.saveOperLog("BPM质量系统判责提交回调接口，单号："  + abnormalBill.getAbnormalNo(),"bpmDutyConfirm",
                    abnormalBill.toString() + abnormalBillDispose.toString(),null,null);
            abnormalBillService.dutyConfirm(abnormalBillOld,abnormalBillDispose,ncCodesVO.getDutyCauseType(),ncCodesVO.getDutyType());
            return new ResultVo(ResultCode.SUCCESS,"BPM判责提交成功");
        }catch(Exception e){
            return new ResultVo(ResultCode.FAILED,R.failed(e.getMessage()).getMsg());
        }
    }

    /**
     * bpm异常取消
     * @param secretKey
     * @param abnormalBill
     * @return
     */
    @PostMapping("/bpmCancelBrowse")
    public ResultVo bpmCancelBrowse(String secretKey,AbnormalBill abnormalBill){
        if (!yunBpmConfig.secretKey.equals(secretKey)){
            return new ResultVo(ResultCode.FAILED,"秘钥错误");
        }else if (StringUtils.isBlank(abnormalBill.getAbnormalNo())){
            return new ResultVo(ResultCode.FAILED,"BPM取消失败:异常单号为空");
        }
        try{
            LocalDateTime now = LocalDateTime.now();
            AbnormalBill bill = new AbnormalBill();
            bill.setAbnormalNo(abnormalBill.getAbnormalNo());
            List<AbnormalBill> abnormalBills = abnormalBillService.selectList(bill);
            abnormalBills.get(0).setStatus(Constants.CANCEL);
            abnormalBills.get(0).setCancelReason(abnormalBill.getCancelReason());
            abnormalBills.get(0).setCancelSendUserGroup("BPM_CancelUserGroup");
            abnormalBills.get(0).setCancelUser(abnormalBill.getCancelUser());
            abnormalBills.get(0).setCancelDateTime(now);
            //保存操作日志
            StringUtil.saveOperLog("BPM质量系统异常取消回调接口，单号："  + abnormalBill.getAbnormalNo(),"bpmCancelBrowse",
                    abnormalBill.toString(),null,null);
            abnormalBillService.saveOrUpdate(abnormalBills.get(0));
            return new ResultVo(ResultCode.SUCCESS,"BPM异常取消成功");
        }catch(Exception e){
            return new ResultVo(ResultCode.FAILED,"BPM取消失败:" + e.getMessage());
        }
    }

    /**
     * bpm异常修改
     * @param secretKey
     * @param abnormalBill
     * @return
     */
    @PostMapping("/bpmAbnormalModify")
    public ResultVo bpmAbnormalModify(String secretKey,AbnormalBill abnormalBill,AbnormalBillDispose abnormalBillDispose){
        com.foreverwin.modular.core.util.CommonMethods.setSite("1000");
        com.foreverwin.modular.core.util.CommonMethods.setUser(abnormalBill.getPbUser());
        if (!yunBpmConfig.secretKey.equals(secretKey)){
            return new ResultVo(ResultCode.FAILED,"秘钥错误");
        }else if (StringUtils.isBlank(abnormalBill.getAbnormalNo())){
            return new ResultVo(ResultCode.FAILED,"BPM异常修改:异常单号为空");
        }
        try{
            //保存操作日志
            StringUtil.saveOperLog("BPM质量系统异常修改回调接口，单号："  + abnormalBill.getAbnormalNo(),"bpmAbnormalModify",
                    abnormalBill.toString(),null,null);
            LocalDateTime now = LocalDateTime.now();
            AbnormalBill bill = new AbnormalBill();
            bill.setAbnormalNo(abnormalBill.getAbnormalNo());
            List<AbnormalBill> abnormalBills = abnormalBillService.selectList(bill);
            abnormalBills.get(0).setPbDescription(abnormalBill.getPbDescription());
            abnormalBills.get(0).setNcQty(abnormalBill.getNcQty());
            abnormalBills.get(0).setDiscover(abnormalBill.getDiscover());
            abnormalBills.get(0).setPbGrade(abnormalBill.getPbGrade());
            abnormalBills.get(0).setPbUser(abnormalBill.getPbUser());
            abnormalBills.get(0).setInspector(abnormalBill.getInspector());
            abnormalBills.get(0).setEntityLocation(abnormalBill.getEntityLocation());
            abnormalBills.get(0).setPbQty(abnormalBill.getPbQty());
            abnormalBills.get(0).setModifiedUser(abnormalBill.getPbUser());
            abnormalBills.get(0).setModifiedDateTime(now);
            abnormalBillService.saveOrUpdate(abnormalBills.get(0));
            AbnormalBillDispose dispose = new AbnormalBillDispose();
            dispose.setHandle(HandleEnum.ABNORMAL_BILL_DISPOSE.getHandle("1000",abnormalBill.getAbnormalNo()));
            List<AbnormalBillDispose> abnormalBillDisposes = abnormalBillDisposeService.selectList(dispose);
            abnormalBillDisposes.get(0).setDutyUser(abnormalBillDispose.getDutyUser());
            abnormalBillDisposes.get(0).setDutyDepart(abnormalBillDispose.getDutyDepart());
            abnormalBillDisposes.get(0).setDutyType(abnormalBillDispose.getDutyType());
            abnormalBillDisposes.get(0).setPrincipalUser(abnormalBillDispose.getPrincipalUser());
            abnormalBillDisposes.get(0).setDutyCauseDescription(abnormalBillDispose.getDutyCauseDescription());
            abnormalBillDisposes.get(0).setDutyCauseType(abnormalBillDispose.getDutyCauseType());
            abnormalBillDisposes.get(0).setModifiedUser(abnormalBill.getPbUser());
            abnormalBillDisposes.get(0).setModifiedDateTime(now);
            abnormalBillDisposeService.saveOrUpdate(abnormalBillDisposes.get(0));
            return new ResultVo(ResultCode.SUCCESS,"BPM异常修改成功");
        }catch(Exception e){
            return new ResultVo(ResultCode.FAILED,"BPM异常修改失败:" + e.getMessage());
        }
    }

    /**
     * bpm闭环关闭
     * @param abnormalBill
     * @param abnormalBillDispose
     * @return
     */
    @PostMapping("/bpmCorrectivePreventive")
    public  ResultVo bpmCorrectivePreventive(String secretKey,String isClose,AbnormalBill abnormalBill,AbnormalBillDispose abnormalBillDispose){
        com.foreverwin.modular.core.util.CommonMethods.setValue("SITE","1000");
        String site = com.foreverwin.modular.core.util.CommonMethods.getSite();
        if (!yunBpmConfig.secretKey.equals(secretKey)){
            return new ResultVo(ResultCode.FAILED,"秘钥错误");
        }else if (StringUtils.isBlank(abnormalBill.getAbnormalNo())){
            return new ResultVo(ResultCode.FAILED,"异常单号为空");
        }
        //保存操作日志
        StringUtil.saveOperLog("BPM质量系统闭环关闭回调接口，单号："  + abnormalBill.getAbnormalNo(),"bpmCorrectivePreventive",
                isClose + abnormalBill.toString() + abnormalBillDispose.toString(),null,null);
        try{
        AbnormalBill aBill = new AbnormalBill();
        aBill.setAbnormalNo(abnormalBill.getAbnormalNo());
        LocalDateTime now = LocalDateTime.now();
        AbnormalBillDispose dispose = new AbnormalBillDispose();
        dispose.setHandle(HandleEnum.ABNORMAL_BILL_DISPOSE.getHandle("1000",abnormalBill.getAbnormalNo()));
        List<AbnormalBillDispose> abnormalBillDisposes = abnormalBillDisposeService.selectList(dispose);
        if ("true".equals(isClose)){
            try {
                com.foreverwin.modular.core.util.CommonMethods.setUser(abnormalBillDispose.getClosedUser());
                AbnormalBill abnormalBillOld = abnormalBillService.getById(HandleEnum.ABNORMAL_BILL.getHandle(site,abnormalBill.getAbnormalNo()));
                abnormalBillOld.setStatus(Constants.SHUT_DOWN);
                String local = LocaleContextHolder.getLocale().getLanguage();
                AbnormalBillDisposeDto abnormalBillDisposeDto = abnormalBillDisposeMapper.findAllByAbnormalNo(abnormalBill, local);
                abnormalPlanService.saveAbnormalPlan(abnormalBillDisposeDto,now,"false");
                abnormalBillService.saveOrUpdate(abnormalBillOld);
                abnormalBillDisposes.get(0).setClosedUser(abnormalBillDispose.getClosedUser());
                abnormalBillDisposes.get(0).setClosedDateTime(now);
                abnormalBillDisposeService.saveOrUpdate(abnormalBillDisposes.get(0));
                return new ResultVo(ResultCode.SUCCESS,"bpm闭环关闭成功");
            } catch (Exception e) {
                return new ResultVo(ResultCode.FAILED,"bpm闭环关闭失败:" + e.getMessage());
            }
        }
        abnormalBillDisposes.get(0).setAbnormalReason(abnormalBillDispose.getAbnormalReason());
        abnormalBillDisposes.get(0).setBeforeMeasure(abnormalBillDispose.getBeforeMeasure());
        abnormalBillDisposeService.saveOrUpdate(abnormalBillDisposes.get(0));
        }catch (Exception e){
            return new ResultVo(ResultCode.FAILED,"bpm纠正预防失败:" + e.getMessage());
        }
        return new ResultVo(ResultCode.SUCCESS,"bpm纠正预防成功");
    }

    /**
     * 质量异常响应检索
     * @param abnormalBill
     * @return
     */
    @GetMapping("/findAllByAbnormalNo")
    public R findAllByAbnormalNo(AbnormalBill abnormalBill){
        try{
            return R.ok(abnormalBillService.findAllByAbnormalNo(abnormalBill));
        }catch (Exception e){
            return R.failed("检索失败:"+e.getMessage());
        }
    }


    /**
     * 设备异常响应检索
     */
    @GetMapping("/findAllByAbnormalNoDevice")
    public R findAllByAbnormalNoDevice(AbnormalBill abnormalBill){
        try{
            return R.ok(abnormalBillService.findAllByAbnormalNoDevice(abnormalBill));
        }catch (Exception e){
            return R.failed("设备异常响应检索失败:"+e.getMessage());
        }
    }

    /**
     * 响应处理
     * @param abnormalBill
     * @return
     */
    @GetMapping("/resProcessing")
    public R resProcessing(AbnormalBill abnormalBill){
        try{
            abnormalBillService.resProcessing(abnormalBill);
            return R.ok();
        }catch(Exception e){
            return R.failed("响应处理失败:"+e.getMessage());
        }
    }

    /**
     * 方案回退
     * @param abnormalBill
     * @return
     */
    @GetMapping("/rollbackProcessing")
    public R rollbackProcessing(AbnormalBill abnormalBill,AbnormalBillDispose abnormalBillDispose){
        try{
            abnormalBillService.rollbackProcessing(abnormalBill,abnormalBillDispose);
            return R.ok();
        }catch(Exception e){
            return R.failed("方案回退失败:"+e.getMessage());
        }
    }

    /**
     * 方案确认
     * @param abnormalBill
     * @param abnormalBillDispose
     * @return
     */
    @GetMapping("/programConfirm")
    public R programConfirm(AbnormalBill abnormalBill,AbnormalBillDispose abnormalBillDispose){
        try{
            abnormalBillService.programConfirm(abnormalBill,abnormalBillDispose);
            return R.ok();
        }catch (Exception e){
            return R.failed("方案确认失败:"+e.getMessage());
        }
    }

    /**
     * 判责提交
     * @param abnormalBill
     * @param abnormalBillDispose
     * @return
     */
    @GetMapping("/dutyConfirm")
    public R dutyConfirm(AbnormalBill abnormalBill,AbnormalBillDispose abnormalBillDispose,
                         @RequestParam List<String> dutyCauseType,
                         @RequestParam List<String> dutyType){
        try{
            abnormalBillService.dutyConfirm(abnormalBill,abnormalBillDispose,dutyCauseType,dutyType);
            return R.ok();
        }catch(Exception e){
            return R.failed("判责提交失败:"+e.getMessage());
        }
    }

    /**
     * 闭环关闭
     * @param abnormalBill
     * @param abnormalBillDispose
     * @return
     */
    @GetMapping("/anomalyShutDown")
    public  R anomalyShutDown(AbnormalBill abnormalBill,AbnormalBillDispose abnormalBillDispose){
        try{
            abnormalBillService.anomalyShutDown(abnormalBill,abnormalBillDispose);
            return R.ok();
        }catch (Exception e){
            return R.failed("闭环关闭失败:"+e.getMessage());
        }

    }

    @GetMapping("/findResrceAndWorkCenterBySfc")
    public R findResrceAndWorkCenterBySfc(SfcDispatchDto sfcDispatch){
        try{
            return R.ok(sfcDispatchCommonService.findResrceAndWorkCenterBySfc(sfcDispatch));
        }catch (Exception e){
            return R.failed("根据sfc查找设备和工作中心失败:"+e.getMessage());
        }
    }

    @GetMapping("/show")
    public R showFile(HttpServletRequest request, HttpServletResponse response) throws Exception {
        try{
            return R.ok(fileUploadedService.showFile(request,response));
        }catch (Exception e){
            return R.failed("显示图片失败:"+e.getMessage());
        }
    }

    @PostMapping("/delete")
    public R delete(String ftpPath,String fileName){
        try{
            fileUploadedService.deleteFile(ftpPath,fileName);
            return R.ok();
        }catch (Exception e){
            return R.failed("图片删除失败:"+e.getMessage());
        }
    }

    @GetMapping("/findUploadPictureByAbnormalBo")
    public R findUploadPictureByAbnormalBo(AbnormalBill abnormalBill){
        return R.ok(abnormalBillService.findUploadPictureByAbnormalBo(abnormalBill));
    }


    @GetMapping("/getNextNumber")
    public R getNextNumber(String type){
        return R.ok(abnormalBillService.getNextNumber(type));
    }

    @GetMapping("/clear")
    public R clear(String type){
        return R.ok(abnormalBillService.getNextNumber(type));
    }


    @GetMapping("/anomalyReveseRepair")
    public R anomalyReveseRepair(AbnormalBill abnormalBill, AbnormalBillDispose abnormalBillDispose){
        return R.ok(abnormalBillService.anomalyReveseRepair(abnormalBill,abnormalBillDispose));
    }

    @GetMapping("/findNcCodeDescriptionByNcCode")
    public R findNcCodeDescriptionByNcCode(String ncCode){
        return R.ok(ncCodeService.findNcCodeDescriptionByNcCode(ncCode));
    }

}