package com.foreverwin.mesnac.anomaly.controller;

import com.foreverwin.mesnac.anomaly.mapper.EquipmentAssetDetailsMapper;
import com.foreverwin.mesnac.common.util.StringUtil;
import com.foreverwin.modular.core.util.R;
import com.foreverwin.modular.core.util.FrontPage;
import com.foreverwin.modular.core.util.CommonMethods;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import com.foreverwin.mesnac.anomaly.service.EquipmentAssetDetailsService;
import com.foreverwin.mesnac.anomaly.model.EquipmentAssetDetails;

import java.time.LocalDateTime;
import java.util.List;

/**
 *
 * @author YinQ
 * @since 2022-09-16
 */
@RestController
@RequestMapping("/Z-EQUIPMENT-ASSET-DETAILS")
public class EquipmentAssetDetailsController {

    @Autowired
    public EquipmentAssetDetailsService equipmentAssetDetailsService;

    @Autowired
    public EquipmentAssetDetailsMapper equipmentAssetDetailsMapper;

    /**
    * 根据id查询
    *
    * @param id 主键
    * @return
    */
    @ResponseBody
    @GetMapping("/{id:.+}")
    public R getEquipmentAssetDetailsById(@PathVariable String id) {
        return R.ok( equipmentAssetDetailsService.getById(id));
    }

    /**
     * 查询所有数据
     *
     * @return
     */
    @ResponseBody
    @GetMapping("")
    public R getEquipmentAssetDetailsList(EquipmentAssetDetails equipmentAssetDetails){
        List<EquipmentAssetDetails> result;
        QueryWrapper<EquipmentAssetDetails> queryWrapper = new QueryWrapper<>();
        queryWrapper
                .like(!StringUtil.isBlank(equipmentAssetDetails.getBelongWorkshop()),"BELONG_WORKSHOP",equipmentAssetDetails.getBelongWorkshop())
                .like(!StringUtil.isBlank(equipmentAssetDetails.getAssetClasses()),"ASSET_CLASSES",equipmentAssetDetails.getAssetClasses())
                .like(!StringUtil.isBlank(equipmentAssetDetails.getAssetNumbers()),"ASSET_NUMBERS",equipmentAssetDetails.getAssetNumbers())
                .like(!StringUtil.isBlank(equipmentAssetDetails.getAssetDescription()),"ASSET_DESCRIPTION",equipmentAssetDetails.getAssetDescription())
                .like(!StringUtil.isBlank(equipmentAssetDetails.getOperators()),"OPERATORS",equipmentAssetDetails.getOperators())
                .like(!StringUtil.isBlank(equipmentAssetDetails.getSapWorkCenterCode()),"SAP_WORK_CENTER_CODE",equipmentAssetDetails.getSapWorkCenterCode())
                .like(!StringUtil.isBlank(equipmentAssetDetails.getEquipmentState()),"EQUIPMENT_STATE",equipmentAssetDetails.getEquipmentState())
                .orderByAsc("BELONG_WORKSHOP","TEAM","ASSET_CLASSES","ASSET_DESCRIPTION");
        result = equipmentAssetDetailsService.list(queryWrapper);
        return R.ok(result);
    }

    /**
     * 分页查询数据
     *
     * @param frontPage  分页信息
     * @return
     */
    @ResponseBody
    @GetMapping("/page")
    public R page(FrontPage<EquipmentAssetDetails> frontPage, EquipmentAssetDetails equipmentAssetDetails){
        IPage result;
        QueryWrapper<EquipmentAssetDetails> queryWrapper = new QueryWrapper<>();
        queryWrapper
                .like(!StringUtil.isBlank(equipmentAssetDetails.getBelongWorkshop()),"BELONG_WORKSHOP",equipmentAssetDetails.getBelongWorkshop())
                .like(!StringUtil.isBlank(equipmentAssetDetails.getAssetClasses()),"ASSET_CLASSES",equipmentAssetDetails.getAssetClasses())
                .like(!StringUtil.isBlank(equipmentAssetDetails.getAssetNumbers()),"ASSET_NUMBERS",equipmentAssetDetails.getAssetNumbers())
                .like(!StringUtil.isBlank(equipmentAssetDetails.getAssetDescription()),"ASSET_DESCRIPTION",equipmentAssetDetails.getAssetDescription())
                .like(!StringUtil.isBlank(equipmentAssetDetails.getOperators()),"OPERATORS",equipmentAssetDetails.getOperators())
                .like(!StringUtil.isBlank(equipmentAssetDetails.getSapWorkCenterCode()),"SAP_WORK_CENTER_CODE",equipmentAssetDetails.getSapWorkCenterCode())
                .like(!StringUtil.isBlank(equipmentAssetDetails.getEquipmentState()),"EQUIPMENT_STATE",equipmentAssetDetails.getEquipmentState())

                .orderByAsc("BELONG_WORKSHOP","TEAM","ASSET_CLASSES","ASSET_DESCRIPTION");
        result = equipmentAssetDetailsService.page(frontPage.getPagePlus(), queryWrapper);
        for (int i = 0; i < result.getRecords().size(); i++) {
            EquipmentAssetDetails assetDetails =  (EquipmentAssetDetails) result.getRecords().get(i);
            assetDetails.setSEQ((frontPage.getPage() - 1) * frontPage.getRows() + i + 1);
            result.getRecords().set(i,assetDetails);
        }
        return R.ok(result);
    }

    /**
     * 新增
     * @param equipmentAssetDetails  传递的实体
     * @return  null 失败  实体成功
     */
    @PostMapping
    public R save(@RequestBody EquipmentAssetDetails equipmentAssetDetails) {
        String handle = StringUtil.createQUID();
        String user = CommonMethods.getUser();
        String site = CommonMethods.getSite();
        EquipmentAssetDetails details = new EquipmentAssetDetails();
        details.setAssetNumbers(equipmentAssetDetails.getAssetNumbers());
        List<EquipmentAssetDetails> selectList = equipmentAssetDetailsService.selectList(details);
        if (selectList.size() != 0){
            return R.failed("资产编号:"+ equipmentAssetDetails.getAssetNumbers() + "不允许重复");
        }

        LocalDateTime now = LocalDateTime.now();
        equipmentAssetDetails.setHandle(handle);
        equipmentAssetDetails.setSite(site);
        equipmentAssetDetails.setCreatedTime(now);
        equipmentAssetDetails.setCreatedBy(user);
        equipmentAssetDetails.setUpdatedBy(user);
        equipmentAssetDetails.setUpdatedTime(now);
        return R.ok(equipmentAssetDetailsService.save(equipmentAssetDetails));
    }

    /**
     * 修改
     * @param equipmentAssetDetails  传递的实体
     * @return  null 失败  实体成功
     */
    @PostMapping("/PUT")
    public R updateById(@RequestBody EquipmentAssetDetails equipmentAssetDetails) {
        String user = CommonMethods.getUser();
        LocalDateTime now = LocalDateTime.now();
        EquipmentAssetDetails details = new EquipmentAssetDetails();
        details.setAssetNumbers(equipmentAssetDetails.getAssetNumbers());
        List<EquipmentAssetDetails> selectList = equipmentAssetDetailsService.selectList(details);
        if (selectList.size() != 0 && !selectList.get(0).getHandle().equals(equipmentAssetDetails.getHandle())){
            return R.failed("资产编号:"+ equipmentAssetDetails.getAssetNumbers() + "不允许重复");
        }
        equipmentAssetDetails.setUpdatedBy(user);
        equipmentAssetDetails.setUpdatedTime(now);
        return R.ok(equipmentAssetDetailsService.updateById(equipmentAssetDetails));
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     * @return 0 失败  1 成功
     */
    @ResponseBody
    @GetMapping("/delete/{id}")
    public R removeById(@PathVariable("id") String id){
        return R.ok(equipmentAssetDetailsService.removeById(id));
    }

    /**
     * 批量删除对象
     * @param ids 实体集合ID
     * @return  0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.POST, value = "/delete-batch")
    public R removeByIds(List<String> ids){
        return R.ok(equipmentAssetDetailsService.removeByIds(ids));
    }
}