package com.foreverwin.mesnac.anomaly.controller;

import com.foreverwin.mesnac.anomaly.mapper.ImprovementProposalsMapper;
import com.foreverwin.mesnac.anomaly.service.FileUploadedService;
import com.foreverwin.mesnac.anomaly.utils.DateReportUtils;
import com.foreverwin.mesnac.common.util.StringUtil;
import com.foreverwin.modular.core.util.R;
import com.foreverwin.modular.core.util.FrontPage;
import com.foreverwin.modular.core.util.CommonMethods;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.apache.commons.fileupload.FileItem;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import com.foreverwin.mesnac.anomaly.service.ImprovementProposalsService;
import com.foreverwin.mesnac.anomaly.model.ImprovementProposals;

import javax.servlet.http.HttpServletRequest;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.List;
import java.util.Map;

/**
 * @author YinQ
 * @since 2022-09-19
 */
@RestController
@RequestMapping("/Z-IMPROVEMENT-PROPOSALS")
public class ImprovementProposalsController {

    @Autowired
    public ImprovementProposalsService improvementProposalsService;

    @Autowired
    FileUploadedService fileUploadedService;

    @Autowired
    private ImprovementProposalsMapper improvementProposalsMapper;

    /**
     * 根据id查询
     *
     * @param id 主键
     * @return
     */
    @ResponseBody
    @GetMapping("/{id:.+}")
    public R getImprovementProposalsById(@PathVariable String id) {
        return R.ok(improvementProposalsService.getById(id));
    }

    /**
     * 查询所有数据
     *
     * @return
     */
    @ResponseBody
    @GetMapping("")
    public R getImprovementProposalsList(@RequestParam(required = false) Map<String, String> paramMap) {
        List<ImprovementProposals> result;
        FrontPage<ImprovementProposals> frontPage = new FrontPage<>();
        frontPage.setPage(Integer.valueOf(paramMap.get("page")));
        frontPage.setRows(Integer.valueOf(paramMap.get("rows")));
        QueryWrapper<ImprovementProposals> queryWrapper = new QueryWrapper<>();
        queryWrapper
                .like(!StringUtil.isBlank(paramMap.get("uploadCenter")), "UPLOAD_CENTER", paramMap.get("uploadCenter"))
                .like(!StringUtil.isBlank(paramMap.get("uploadUser")), "UPLOAD_USER", paramMap.get("uploadUser"))
                .like(!StringUtil.isBlank(paramMap.get("sequenceCode")), "SEQUENCE_CODE", paramMap.get("sequenceCode"))
                .ge(!StringUtil.isBlank(paramMap.get("startDate")), "CREATED_TIME", DateReportUtils.stringToDate(paramMap.get("startDate") + " 00:00:00", "yyyy-MM-dd HH:mm:ss"))
                .le(!StringUtil.isBlank(paramMap.get("endDate")), "CREATED_TIME", DateReportUtils.stringToDate(paramMap.get("endDate") + " 23:59:59", "yyyy-MM-dd HH:mm:ss"))
                .isNull("IS_DELETE")
        ;
        result = improvementProposalsService.list(queryWrapper);
        return R.ok(result);
    }

    /**
     * 分页查询数据
     *
     * @param paramMap
     * @return
     */
    @ResponseBody
    @GetMapping("/page")
    public R page(@RequestParam(required = false) Map<String, String> paramMap) {
        IPage result;
        FrontPage<ImprovementProposals> frontPage = new FrontPage<>();
        frontPage.setPage(Integer.valueOf(paramMap.get("page")));
        frontPage.setRows(Integer.valueOf(paramMap.get("rows")));
        QueryWrapper<ImprovementProposals> queryWrapper = new QueryWrapper<>();
        queryWrapper
                .like(!StringUtil.isBlank(paramMap.get("uploadCenter")), "UPLOAD_CENTER", paramMap.get("uploadCenter"))
                .like(!StringUtil.isBlank(paramMap.get("uploadUser")), "UPLOAD_USER", paramMap.get("uploadUser"))
                .like(!StringUtil.isBlank(paramMap.get("sequenceCode")), "SEQUENCE_CODE", paramMap.get("sequenceCode"))
                .ge(!StringUtil.isBlank(paramMap.get("startDate")), "CREATED_TIME", DateReportUtils.stringToDate(paramMap.get("startDate") + " 00:00:00", "yyyy-MM-dd HH:mm:ss"))
                .le(!StringUtil.isBlank(paramMap.get("endDate")), "CREATED_TIME", DateReportUtils.stringToDate(paramMap.get("endDate") + " 23:59:59", "yyyy-MM-dd HH:mm:ss"))
                .isNull("IS_DELETE")
        ;

        result = improvementProposalsService.page(frontPage.getPagePlus(), queryWrapper);
        return R.ok(result);
    }

    /**
     * 新增
     *
     * @param improvementProposals 传递的实体
     * @return null 失败  实体成功
     */
    @PostMapping
    public R save(@RequestBody ImprovementProposals improvementProposals) {
        improvementProposals = improvementProposalsService.insertData(improvementProposals);
        return R.ok(improvementProposalsService.save(improvementProposals));
    }

    /**
     * 修改
     *
     * @param improvementProposals 传递的实体
     * @return null 失败  实体成功
     */
    @PostMapping("/PUT")
    public R updateById(@RequestBody ImprovementProposals improvementProposals) {
        LocalDateTime now = LocalDateTime.now();
        String user = CommonMethods.getUser();
        improvementProposals.setUpdatedBy(user);
        improvementProposals.setUpdatedTime(now);

        return R.ok(improvementProposalsService.updateById(improvementProposals));
    }


    /**
     * 根据id删除对象
     *
     * @param id 实体ID
     * @return 0 失败  1 成功
     */
    @ResponseBody
    @GetMapping("/delete/{id}")
    public R removeById(@PathVariable("id") String id) {
        ImprovementProposals improvementProposals = new ImprovementProposals();
        improvementProposals.setHandle(id);
        improvementProposals.setIsDelete("true");
        int res = improvementProposalsMapper.updateById(improvementProposals);
//        boolean res = improvementProposalsService.removeById(id);
        return R.ok(res);
    }

    /**
     * 批量删除对象
     *
     * @param ids 实体集合ID
     * @return 0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.POST, value = "/delete-batch")
    public R removeByIds(List<String> ids) {
        return R.ok(improvementProposalsService.removeByIds(ids));
    }
}