package com.foreverwin.mesnac.anomaly.controller;

import com.foreverwin.mesnac.anomaly.utils.DateReportUtils;
import com.foreverwin.mesnac.common.util.StringUtil;
import com.foreverwin.modular.core.util.R;
import com.foreverwin.modular.core.util.FrontPage;
import com.foreverwin.modular.core.util.CommonMethods;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import com.foreverwin.mesnac.anomaly.service.OutStoreService;
import com.foreverwin.mesnac.anomaly.model.OutStore;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;

/**
 *
 * @author YinQ
 * @since 2022-09-15
 */
@RestController
@RequestMapping("/Z-OUT-STORE")
public class OutStoreController {

    @Autowired
    public OutStoreService outStoreService;

    /**
    * 根据id查询
    *
    * @param id 主键
    * @return
    */
    @ResponseBody
    @GetMapping("/{id:.+}")
    public R getOutStoreById(@PathVariable String id) {
        return R.ok( outStoreService.getById(id));
    }

    /**
     * 查询所有数据
     *
     * @return
     */
    @ResponseBody
    @GetMapping("")
    public R getOutStoreList(@RequestParam(required = false) OutStore outStore,String startDate,String endDate){
        List<OutStore> result;
        QueryWrapper<OutStore> queryWrapper = new QueryWrapper<>();
        queryWrapper
                .ge(!StringUtil.isBlank(startDate),"IMPORT_DATE_TIME", DateReportUtils.stringToDate(startDate + " 00:00:00","yyyy-MM-dd HH:mm:ss"))
                .le(!StringUtil.isBlank(endDate),"IMPORT_DATE_TIME",DateReportUtils.stringToDate(endDate + " 23:59:59","yyyy-MM-dd HH:mm:ss"))
;
        result = outStoreService.list(queryWrapper);
        return R.ok(result);
    }

    /**
     * 分页查询数据
     *
     * @param frontPage  分页信息
     * @return
     */
    @ResponseBody
    @GetMapping("/page")
    public R page(FrontPage<OutStore> frontPage, OutStore outStore,String startDate,String endDate){
        IPage result;
        QueryWrapper<OutStore> queryWrapper = new QueryWrapper<>();
        queryWrapper
                .ge(!StringUtil.isBlank(startDate),"IMPORT_DATE_TIME", DateReportUtils.stringToDate(startDate + " 00:00:00","yyyy-MM-dd HH:mm:ss"))
                .le(!StringUtil.isBlank(endDate),"IMPORT_DATE_TIME",DateReportUtils.stringToDate(endDate + " 23:59:59","yyyy-MM-dd HH:mm:ss"))
        ;
        result = outStoreService.page(frontPage.getPagePlus(), queryWrapper);
        return R.ok(result);
    }

    /**
     * 新增
     * @param outStore  传递的实体
     * @return  null 失败  实体成功
     */
    @PostMapping
    public R save(@RequestBody OutStore outStore) {
        String handle = StringUtil.createQUID();
        String user = CommonMethods.getUser();
        LocalDateTime now = LocalDateTime.now();
        outStore.setHandle(handle);
        if (StringUtil.isBlank(outStore.getImportUser())){
            outStore.setImportUser(user);
        }
        outStore.setImportDateTime(now);

        return R.ok(outStoreService.save(outStore));
    }

    /**
     * 修改
     * @param outStore  传递的实体
     * @return  null 失败  实体成功
     */
    @PostMapping("/PUT")
    public R updateById(@RequestBody OutStore outStore) {
        return R.ok(outStoreService.updateById(outStore));
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     * @return 0 失败  1 成功
     */
    @ResponseBody
    @GetMapping("/delete/{id}")
    public R removeById(@PathVariable("id") String id){
        return R.ok(outStoreService.removeById(id));
    }

    /**
     * 批量删除对象
     * @param ids 实体集合ID
     * @return  0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.POST, value = "/delete-batch")
    public R removeByIds(List<String> ids){
        return R.ok(outStoreService.removeByIds(ids));
    }
}