package com.foreverwin.mesnac.anomaly.controller;

import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONUtil;
import com.foreverwin.mesnac.anomaly.utils.DateReportUtils;
import com.foreverwin.mesnac.anomaly.utils.OAuthTokenInfo;
import com.foreverwin.mesnac.anomaly.utils.yunBpmConfig;
import com.foreverwin.mesnac.common.util.DataUtils;
import com.foreverwin.mesnac.common.util.DateUtil;
import com.foreverwin.mesnac.common.util.HttpUtils;
import com.foreverwin.mesnac.common.util.StringUtil;
import com.foreverwin.mesnac.meapi.util.StringUtils;
import com.foreverwin.modular.core.util.R;
import com.foreverwin.modular.core.util.FrontPage;
import com.foreverwin.modular.core.util.CommonMethods;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import com.foreverwin.mesnac.anomaly.service.TemporaryFixService;
import com.foreverwin.mesnac.anomaly.model.TemporaryFix;
import org.springframework.web.multipart.MultipartFile;

import javax.websocket.server.PathParam;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Handler;

/**
 * @author Y.Q
 * @since 2022-09-08
 */
@RestController
@RequestMapping("/Z-TEMPORARY-FIX")
public class TemporaryFixController {

    @Autowired
    public TemporaryFixService temporaryFixService;

    /**
     * 根据id查询
     *
     * @param id 主键
     * @return
     */
    @ResponseBody
    @GetMapping("/{id:.+}")
    public R getTemporaryFixById(@PathVariable String id) {
        return R.ok(temporaryFixService.getById(id));
    }

    /**
     * 查询所有数据
     *
     * @return
     */
    @ResponseBody
    @GetMapping("")
    public R getTemporaryFixList(@RequestParam(required = false) Map paramMap, String startDate, String endDate, String createdTimeNew) {
        List<TemporaryFix> result;
        QueryWrapper<TemporaryFix> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(!StringUtil.isBlank(String.valueOf(paramMap.get("entrustUnit"))), "ENTRUST_UNIT", String.valueOf(paramMap.get("entrustUnit")))
                .like(!StringUtil.isBlank(String.valueOf(paramMap.get("dutyPerson"))), "DUTY_PERSON", String.valueOf(paramMap.get("dutyPerson")))
                .like(!StringUtil.isBlank(String.valueOf(paramMap.get("projectNumber"))), "PROJECT_NUMBER", String.valueOf(paramMap.get("projectNumber")))
                .like(!StringUtil.isBlank(String.valueOf(paramMap.get("qualityReviewNumber"))), "QUALITY_REVIEW_NUMBER", String.valueOf(paramMap.get("qualityReviewNumber")))
                .like(!StringUtil.isBlank(String.valueOf(paramMap.get("workContactNumber"))), "WORK_CONTACT_NUMBER", String.valueOf(paramMap.get("workContactNumber")))
                .like(!StringUtil.isBlank(String.valueOf(paramMap.get("materialCode"))), "MATERIAL_CODE", String.valueOf(paramMap.get("materialCode")))
                .like(!StringUtil.isBlank(String.valueOf(paramMap.get("materialName"))), "MATERIAL_NAME", String.valueOf(paramMap.get("materialName")))
                .like(!StringUtil.isBlank(String.valueOf(paramMap.get("mapNo"))), "MAP_NO", String.valueOf(paramMap.get("mapNo")))
                .like(!StringUtil.isBlank(String.valueOf(paramMap.get("operator"))), "OPERATOR", String.valueOf(paramMap.get("operator")))
                .like(!StringUtil.isBlank(String.valueOf(paramMap.get("useDevice"))), "USE_DEVICE", String.valueOf(paramMap.get("useDevice")))
                .like(!StringUtil.isBlank(String.valueOf(paramMap.get("entrustDate"))), "ENTRUST_DATE", String.valueOf(paramMap.get("entrustDate")))
                .like(!StringUtil.isBlank(String.valueOf(paramMap.get("classify"))), "CLASSIFY", String.valueOf(paramMap.get("classify")))
                .eq(!StringUtil.isBlank(String.valueOf(paramMap.get("uploadCenter"))), "UPLOAD_CENTER", String.valueOf(paramMap.get("uploadCenter")))
                .ge(!StringUtil.isBlank(startDate), "CREATED_TIME", DateReportUtils.stringToDate(startDate + " 00:00:00", "yyyy-MM-dd HH:mm:ss"))
                .le(!StringUtil.isBlank(endDate), "CREATED_TIME", DateReportUtils.stringToDate(endDate + " 23:59:59", "yyyy-MM-dd HH:mm:ss"))
                .ge(!StringUtil.isBlank(createdTimeNew), "CREATED_TIME", DateReportUtils.stringToDate(createdTimeNew + " 00:00:00", "yyyy-MM-dd HH:mm:ss"))
                .le(!StringUtil.isBlank(createdTimeNew), "CREATED_TIME", DateReportUtils.stringToDate(createdTimeNew + " 23:59:59", "yyyy-MM-dd HH:mm:ss"))
                .orderByDesc("CREATED_TIME");

        result = temporaryFixService.list(queryWrapper);
        return R.ok(result);
    }

    /**
     * 分页查询数据
     *
     * @param frontPage 分页信息
     * @return
     */
    @GetMapping("/page")
    public R page(FrontPage<TemporaryFix> frontPage, TemporaryFix temporaryFix, String startDate, String endDate, String createdTimeNew) {
        IPage result;
        QueryWrapper<TemporaryFix> queryWrapper = new QueryWrapper<>();

        queryWrapper.eq(!StringUtil.isBlank(temporaryFix.getEntrustUnit()), "ENTRUST_UNIT", temporaryFix.getEntrustUnit())
                .like(!StringUtil.isBlank(temporaryFix.getDutyPerson()), "DUTY_PERSON", temporaryFix.getDutyPerson())
                .like(!StringUtil.isBlank(temporaryFix.getProjectNumber()), "PROJECT_NUMBER", temporaryFix.getProjectNumber())
                .like(!StringUtil.isBlank(temporaryFix.getQualityReviewNumber()), "QUALITY_REVIEW_NUMBER", temporaryFix.getQualityReviewNumber())
                .like(!StringUtil.isBlank(temporaryFix.getWorkContactNumber()), "WORK_CONTACT_NUMBER", temporaryFix.getWorkContactNumber())
                .like(!StringUtil.isBlank(temporaryFix.getMaterialCode()), "MATERIAL_CODE", temporaryFix.getMaterialCode())
                .like(!StringUtil.isBlank(temporaryFix.getMaterialName()), "MATERIAL_NAME", temporaryFix.getMaterialName())
                .like(!StringUtil.isBlank(temporaryFix.getMapNo()), "MAP_NO", temporaryFix.getMapNo())
                .like(!StringUtil.isBlank(temporaryFix.getOperator()), "OPERATOR", temporaryFix.getOperator())
                .like(!StringUtil.isBlank(temporaryFix.getUseDevice()), "USE_DEVICE", temporaryFix.getUseDevice())
                .like(!StringUtil.isBlank(temporaryFix.getEntrustDate()), "ENTRUST_DATE", temporaryFix.getEntrustDate())
                .like(!StringUtil.isBlank(temporaryFix.getClassify()), "CLASSIFY", temporaryFix.getClassify())
                .eq(!StringUtil.isBlank(temporaryFix.getUploadCenter()), "UPLOAD_CENTER", temporaryFix.getUploadCenter())
                .ge(!StringUtil.isBlank(startDate), "CREATED_TIME", DateReportUtils.stringToDate(startDate + " 00:00:00", "yyyy-MM-dd HH:mm:ss"))
                .le(!StringUtil.isBlank(endDate), "CREATED_TIME", DateReportUtils.stringToDate(endDate + " 23:59:59", "yyyy-MM-dd HH:mm:ss"))
                .ge(!StringUtil.isBlank(createdTimeNew), "CREATED_TIME", DateReportUtils.stringToDate(createdTimeNew + " 00:00:00", "yyyy-MM-dd HH:mm:ss"))
                .le(!StringUtil.isBlank(createdTimeNew), "CREATED_TIME", DateReportUtils.stringToDate(createdTimeNew + " 23:59:59", "yyyy-MM-dd HH:mm:ss"))
                .orderByDesc("CREATED_TIME");
        result = temporaryFixService.page(frontPage.getPagePlus(), queryWrapper);
        return R.ok(result);
    }

    /**
     * 新增
     *
     * @param temporaryFix 传递的实体
     * @return null 失败  实体成功
     */
    @PostMapping
    public R save(@RequestBody TemporaryFix temporaryFix) {
        String handle = StringUtil.createQUID();
        String user = CommonMethods.getUser();
        String site = CommonMethods.getSite();
        LocalDateTime now = LocalDateTime.now();
        temporaryFix.setHandle(handle);
        temporaryFix.setSite(site);
        temporaryFix.setCreatedTime(now);
        temporaryFix.setCreatedBy(user);
        temporaryFix.setUpdatedBy(user);
        temporaryFix.setUpdatedTime(now);

        if (temporaryFix.getAmount() != null && temporaryFix.getSingleWorkHours() != null && temporaryFix.getAmount() != null && temporaryFix.getSingleWorkHours() != null) {
            temporaryFix.setTotalWorkHours(String.valueOf(Float.valueOf(temporaryFix.getAmount()) * Float.valueOf(temporaryFix.getSingleWorkHours())));
        }
        boolean result = temporaryFixService.save(temporaryFix);
        return R.ok(result);
    }


    @PostMapping("/PUT")
    public R updateById(@RequestBody TemporaryFix temporaryFix) {
        LocalDateTime now = LocalDateTime.now();
        String user = CommonMethods.getUser();
        temporaryFix.setUpdatedBy(user);
        temporaryFix.setUpdatedTime(now);
        if (temporaryFix.getAmount() != null && temporaryFix.getSingleWorkHours() != null && temporaryFix.getAmount() != "" && temporaryFix.getSingleWorkHours() != "") {
            temporaryFix.setTotalWorkHours(String.valueOf(Float.valueOf(temporaryFix.getAmount()) * Float.valueOf(temporaryFix.getSingleWorkHours())));
        }

        boolean result = temporaryFixService.updateById(temporaryFix);
        return R.ok(result);
    }

    /**
     * 测试
     *
     * @param paramMap
     * @return
     */
    @PostMapping("/getURL")
    public R getURL(@RequestBody Map paramMap) {
        String respBody = null;
        try {
            if (paramMap.containsKey("url1")){
                String params = JSONUtil.toJsonStr(paramMap);
                respBody = HttpUtil.post(String.valueOf(paramMap.get("url1")), params, 1000 * 10);
                respBody = respBody.concat("[测试]");
            }else {
                String requestUrl = yunBpmConfig.getApiBaseUrl() + "/api/login/Authentication/get_code";
                Map<String, Object> params = new HashMap<>();
                params.put("username", yunBpmConfig.userName);
                params.put("password", yunBpmConfig.passWord);
                params.put("url", yunBpmConfig.getYunBpmUrl() + "/api/login?redirect_uri="
                        + yunBpmConfig.getYunBpmUrl() +
                        "/api/oauth/authorize?client_id=api&response_type=code&scope=read&redirect_uri="
                        + yunBpmConfig.getYunBpmUrl() + "/oauth");
                String jsonStr = JSONUtil.toJsonStr(params);
                respBody = HttpUtil.post(requestUrl, jsonStr,1000 * 10);
            }
        } catch (Exception e) {
            return R.failed(e.toString());
        }
        return R.ok(respBody);
    }

    /**
     * 根据id删除对象
     *
     * @param id 实体ID
     * @return 0 失败  1 成功
     */
    @ResponseBody
    @GetMapping("/delete/{id}")
    public R removeById(@PathVariable("id") String id) {
        return R.ok(temporaryFixService.removeById(id));
    }

    /**
     * 批量删除对象
     *
     * @param ids 实体集合ID
     * @return 0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.POST, value = "/delete-batch")
    public R removeByIds(List<String> ids) {
        return R.ok(temporaryFixService.removeByIds(ids));
    }
}