package com.foreverwin.mesnac.anomaly.controller;

import com.foreverwin.mesnac.anomaly.model.EquipmentAssetDetails;
import com.foreverwin.mesnac.common.util.StringUtil;
import com.foreverwin.modular.core.util.R;
import com.foreverwin.modular.core.util.FrontPage;
import com.foreverwin.modular.core.util.CommonMethods;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import com.foreverwin.mesnac.anomaly.service.WorkCentreDetailsService;
import com.foreverwin.mesnac.anomaly.model.WorkCentreDetails;

import java.time.LocalDateTime;
import java.util.List;

/**
 * @author YinQ
 * @since 2022-11-29
 */
@RestController
@RequestMapping("/Z-WORK-CENTRE-DETAILS")
public class WorkCentreDetailsController {

    @Autowired
    public WorkCentreDetailsService workCentreDetailsService;

    /**
     * 根据id查询
     *
     * @param id 主键
     * @return
     */
    @ResponseBody
    @GetMapping("/{id:.+}")
    public R getWorkCentreDetailsById(@PathVariable String id) {
        return R.ok(workCentreDetailsService.getById(id));
    }

    /**
     * 查询所有数据
     *
     * @return
     */
    @ResponseBody
    @GetMapping("")
    public R getWorkCentreDetailsList(WorkCentreDetails workCentreDetails) {
        List<WorkCentreDetails> result;
        QueryWrapper<WorkCentreDetails> queryWrapper = new QueryWrapper<>();
        queryWrapper
                .like(!StringUtil.isBlank(workCentreDetails.getWorkshop()), "WORKSHOP", workCentreDetails.getWorkshop())
                .like(!StringUtil.isBlank(workCentreDetails.getWorkCenter()), "WORK_CENTER", workCentreDetails.getWorkCenter())
                .like(!StringUtil.isBlank(workCentreDetails.getWorkCenterDescription()), "WORK_CENTER_DESCRIPTION", workCentreDetails.getWorkCenterDescription())
                .like(!StringUtil.isBlank(workCentreDetails.getCostCenter()), "COST_CENTER", workCentreDetails.getCostCenter())
                .like(!StringUtil.isBlank(workCentreDetails.getCostCenterDescription()), "COST_CENTER_DESCRIPTION", workCentreDetails.getCostCenterDescription())
                .orderByAsc("WORKSHOP_WORK_CENTER_NO");

        result = workCentreDetailsService.list(queryWrapper);
        for (int i = 0; i < result.size(); i++) {
            result.get(i).setSEQ(i + 1);
        }
        return R.ok(result);
    }

    /**
     * 分页查询数据
     *
     * @param frontPage 分页信息
     * @return
     */
    @ResponseBody
    @GetMapping("/page")
    public R page(FrontPage<WorkCentreDetails> frontPage, WorkCentreDetails workCentreDetails) {
        IPage result;
        QueryWrapper<WorkCentreDetails> queryWrapper = new QueryWrapper<>();

        queryWrapper
                .like(!StringUtil.isBlank(workCentreDetails.getWorkshop()), "WORKSHOP", workCentreDetails.getWorkshop())
                .like(!StringUtil.isBlank(workCentreDetails.getWorkCenter()), "WORK_CENTER", workCentreDetails.getWorkCenter())
                .like(!StringUtil.isBlank(workCentreDetails.getWorkCenterDescription()), "WORK_CENTER_DESCRIPTION", workCentreDetails.getWorkCenterDescription())
                .like(!StringUtil.isBlank(workCentreDetails.getCostCenter()), "COST_CENTER", workCentreDetails.getCostCenter())
                .like(!StringUtil.isBlank(workCentreDetails.getCostCenterDescription()), "COST_CENTER_DESCRIPTION", workCentreDetails.getCostCenterDescription())
                .orderByAsc("WORKSHOP_WORK_CENTER_NO");
        result = workCentreDetailsService.page(frontPage.getPagePlus(), queryWrapper);
        for (int i = 0; i < result.getRecords().size(); i++) {
            WorkCentreDetails assetDetails = (WorkCentreDetails) result.getRecords().get(i);
            assetDetails.setSEQ((frontPage.getPage() - 1) * frontPage.getRows() + i + 1);
            result.getRecords().set(i, assetDetails);
        }
        return R.ok(result);
    }

    /**
     * 新增
     *
     * @param workCentreDetails 传递的实体
     * @return null 失败  实体成功
     */
    @PostMapping
    public R save(@RequestBody WorkCentreDetails workCentreDetails) {


        String user = CommonMethods.getUser();
        String site = CommonMethods.getSite();
        LocalDateTime now = LocalDateTime.now();
        String workshopWorkCenterNo = workCentreDetails.getWorkshop() + "_" + workCentreDetails.getWorkCenter();

        WorkCentreDetails details = new WorkCentreDetails();
        details.setWorkshopWorkCenterNo(workshopWorkCenterNo);
        List<WorkCentreDetails> selectList = workCentreDetailsService.selectList(details);
        if (selectList.size() != 0) {
            return R.failed("车间工作中心编号:" + workshopWorkCenterNo + "不允许重复");
        }
        String handle = "CostCenterBO:" + site + "," + workshopWorkCenterNo;
        workCentreDetails.setHandle(handle);
        workCentreDetails.setSite(site);
        workCentreDetails.setCreatedTime(now);
        workCentreDetails.setCreatedBy(user);
        workCentreDetails.setUpdatedBy(user);
        workCentreDetails.setUpdatedTime(now);
        workCentreDetails.setWorkshopWorkCenterNo(workshopWorkCenterNo);
        return R.ok(workCentreDetailsService.save(workCentreDetails));
    }

    /**
     * 修改
     *
     * @param workCentreDetails 传递的实体
     * @return null 失败  实体成功
     */
    @PostMapping("/PUT")
    public R updateById(@RequestBody WorkCentreDetails workCentreDetails) {
        String user = CommonMethods.getUser();
        LocalDateTime now = LocalDateTime.now();
        String workshopWorkCenterNo = workCentreDetails.getWorkshop() + "_" + workCentreDetails.getWorkCenter();
        WorkCentreDetails details = new WorkCentreDetails();
        details.setWorkshopWorkCenterNo(workshopWorkCenterNo);
        List<WorkCentreDetails> selectList = workCentreDetailsService.selectList(details);
        if (selectList.size() != 0 && !selectList.get(0).getHandle().equals(workCentreDetails.getHandle())) {
            return R.failed("车间工作中心编号:" + workshopWorkCenterNo + "不允许重复");
        }
        workCentreDetails.setWorkshopWorkCenterNo(workshopWorkCenterNo);
        workCentreDetails.setUpdatedBy(user);
        workCentreDetails.setUpdatedTime(now);
        return R.ok(workCentreDetailsService.updateById(workCentreDetails));
    }

    /**
     * 根据id删除对象
     *
     * @param id 实体ID
     * @return 0 失败  1 成功
     */
    @ResponseBody
    @GetMapping("/delete/{id}")
    public R removeById(@PathVariable("id") String id) {
        return R.ok(workCentreDetailsService.removeById(id));
    }

    /**
     * 批量删除对象
     *
     * @param ids 实体集合ID
     * @return 0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.POST, value = "/delete-batch")
    public R removeByIds(List<String> ids) {
        return R.ok(workCentreDetailsService.removeByIds(ids));
    }
}