package com.foreverwin.mesnac.anomaly.controller;

import com.foreverwin.mesnac.anomaly.model.WorkCentreDetails;
import com.foreverwin.mesnac.common.util.StringUtil;
import com.foreverwin.modular.core.util.R;
import com.foreverwin.modular.core.util.FrontPage;
import com.foreverwin.modular.core.util.CommonMethods;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import com.foreverwin.mesnac.anomaly.service.WorkHourStatisticsService;
import com.foreverwin.mesnac.anomaly.model.WorkHourStatistics;

import java.time.LocalDateTime;
import java.util.List;

/**
 *
 * @author YinQ
 * @since 2022-12-16
 */
@RestController
@RequestMapping("/Z-WORK-HOUR-STATISTICS")
public class WorkHourStatisticsController {

    @Autowired
    public WorkHourStatisticsService workHourStatisticsService;

    /**
    * 根据id查询
    *
    * @param id 主键
    * @return
    */
    @ResponseBody
    @GetMapping("/{id:.+}")
    public R getWorkHourStatisticsById(@PathVariable String id) {
        return R.ok( workHourStatisticsService.getById(id));
    }

    /**
     * 查询所有数据
     *
     * @return
     */
    @ResponseBody
    @GetMapping("")
    public R getWorkHourStatisticsList(WorkHourStatistics workHourStatistics){
        List<WorkHourStatistics> result;
        QueryWrapper<WorkHourStatistics> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(workHourStatistics);
        result = workHourStatisticsService.list(queryWrapper);
        return R.ok(result);
    }

    /**
     * 分页查询数据
     *
     * @param frontPage  分页信息
     * @return
     */
    @ResponseBody
    @GetMapping("/page")
    public R page(FrontPage<WorkHourStatistics> frontPage, WorkHourStatistics workHourStatistics){
        IPage result;
        QueryWrapper<WorkHourStatistics> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(workHourStatistics);
        result = workHourStatisticsService.page(frontPage.getPagePlus(), queryWrapper);
        for (int i = 0; i < result.getRecords().size(); i++) {
            WorkHourStatistics workHourStatistics1 = (WorkHourStatistics) result.getRecords().get(i);
            workHourStatistics1.setSEQ((frontPage.getPage() - 1) * frontPage.getRows() + i + 1);
            result.getRecords().set(i, workHourStatistics1);
        }
        return R.ok(result);
    }

    /**
     * 新增
     * @param workHourStatistics  传递的实体
     * @return  null 失败  实体成功
     */
    @PostMapping
    public R save(@RequestBody WorkHourStatistics workHourStatistics) {
        String handle = StringUtil.createQUID();
        String user = CommonMethods.getUser();
        String site = CommonMethods.getSite();
        LocalDateTime now = LocalDateTime.now();
        workHourStatistics.setHandle(handle);
        workHourStatistics.setSite(site);
        workHourStatistics.setCreatedTime(now);
        workHourStatistics.setCreatedBy(user);
        workHourStatistics.setUpdatedBy(user);
        workHourStatistics.setUpdatedTime(now);
        return R.ok(workHourStatisticsService.save(workHourStatistics));
    }

    /**
     * 修改
     * @param workHourStatistics  传递的实体
     * @return  null 失败  实体成功
     */
    @PostMapping("/PUT")
    public R updateById(@RequestBody WorkHourStatistics workHourStatistics) {
        String user = CommonMethods.getUser();
        LocalDateTime now = LocalDateTime.now();
        workHourStatistics.setUpdatedBy(user);
        workHourStatistics.setUpdatedTime(now);
        return R.ok(workHourStatisticsService.updateById(workHourStatistics));
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     * @return 0 失败  1 成功
     */
    @ResponseBody
    @GetMapping("/delete/{id}")
    public R removeById(@PathVariable("id") String id){
        return R.ok(workHourStatisticsService.removeById(id));
    }

    /**
     * 批量删除对象
     * @param ids 实体集合ID
     * @return  0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.POST, value = "/delete-batch")
    public R removeByIds(List<String> ids){
        return R.ok(workHourStatisticsService.removeByIds(ids));
    }
}