package com.foreverwin.mesnac.anomaly.controller;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.foreverwin.mesnac.anomaly.model.WxOrderDto;
import com.foreverwin.mesnac.anomaly.model.WxOrderReceived;
import com.foreverwin.mesnac.anomaly.service.WxOrderReceivedService;
import com.foreverwin.mesnac.anomaly.service.WxOrderService;
import com.foreverwin.modular.core.exception.BaseException;
import com.foreverwin.modular.core.util.R;
import com.foreverwin.modular.core.util.FrontPage;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.apache.ibatis.annotations.Param;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;


import com.foreverwin.mesnac.anomaly.model.WxOrder;
import sun.misc.UUDecoder;

import java.time.LocalDateTime;
import java.util.*;

/**
 *
 * @author YANG.WL
 * @since 2022-10-19
 */
@RestController
@RequestMapping("/Z-WX-ORDER")
public class WxOrderController {

    @Autowired
    public WxOrderService wxOrderService;

    @Autowired
    public WxOrderReceivedService wxOrderReceivedService;

    /**
    * 根据id查询
    *
    * @param id 主键
    * @return
    */
    @ResponseBody
    @GetMapping("/{id:.+}")
    public R getWxOrderById(@PathVariable String id) {
        return R.ok( wxOrderService.getById(id));
    }

    @ResponseBody
    @GetMapping("/androidFindWxOrderList")
    public String androidFindWxOrderList(@Param("param") String param){
        QueryWrapper<WxOrder> queryWrapper = new QueryWrapper<>();
        queryWrapper.like("PROJECT_NUMBER",param);
        List<WxOrder> wxOrderList = wxOrderService.list(queryWrapper);
        return Optional.ofNullable(wxOrderList)
                .map(t -> JSONObject.toJSONString(t)).orElse("null");
    }

    @ResponseBody
    @RequestMapping(method = RequestMethod.POST, value = "/insertWxOrderReceive")
    public R updateStatusById(String list,String user) {
         System.out.println(list+user);
        List<WxOrder> wxOrderList = JSONArray.parseArray(list, WxOrder.class);
        try {
            if (!wxOrderList.isEmpty()){
                for (WxOrder wxOrder : wxOrderList){
                    WxOrderReceived wxOrderReceived =new WxOrderReceived();
                    wxOrderReceived.setHandle(UUID.randomUUID().toString());
                    wxOrderReceived.setWxOrderBo(wxOrder.getHandle());
                    wxOrderReceived.setReceiveNumber(Integer.parseInt(wxOrder.getRemark1()));
                    wxOrderReceived.setCreatedDateTime(LocalDateTime.now());
                    wxOrderReceived.setCreatedUser(user);
                    wxOrderReceivedService.save(wxOrderReceived);
                }
                return R.ok();
            }else {
                return R.failed("收货数据为空");
            }

        }catch (Exception e){
            e.printStackTrace();
            return R.failed("收货失败");
        }

    }

    /**
     * 查询所有数据
     *
     * @return
     */
    @ResponseBody
    @GetMapping("")
    public R getWxOrderList(WxOrder wxOrder){
        List<WxOrderDto> result;
        List<WxOrder> wxOrderList;
        QueryWrapper<WxOrder> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(wxOrder);
        wxOrderList = wxOrderService.list(queryWrapper);
        result=wxOrderService.selectReceiveNumber();

        WxOrderDto wxOrderDtoDes=new WxOrderDto();
        wxOrderDtoDes.setReceiveNumber("收货数量");
        wxOrderDtoDes.setFullName("收货人");
        wxOrderDtoDes.setReceiveDateTime("收货时间");

        for (WxOrder wxOrder1: wxOrderList) {
            List<WxOrderDto> list=new ArrayList<>();
            wxOrder1.setCategories(list);
            for (WxOrderDto wxOrderDto : result){
                if (wxOrderDto.getHandle().equals(wxOrder1.getHandle())){
                    list.add(wxOrderDto);
                }
            }
            list.add(wxOrderDtoDes);
            Collections.swap(list, list.size()-1, 0);
        }
        Map map=new HashMap();
        map.put("categories",wxOrderList);
        return R.ok(map);
    }

    /**
     * 分页查询数据
     *
     * @param frontPage  分页信息
     * @return
     */
    @ResponseBody
    @GetMapping("/page")
    public R page(FrontPage<WxOrder> frontPage, WxOrder wxOrder){
        IPage result;
        QueryWrapper<WxOrder> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(wxOrder);
        if (frontPage.getGlobalQuery() != null && !"".equals(frontPage.getGlobalQuery().trim())) {
            //TODO modify global query
            queryWrapper.lambda().and(wrapper -> wrapper
                .like(WxOrder::getHandle, frontPage.getGlobalQuery())
                .or().like(WxOrder::getProjectNumber, frontPage.getGlobalQuery())
                .or().like(WxOrder::getCustomer, frontPage.getGlobalQuery())
                .or().like(WxOrder::getModel, frontPage.getGlobalQuery())
                .or().like(WxOrder::getItem, frontPage.getGlobalQuery())
                .or().like(WxOrder::getItemDescription, frontPage.getGlobalQuery())
                .or().like(WxOrder::getDrawingNo, frontPage.getGlobalQuery())
                .or().like(WxOrder::getSubassemblies, frontPage.getGlobalQuery())
                .or().like(WxOrder::getSurface, frontPage.getGlobalQuery())
                .or().like(WxOrder::getFactoryNo, frontPage.getGlobalQuery())
                .or().like(WxOrder::getFactoryDescription, frontPage.getGlobalQuery())
                .or().like(WxOrder::getExecutiveManager, frontPage.getGlobalQuery())
                .or().like(WxOrder::getPurchaseOrder, frontPage.getGlobalQuery())
                .or().like(WxOrder::getRemark1, frontPage.getGlobalQuery())
                .or().like(WxOrder::getRemark2, frontPage.getGlobalQuery())
                .or().like(WxOrder::getCreateUser, frontPage.getGlobalQuery())
                .or().like(WxOrder::getModifyUser, frontPage.getGlobalQuery())
                .or().like(WxOrder::getOther1, frontPage.getGlobalQuery())
                .or().like(WxOrder::getOther2, frontPage.getGlobalQuery())
    );
        }
        result = wxOrderService.page(frontPage.getPagePlus(), queryWrapper);
        return R.ok(result);
    }

    /**
     * 新增
     * @param wxOrder  传递的实体
     * @return  null 失败  实体成功
     */
    @PostMapping
    public R save(@RequestBody WxOrder wxOrder) {
        return R.ok(wxOrderService.save(wxOrder));
    }

    /**
     * 修改
     * @param wxOrder  传递的实体
     * @return  null 失败  实体成功
     */
    @PutMapping
    public R updateById(@RequestBody WxOrder wxOrder) {
        return R.ok(wxOrderService.updateById(wxOrder));
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     * @return 0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.DELETE, value = "/{id:.+}")
    public R removeById(@PathVariable("id") String id){
        return R.ok(wxOrderService.removeById(id));
    }

    /**
     * 批量删除对象
     * @param ids 实体集合ID
     * @return  0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.POST, value = "/delete-batch")
    public R removeByIds(List<String> ids){
        return R.ok(wxOrderService.removeByIds(ids));
    }
    class   Categories{
        private String name;
        private String categories;

        public String getName() {
            return name;
        }

        public void setName(String name) {
            this.name = name;
        }

        public String getCategories() {
            return categories;
        }

        public void setCategories(String categories) {
            this.categories = categories;
        }
    }
}