package com.foreverwin.mesnac.anomaly.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.foreverwin.mesnac.anomaly.dto.AbnormalBillDisposeDto;
import com.foreverwin.mesnac.anomaly.dto.AbnormalPlanDto;
import com.foreverwin.mesnac.anomaly.mapper.AbnormalBillDisposeMapper;
import com.foreverwin.mesnac.anomaly.mapper.AbnormalBillMapper;
import com.foreverwin.mesnac.anomaly.mapper.AbnormalPlanMapper;
import com.foreverwin.mesnac.anomaly.model.AbnormalBill;
import com.foreverwin.mesnac.anomaly.model.AbnormalBillDispose;
import com.foreverwin.mesnac.anomaly.model.AbnormalPlan;
import com.foreverwin.mesnac.anomaly.service.AbnormalBillService;
import com.foreverwin.mesnac.anomaly.service.AbnormalPlanService;
import com.foreverwin.mesnac.common.constant.Constants;
import com.foreverwin.mesnac.common.constant.CustomFieldConstant;
import com.foreverwin.mesnac.common.enums.HandleEnum;
import com.foreverwin.mesnac.common.mapper.MessageMapper;
import com.foreverwin.mesnac.common.model.Message;
import com.foreverwin.mesnac.common.service.MessageService;
import com.foreverwin.mesnac.common.util.ActiveMQUtil;
import com.foreverwin.mesnac.common.util.StringUtil;
import com.foreverwin.mesnac.integration.service.InterfaceService;
import com.foreverwin.mesnac.listener.mapper.UsrMapper;
import com.foreverwin.mesnac.meapi.mapper.MessageTypeMapper;
import com.foreverwin.mesnac.meapi.mapper.UserGroupMapper;
import com.foreverwin.mesnac.meapi.model.*;
import com.foreverwin.mesnac.meapi.service.*;
import com.foreverwin.modular.core.exception.BaseException;
import com.foreverwin.modular.core.exception.BusinessException;
import com.foreverwin.modular.core.meext.MEServices;
import com.foreverwin.modular.core.util.CommonMethods;
import com.foreverwin.modular.core.util.FrontPage;
import com.sap.me.common.RouterType;
import com.sap.me.demand.CreateShopOrderRequest;
import com.sap.me.demand.ShopOrderServiceInterface;
import com.sap.me.demand.ShopOrderStatus;
import com.sap.me.demand.ShopOrderType;
import com.sap.me.productdefinition.BomType;
import com.visiprise.globalization.DateGlobalizationServiceInterface;
import com.visiprise.globalization.GlobalizationService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 计划异常待办事项 服务实现类
 * </p>
 *
 * @author zjw
 * @since 2021-08-07
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class AbnormalPlanServiceImpl extends ServiceImpl<AbnormalPlanMapper, AbnormalPlan> implements AbnormalPlanService {


    @Autowired
    private AbnormalPlanMapper abnormalPlanMapper;

    @Autowired
    private MessageMapper messageMapper;

    @Autowired
    private MessageService messageService;

    @Autowired
    private NwaUserService nwaUserService;

    @Autowired
    private InterfaceService interfaceService;

    @Autowired
    private ActiveMQUtil activeMQUtil;

    @Autowired
    private AbnormalBillService abnormalBillService;

    @Autowired
    private ShopOrderService shopOrderService;

    @Autowired
    private AbnormalBillMapper abnormalBillMapper;

    @Autowired
    private AbnormalBillDisposeMapper abnormalBillDisposeMapper;

    @Autowired
    private UserGroupMapper userGroupMapper;

    @Autowired
    private MessageTypeMapper messageTypeMapper;

    @Autowired
    private UsrMapper usrMapper;

    @Autowired
    private BomService bomService;

    @Autowired
    private RouterService routerService;
    @Autowired
    private CustomFieldsService customFieldsService;
    @Autowired
    private RouterStepcontentService routerStepcontentService;



    @Override
    public IPage<AbnormalPlan> selectPage(FrontPage<AbnormalPlan> frontPage, AbnormalPlan abnormalPlan) {
        QueryWrapper<AbnormalPlan> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(abnormalPlan);
        return super.page(frontPage.getPagePlus(), queryWrapper);
    }

    @Override
    public List<AbnormalPlan> selectList(AbnormalPlan abnormalPlan) {
        QueryWrapper<AbnormalPlan> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(abnormalPlan);
        return super.list(queryWrapper);
    }


    @Override
    public void saveAbnormalPlan(AbnormalBillDisposeDto abnormalBillDisposeDto, LocalDateTime now, String check) {
        String site = CommonMethods.getSite();
        AbnormalPlan abnormalPlan = new AbnormalPlan();
        String user = CommonMethods.getUser();
        //设置主键
        abnormalPlan.setHandle(HandleEnum.ABNORMAL_PLAN.getHandle(site,abnormalBillDisposeDto.getAbnormalNo()));
        abnormalPlan.setSite(site);
        abnormalPlan.setAbnormalNo(abnormalBillDisposeDto.getAbnormalNo());
        //未处理
        if (check.equals("true")){
            abnormalPlan.setStatus(Constants.UNTREATED);
        }
        abnormalPlan.setWorkCenter(abnormalBillDisposeDto.getWorkCenter());
        abnormalPlan.setAbnormalMethod(abnormalBillDisposeDto.getAbnormalMethod());
        abnormalPlan.setShopOrder(abnormalBillDisposeDto.getShopOrder());
        abnormalPlan.setResrce(abnormalBillDisposeDto.getResrce());
        abnormalPlan.setItemBo(abnormalBillDisposeDto.getItemBo());
        if(!StringUtil.isBlank(abnormalBillDisposeDto.getNcQty())){
            abnormalPlan.setQty(Long.parseLong(abnormalBillDisposeDto.getNcQty()));
        }
        //没有处理人和处理时间
        abnormalPlan.setClosedUser(user);
        abnormalPlan.setClosedDateTime(now);
        abnormalPlan.setCreatedUser(user);
        abnormalPlan.setCreatedDateTime(now);
        abnormalPlan.setModifiedUser(user);
        abnormalPlan.setModifiedDateTime(now);
        this.saveOrUpdate(abnormalPlan);
    }

    @Override
    public void saveAbnormalPlan(AbnormalBill abnormalBill, AbnormalBillDispose abnormalBillDispose, LocalDateTime now) {
        AbnormalBillDisposeDto abnormalBillDisposeDto = new AbnormalBillDisposeDto();
        abnormalBillDisposeDto.setHandle(abnormalBill.getHandle());
        abnormalBillDisposeDto.setAbnormalNo(abnormalBill.getAbnormalNo());
        abnormalBillDisposeDto.setStatus(abnormalBill.getStatus());
        abnormalBillDisposeDto.setWorkCenter(abnormalBill.getWorkCenter());
        abnormalBillDisposeDto.setAbnormalMethod(abnormalBillDispose.getAbnormalMethod());
        abnormalBillDisposeDto.setShopOrder(abnormalBill.getShopOrder());
        abnormalBillDisposeDto.setResrce(abnormalBill.getResrce());
        abnormalBillDisposeDto.setItemBo(abnormalBill.getItemBo());
        abnormalBillDisposeDto.setNcQty(abnormalBill.getNcQty());
        this.saveAbnormalPlan(abnormalBillDisposeDto,now,"true");
    }

    @Override
    public List<AbnormalPlanDto> findAbnormalPlan(AbnormalPlanDto abnormalPlanDto) {
        String site = CommonMethods.getSite();
        String locale = LocaleContextHolder.getLocale().getLanguage();
        abnormalPlanDto.setSite(site);
        List<AbnormalPlanDto> findAbnormalPlan=abnormalPlanMapper.findAbnormalPlan(locale,abnormalPlanDto);
        Map map=new HashMap();
        map.put("site",site);
//        List<AbnormalPlanDto> abnormalPlanDtoList = findAbnormalPlan.stream().map(dto -> {
//            AbnormalPlanDto planDto = dto;
//            map.put("abnormalBo",planDto.getAbnormalNo());
//            map.put("routerBo",planDto.getRepairRouter());
//            List<Map<String, Object>> result =routerStepcontentService.selectDisRouterInfo(map);
//            String ReapirContent="";
//            if(result!=null&&result.size()!=0){
//                for (int j = 0; j < result.size(); j++) {
//                    ReapirContent=ReapirContent+"【"+(String) result.get(j).get("STEP_ID")+"】"+
//                            (String) result.get(j).get("ROUTER_STEP_CONTENT");
//                }
//            }
//            planDto.setReapirContent(ReapirContent);
//            return planDto;
//
//        }).collect(Collectors.toList());

        for (int i = 0; i < findAbnormalPlan.size(); i++) {
            String abnormalBo = findAbnormalPlan.get(i).getAbnormalNo();
            String routerBo=findAbnormalPlan.get(i).getRtHandle();
            if(StringUtil.notBlank(routerBo)){
                //map.put("abnormalBo",abnormalBo);
                map.put("routerBo",routerBo);
                List<Map<String, Object>> result =routerStepcontentService.selectDisRouterInfo(map);
                String ReapirContent="";
                if(result!=null&&result.size()!=0){
                    for (int j = 0; j < result.size(); j++) {
                        ReapirContent=ReapirContent+"【"+(String) result.get(j).get("STEP_ID")+"】"+
                                (String) result.get(j).get("ROUTER_STEP_CONTENT");
                    }
                }
                findAbnormalPlan.get(i).setReapirContent(ReapirContent);
            }
        }

//        for (AbnormalPlanDto planDto:findAbnormalPlan){
//            String abnormalBo = planDto.getAbnormalNo();
//            String routerBo=planDto.getRepairRouter();
//            map.put("abnormalBo",abnormalBo);
//            map.put("routerBo",routerBo);
//            List<Map<String, Object>> result =routerStepcontentService.selectDisRouterInfo(map);
//            String ReapirContent="";
//            if(result!=null&&result.size()!=0){
//                for (int j = 0; j < result.size(); j++) {
//                    ReapirContent=ReapirContent+"【"+(String) result.get(j).get("STEP_ID")+"】"+
//                            (String) result.get(j).get("ROUTER_STEP_CONTENT");
//                }
//            }
//        }
        return findAbnormalPlan;
    }

    @Override
    public void processes(List<AbnormalPlan> abnormalPlanList)  {
        String site = CommonMethods.getSite();
        LocalDateTime now = LocalDateTime.now();
        String user = CommonMethods.getUser();
        ShopOrder shopOrder=new ShopOrder();
        List<AbnormalPlan> abnormalPlans = new ArrayList<>();
        List<AbnormalBill> abnormalBills = new ArrayList<>();
        for (AbnormalPlan abnormalPlan : abnormalPlanList){
            AbnormalBill abnormalBill = abnormalBillMapper.selectById(HandleEnum.ABNORMAL_BILL.getHandle(site, abnormalPlan.getAbnormalNo()));
            //设置handle
            abnormalPlan.setHandle(HandleEnum.ABNORMAL_PLAN.getHandle(site,abnormalPlan.getAbnormalNo()));

            if(Constants.PROCESSED.equals(abnormalPlan.getStatus())){
                throw new BaseException("异常单："+abnormalPlan.getAbnormalNo()+"已经处理，不能再次处理");
            }
            //设置状态为已处理
            abnormalPlan.setStatus(Constants.PROCESSED);
            //设置处理人
            abnormalPlan.setProcessor(user);
            //设置发送时间
            abnormalPlan.setSendDateTime(now);
            abnormalPlans.add(abnormalPlan);
            abnormalBills.add(abnormalBill);


            /**
             * 如果工单编号为空值就把异常单号作为返修工单号
             */
            if(abnormalPlan.getShopOrder()==null||abnormalPlan.getShopOrder()=="") {
               if(abnormalPlan.getAbnormalMethod().equals("FP") || abnormalPlan.getAbnormalMethod().equals("F")  || abnormalPlan.getAbnormalMethod().equals("FB")){
                    shopOrder.setShopOrder(abnormalBill.getAbnormalNo());
                    shopOrder.setPlannedItemBo(abnormalBill.getItemBo());
                    shopOrder.setPlannedBomBo(abnormalBill.getAbnormalNo());
                    shopOrder.setRouterBo(abnormalPlan.getRtHandle());
                    shopOrder.setQtyOrdered(new Double(abnormalBill.getNcQty()));
                    shopOrder.setQtyToBuild(new Double(abnormalBill.getNcQty()));
                    try {
                        CreateRepairShopOrder(site,shopOrder);
                    }catch (Exception e){
                        throw new BaseException(e.getMessage().trim());
                    }
                }
            }else if (abnormalPlan.getShopOrder()!=null||abnormalPlan.getShopOrder()!=""){
                ShopOrder shopstatus=shopOrderService.getById(HandleEnum.SHOP_ORDER.getHandle(site, abnormalPlan.getShopOrder()));
                if (shopstatus.getStatusBo().equals("StatusBO:1000,503") && (abnormalPlan.getAbnormalMethod().equals("FP") || abnormalPlan.getAbnormalMethod().equals("F")  || abnormalPlan.getAbnormalMethod().equals("FB"))){
                    shopOrder.setShopOrder(abnormalBill.getAbnormalNo());
                    shopOrder.setPlannedItemBo(abnormalBill.getItemBo());
                    shopOrder.setPlannedBomBo(abnormalBill.getAbnormalNo());
                    shopOrder.setRouterBo(abnormalPlan.getRtHandle());
                    shopOrder.setQtyOrdered(new Double(abnormalBill.getNcQty()));
                    shopOrder.setQtyToBuild(new Double(abnormalBill.getNcQty()));
                    try {
                        CreateRepairShopOrder(site,shopOrder);
                    }catch (Exception e){
                        throw new BaseException(e.getMessage().trim());
                    }
                }
            }
        }

        //发送消息
        for(int i = 0; i < abnormalBills.size() && i < abnormalPlans.size(); i++) {
            String sendUserGroup = abnormalPlans.get(i).getSendUserGroup();
            List<String> sendUserGroupList = new ArrayList<>(Arrays.asList(sendUserGroup.split(",")));

            for (String userGroup : sendUserGroupList) {
                List<String> userList=new ArrayList<>();
                abnormalPlans.get(i).setSendUserGroup(userGroup);
                for (String u : sendUserGroupList){
                    List<NwaUser> nwaUsers  = nwaUserService.checkUserGroup(site,u);
                    for (NwaUser nwaUser:nwaUsers){
                        userList.add(nwaUser.getFullName());
                    }
                }
                this.sendMessage(userList,abnormalBills.get(i),abnormalPlans.get(i));
            }
            abnormalPlans.get(i).setSendUserGroup(sendUserGroup);
        }
        //更新

        this.saveOrUpdateBatch(abnormalPlans);
    }




    @Override
    public void sendMessage(List<String> userList,AbnormalBill abnormalBill,AbnormalPlan abnormalPlan) {
        //创建一个消息记录表
        String site = CommonMethods.getSite();
        String user = CommonMethods.getUser();
        LocalDateTime now = LocalDateTime.now();
        Message message = new Message();
        message.setHandle(UUID.randomUUID().toString());
        message.setSite(site);
        message.setObjectBo(abnormalPlan.getHandle());
        message.setType(Constants.ABNORMAL_MESSAGE_PLAN);
        //设置消息的类型，消息的唯一id
        MessageType messageType = messageTypeMapper.findMessageTypeByUserGroup(site, abnormalPlan.getSendUserGroup());
        if(messageType == null){
            throw new BaseException("根据发送的用户组找不到对应的消息类型，请维护。");
        }
        message.setMessageType(messageType.getMessageType());
        message.setSendUserGroup(abnormalPlan.getSendUserGroup());
        //格式化消息内容
        String messageContent = abnormalBillService.formatMessage(abnormalBill, message.getMessageType(), site);
        NwaUser nwaUser=nwaUserService.findUserByUserName(user);
        messageContent=messageContent+"计划人员："+nwaUser.getFullName()+"，返修方案描述："+abnormalPlan.getReapirContent()+",   发送用户组："+userList.toString();
        message.setContent(messageContent);
        message.setResponseDateTime(now);
        UserGroup customFieldByGroup = userGroupMapper.getCustomFieldByGroup(site, message.getSendUserGroup());
       //设置节点（计划异常待办事项不需要设置节点）


        if(customFieldByGroup != null && !StringUtil.isBlank(customFieldByGroup.getMessageNoticeLevel())) {
            message.setGrade(Double.parseDouble(customFieldByGroup.getMessageNoticeLevel()));
        }
        message.setStatus("Y");
        message.setCreatedUser(user);
        message.setCreatedDateTime(now);
        message.setModifiedUser(user);
        message.setModifiedDateTime(now);


        //发送消息
        List<NwaUser> nwaUsers  = nwaUserService.checkUserGroup(site,message.getSendUserGroup());
        if(nwaUsers != null && nwaUsers.size() <= 0){
            throw new BaseException("消息发送失败，发送的用户组下面没有用户");
        }
        List<String> customFieldByUserId = usrMapper.findCustomFieldByUserId(site, nwaUsers);
        if(customFieldByUserId != null && customFieldByUserId.size() <= 0){
            throw new BaseException("消息发送失败，发送的用户组下面的自定义数据没有维护");
        }
        StringBuilder sendUsers = new StringBuilder();
        for (int i = 0; i < Objects.requireNonNull(customFieldByUserId).size(); i++) {
            if(i == (customFieldByUserId.size() - 1)){
                sendUsers.append(customFieldByUserId.get(i));
            }else{
                sendUsers.append(customFieldByUserId.get(i)).append("|");
            }
        }
        activeMQUtil.wechatSendMessage(sendUsers.toString(),message.getContent());
        messageService.saveOrUpdate(message);
    }


    public void CreateRepairShopOrder(String site, ShopOrder repairshopOrder) throws Exception {

        //获取工单标准服务
        ShopOrderServiceInterface shopOrderMeService = MEServices.create("com.sap.me.demand", "ShopOrderService", site);
        DateGlobalizationServiceInterface dateService = GlobalizationService.getInvariantService("com.visiprise.globalization.DateGlobalizationService");

        //工单号为异常单号
        String shopOrder = repairshopOrder.getShopOrder();
        if (StringUtil.isBlank(shopOrder)) {
            throw BusinessException.build("工单编号不能为空");
        }
        String reRouter = repairshopOrder.getRouterBo();
        if (StringUtil.isBlank(reRouter)) {
            throw BusinessException.build("请联系工艺维护返修工艺路线");
        }
        String shopOrderBo = HandleEnum.SHOP_ORDER.getHandle(site, shopOrder);

        //工单状态为可以下达

        //工单时间时间处理
//        String startDateTime = shopOrderSyncRequest.getPLANNED_START_DATE();
//        String completeDateTime = shopOrderSyncRequest.getPLANNED_COMP_DATE();
//        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

        //查询工单是否存在
        ShopOrder shopOrderModel = shopOrderService.getById(shopOrderBo);
        if (shopOrderModel == null) {
            //工单不存在新建
            CreateShopOrderRequest request = new CreateShopOrderRequest();
            request.setShopOrderRef(shopOrderBo);
            request.setShopOrder(shopOrder);
            request.setStatus(ShopOrderStatus.fromValue("501"));
            //订单类型为返修订单
            request.setShopOrderType(ShopOrderType.PRODUCTION);

            //计划物料不能为空 为返修物料
            String plannedItem = repairshopOrder.getPlannedItemBo();
            //固定返修Bom清单为 000100490612
            String plannedBom = "000100490612";
//            plannedBom = StringUtil.notBlank(plannedBom) ? plannedBom : shopOrder;
            //计划工艺路线为 工艺维护的返修工艺路线
            String plannedRouter = repairshopOrder.getRouterBo();
//            plannedRouter = StringUtil.notBlank(plannedRouter) ? plannedRouter : shopOrder;



            //计划BOM类型，默认为工单BOM：H
            String bomType = BomType.SHOPORDERBOM.value();
            String routerType = RouterType.SPECIAL.value();
            if (StringUtil.isBlank(plannedItem)) {
                throw BusinessException.build("工单的计划物料不能为空");
            }

            //查询物料清单和工艺路线
            Bom bomModel = bomService.getMaxRevisionBom(site, plannedBom);
            if (bomModel == null) {
                throw BusinessException.build("返修工单【" +shopOrder+ "】的物料清单未同步！");
            }

            Router routerModel = routerService.getById(reRouter);
            if (routerModel == null) {
                throw BusinessException.build("返修工单【" +shopOrder+ "】的返修工艺路线不存在！");
            }

           // String plannedItemBo = HandleEnum.ITEM.getHandle(site, plannedItem, "A");
            String plannedBomBo = HandleEnum.BOM.getHandle(site, plannedBom, bomType, "#");
            String plannedRouterBo = reRouter;

            request.setPlannedItemRef(plannedItem);
            request.setPlannedBomRef(plannedBomBo);
            request.setPlannedRouterRef(plannedRouterBo);
            request.setPriority(new BigDecimal(500));
            request.setQuantityOrdered(new BigDecimal(repairshopOrder.getQtyOrdered()));
            request.setQuantityToBuild(new BigDecimal(repairshopOrder.getQtyToBuild()));

            //工单创建
            try {
                shopOrderMeService.createShopOrder(request);
            } catch (com.sap.me.frame.domain.BusinessException e) {
               throw BusinessException.build("返修工单【" +shopOrder+ "】创建失败！"+e.getMessage());
            }
        }

        //工单自定义字段
//        List<CustomFields> customFieldsList = new ArrayList<>();
//        CustomFields customFields = new CustomFields();
//        if (StringUtil.notBlank(shopOrderSyncRequest.getFACTORY())) {
//            customFields.setHandle(shopOrderBo);
//            customFields.setAttribute(CustomFieldConstant.SO_FACTORY);
//            customFields.setValue(shopOrderSyncRequest.getFACTORY());
//            customFieldsList.add(customFields);
//        }
//
//        if (StringUtil.notBlank(shopOrderSyncRequest.getWORK_ORDER())) {
//            customFields = new CustomFields();
//            customFields.setHandle(shopOrderBo);
//            customFields.setAttribute(CustomFieldConstant.SO_WORK_ORDER);
//            customFields.setValue(shopOrderSyncRequest.getWORK_ORDER());
//            customFieldsList.add(customFields);
//        }

//        if (StringUtil.notBlank(repairshopOrder.getItemBo())) {
//            customFields = new CustomFields();
//            customFields.setHandle(shopOrderBo);
//            customFields.setAttribute(CustomFieldConstant.SO_ITEM_NUMBER);
//            customFields.setValue(repairshopOrder.getItemBo());
//            customFieldsList.add(customFields);
//        }

//        if (StringUtil.notBlank(shopOrderSyncRequest.getSHOP_ORDER_TYPE())) {
//            customFields = new CustomFields();
//            customFields.setHandle(shopOrderBo);
//            customFields.setAttribute(CustomFieldConstant.SO_SHOP_ORDER_TYPE);
//            customFields.setValue(shopOrderSyncRequest.getSHOP_ORDER_TYPE());
//            customFieldsList.add(customFields);
//        }
//
//        if (StringUtil.notBlank(shopOrderSyncRequest.getPRODUCT_CATEGORY())) {
//            customFields = new CustomFields();
//            customFields.setHandle(shopOrderBo);
//            customFields.setAttribute(CustomFieldConstant.SO_PRODUCT_CATEGORY);
//            customFields.setValue(shopOrderSyncRequest.getPRODUCT_CATEGORY());
//            customFieldsList.add(customFields);
//        }
//
//        if (StringUtil.notBlank(shopOrderSyncRequest.getCOMMENTS())) {
//            customFields = new CustomFields();
//            customFields.setHandle(shopOrderBo);
//            customFields.setAttribute(CustomFieldConstant.SO_COMMENTS);
//            customFields.setValue(shopOrderSyncRequest.getCOMMENTS());
//            customFieldsList.add(customFields);
//        }
//
//        if (StringUtil.notBlank(shopOrderSyncRequest.getSTART_PRODUCT())) {
//            customFields = new CustomFields();
//            customFields.setHandle(shopOrderBo);
//            customFields.setAttribute(CustomFieldConstant.SO_START_PRODUCT);
//            customFields.setValue(shopOrderSyncRequest.getSTART_PRODUCT());
//            customFieldsList.add(customFields);
//        }


        //自定义字段保存
//        if (customFieldsList != null && customFieldsList.size() > 0) {
//            customFieldsService.saveOrUpdateCustomFields(customFieldsList);
//        }
    }



}