package com.foreverwin.mesnac.anomaly.service.impl;

import com.foreverwin.mesnac.anomaly.model.OutStore;
import com.foreverwin.mesnac.common.util.ExcelUtils;
import com.foreverwin.mesnac.common.util.StringUtil;
import com.foreverwin.modular.core.util.CommonMethods;
import com.foreverwin.modular.core.util.FrontPage;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.foreverwin.mesnac.anomaly.model.ImprovementProposals;
import com.foreverwin.mesnac.anomaly.mapper.ImprovementProposalsMapper;
import com.foreverwin.mesnac.anomaly.service.ImprovementProposalsService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.foreverwin.modular.core.util.R;
import org.apache.commons.fileupload.FileItem;
import org.springframework.stereotype.Service;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.commons.CommonsMultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.websocket.server.PathParam;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Date;
import java.util.List;
/**
 * <p>
 * 改善提案报告书 服务实现类
 * </p>
 *
 * @author YinQ
 * @since 2022-09-19
 */
@Service("ImprovementProposalsService")
@Transactional(rollbackFor = Exception.class)
public class ImprovementProposalsServiceImpl extends ServiceImpl<ImprovementProposalsMapper, ImprovementProposals> implements ImprovementProposalsService {


    @Autowired
    private ImprovementProposalsMapper improvementProposalsMapper;

    @Override
    public IPage<ImprovementProposals> selectPage(FrontPage<ImprovementProposals> frontPage, ImprovementProposals improvementProposals) {
        QueryWrapper<ImprovementProposals> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(improvementProposals);
        return super.page(frontPage.getPagePlus(), queryWrapper);
    }

    @Override
    public List<ImprovementProposals> selectList(ImprovementProposals improvementProposals) {
        QueryWrapper<ImprovementProposals> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(improvementProposals);
        return super.list(queryWrapper);
    }

    @Override
    public ImprovementProposals insertData(ImprovementProposals improvementProposals) {
        String user = CommonMethods.getUser();
        String site = CommonMethods.getSite();
        LocalDateTime now = LocalDateTime.now();
        String handle = StringUtil.createQUID();

        improvementProposals.setHandle(handle);
        improvementProposals.setSite(site);
        improvementProposals.setCreatedBy(user);
        improvementProposals.setCreatedTime(now);
        improvementProposals.setUpdatedBy(user);
        improvementProposals.setUpdatedTime(now);

        StringBuffer sequenceCode = new StringBuffer();
        String format = now.format(DateTimeFormatter.ofPattern("yyyyMMdd"));
        for (int i = 0; i < format.length(); i++) {
            int random = (int)(Math.random() * 10);
            sequenceCode.append(format.charAt(i)).append(handle.substring(random, random + 1));
        }
        improvementProposals.setSequenceCode(sequenceCode.toString());

        return improvementProposals;
    }

    /**
     * 导入
     * @param improvementProposals
     * @return
     */
    @Override
    public Boolean improvementProposalsListImportFile(List<ImprovementProposals> improvementProposals) {

        try {
            for (ImprovementProposals improvementProposal : improvementProposals) {
                improvementProposal = this.insertData(improvementProposal);
            }
            super.saveBatch(improvementProposals);
            return true;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return false;
    }
    /**
     * 改善提案导入
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public R improvementProposalsImportFile(FileItem fileItem, HttpServletRequest req) {
        String fileType = req.getParameter("fileType");
        String site = req.getParameter("site");
        String user = req.getParameter("user");
        CommonMethods.setSite(site);
        CommonMethods.setUser(user);
        try {
            CommonMethods.setSite(site);
            CommonMethods.setUser(user);
            CommonsMultipartFile multipartFile = new CommonsMultipartFile(fileItem);
            List<ImprovementProposals> improvementProposals = ExcelUtils.readExcel(fileType, ImprovementProposals.class, multipartFile, null);
            if (this.improvementProposalsListImportFile(improvementProposals)){
                return R.ok("导入数据成功");
            }else {
                return R.failed("导入数据失败");
            }
        } catch (Exception e) {
            e.printStackTrace();
            return R.failed(e.getMessage());
        }
    }

}