package com.foreverwin.mesnac.anomaly.service.impl;

import cn.hutool.core.util.StrUtil;
import com.foreverwin.mesnac.common.util.ExcelUtils;
import com.foreverwin.mesnac.common.util.StringUtil;
import com.foreverwin.modular.core.util.CommonMethods;
import com.foreverwin.modular.core.util.FrontPage;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.foreverwin.mesnac.anomaly.model.OutStore;
import com.foreverwin.mesnac.anomaly.mapper.OutStoreMapper;
import com.foreverwin.mesnac.anomaly.service.OutStoreService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.foreverwin.modular.core.util.R;
import org.apache.commons.fileupload.FileItem;
import org.springframework.stereotype.Service;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.commons.CommonsMultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.websocket.server.PathParam;
import java.io.IOException;
import java.io.InputStream;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.Optional;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author YinQ
 * @since 2022-09-15
 */
@Service("outStoreService")
@Transactional(rollbackFor = Exception.class)
public class OutStoreServiceImpl extends ServiceImpl<OutStoreMapper, OutStore> implements OutStoreService {


    @Autowired
    private OutStoreMapper outStoreMapper;

    @Override
    public IPage<OutStore> selectPage(FrontPage<OutStore> frontPage, OutStore outStore) {
        QueryWrapper<OutStore> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(outStore);
        return super.page(frontPage.getPagePlus(), queryWrapper);
    }

    @Override
    public List<OutStore> selectList(OutStore outStore) {
        QueryWrapper<OutStore> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(outStore);
        return super.list(queryWrapper);
    }

    @Override
    public boolean outStoreImportFile(List<OutStore> outStoreList) {
        String user = CommonMethods.getUser();
        LocalDateTime now = LocalDateTime.now();
        try {
            for (OutStore outStore : outStoreList) {
                String handle = StringUtil.createQUID();
                outStore.setHandle(handle);
                if (StringUtil.isBlank(outStore.getImportUser())){
                    outStore.setImportUser(user);
                }
                outStore.setImportDateTime(now);
            }
            super.saveBatch(outStoreList);
            return true;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return false;
    }

    /**
     * 出库导入
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public R outStoreListImportFile(FileItem fileItem, HttpServletRequest req) throws Exception {
        String fileType = req.getParameter("fileType");
        String site = req.getParameter("site");
        String user = req.getParameter("user");
        CommonMethods.setSite(site);
        CommonMethods.setUser(user);
        try {
            CommonsMultipartFile multipartFile = new CommonsMultipartFile(fileItem);
            List<OutStore> outStoreList = ExcelUtils.readExcel(fileType, OutStore.class, multipartFile, null);
            if (outStoreList.size() == 0){
                return R.failed("导出数据失败：0条数据");
            }
            if (!Optional.ofNullable(outStoreList.get(0).getPostingDateTime()).isPresent()){
                return R.failed("过账日期格式错误，格式需为：yyyy-MM-dd HH:mm:ss");
            }else {
                for (OutStore outStore : outStoreList) {
                    String dateTime = outStore.getPostingDateTime();
                    if (outStore.getPostingDateTime().indexOf("CST") != -1){
                        SimpleDateFormat sdf = new SimpleDateFormat("EEE MMM dd HH:mm:ss zzz yyyy", Locale.US);
                        Date date = sdf.parse(dateTime);
                        String formatStr = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(date);
                        outStore.setPostingDateTime(formatStr);
                    } else {
                        boolean result = false;
                        try {
                            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                            Date date =format.parse(dateTime);
                            outStore.setPostingDateTime(dateTime);
                        } catch (ParseException e) {
                            result = true;
                        }
                        if (result)
                        return R.failed("过账日期格式错误，格式需为：yyyy-MM-dd HH:mm:ss");
                    }
                }
            }
            if (this.outStoreImportFile(outStoreList)){
                return R.ok("导入数据成功");
            }else {
                return R.failed("导入数据失败");
            }
        } catch (Exception e) {
            e.printStackTrace();
            return R.failed("导入数据失败" + e.getMessage());
        }
    }
}