package com.foreverwin.mesnac.anomaly.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.foreverwin.mesnac.anomaly.dto.IntegrationLogDto;
import com.foreverwin.mesnac.anomaly.mapper.ReportMapper;
import com.foreverwin.mesnac.anomaly.service.ReportService;
import com.foreverwin.mesnac.anomaly.utils.CollectionUtil;
import com.foreverwin.mesnac.anomaly.utils.DateReportUtils;
import com.foreverwin.mesnac.common.enums.HandleEnum;
import com.foreverwin.mesnac.common.util.DateUtil;
import com.foreverwin.mesnac.common.util.StringUtil;
import com.foreverwin.mesnac.meapi.mapper.DataFieldListMapper;
import com.foreverwin.mesnac.meapi.mapper.ItemMapper;
import com.foreverwin.mesnac.meapi.mapper.ShopOrderMapper;
import com.foreverwin.mesnac.meapi.model.*;
import com.foreverwin.mesnac.meapi.service.NcCodeService;
import com.foreverwin.mesnac.meapi.service.NwaUserService;
import com.foreverwin.mesnac.meapi.service.RouterStepcontentService;
import com.foreverwin.mesnac.meapi.service.UserGroupService;
import com.foreverwin.mesnac.meapi.util.StringUtils;
import com.foreverwin.modular.core.util.CommonMethods;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.text.SimpleDateFormat;
import java.time.Duration;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

import static com.foreverwin.mesnac.anomaly.utils.DateReportUtils.getMonthRemainingDays;

/**
 * @Description TODO
 * @Author zhaojiawei
 * @Since 2021-08-23
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class ReportServiceImpl implements ReportService {

    @Autowired
    private NcCodeService ncCodeService;

    @Autowired
    private ReportMapper reportMapper;

    @Autowired
    private NwaUserService nwaUserService;

    @Autowired
    private DataFieldListMapper dataFieldListMapper;

    @Autowired
    private ItemMapper itemMapper;

    @Autowired
    private ShopOrderMapper shopOrderMapper;

    @Autowired
    private RouterStepcontentService routerStepcontentService;

    @Autowired
    private UserGroupService userGroupService;

    /**
     * 接口日志表
     *
     * @return
     */
    @Override
    public List<IntegrationLogDto> findIntegrationLog(Map<String, Object> map) {
        String site = CommonMethods.getSite();
        return reportMapper.findIntegrationLog(site, map);
    }

    /**
     * 零件加工报表
     *
     * @return
     */
    @Override
    public List<Map<String, Object>> partsManufacturing(Map<String, Object> map) {
        String site = CommonMethods.getSite();
        String locale = LocaleContextHolder.getLocale().getLanguage();
        map.put("site", site);
        map.put("locale", locale);
        List<Map<String, Object>> maps = reportMapper.partsManufacturing(map);
//        for(Map<String,Object> m: maps){
//            //判断子件是否全部已经完工，如果有一个没有完工，则全部都没有完工，设置为否
//            String isComplete = (String)m.get("ZI_JIAN");
//            if("否".equals(isComplete)){
//                for(Map<String,Object> m2: maps){
//                    m2.put("ZI_JIAN","否");
//                }
//                break;
//            }
//        }
        return maps;
    }

    /**
     * 余料退回明细表
     *
     * @param item
     * @param itemDescription
     * @return
     */
    @Override
    public List<Map<String, Object>> surplusReturnTable(String item, String itemDescription) {
        String locale = LocaleContextHolder.getLocale().getLanguage();
        String site = CommonMethods.getSite();
        return reportMapper.surplusReturnTable(site, locale, item, itemDescription);
    }

    /**
     * 资源维修报表
     *
     * @param map
     * @return
     */
    @Override
    public List<Map<String, Object>> resourceRepair(Map<String, Object> map) {
        String locale = LocaleContextHolder.getLocale().getLanguage();
        String site = CommonMethods.getSite();

        Map<String, List<NcCode>> ncCodeMap = new HashMap<>();
        NcCode ncCode = new NcCode();
        ncCode.setSite(site);
        List<NcCode> ncCodeList = ncCodeService.selectList(ncCode);
        if (ncCodeList != null && ncCodeList.size() > 0) {
            ncCodeMap = ncCodeList.stream().collect(Collectors.groupingBy(NcCode::getNcCode));
        }

        Map<String, List<NwaUser>> userMap = new HashMap<>();
        NwaUser nwaUser = new NwaUser();
        nwaUser.setSite(site);
        List<NwaUser> nwaUserList = nwaUserService.selectList(nwaUser);
        if (nwaUserList != null && nwaUserList.size() > 0) {
            userMap = nwaUserList.stream().collect(Collectors.groupingBy(NwaUser::getUserName));
        }

        List<Map<String, Object>> maps = reportMapper.resourceRepair(site, locale, map);


        for (int i = 0; i < maps.size(); i++) {
            Map<String, Object> temporaryMap = maps.get(i);
            String ncCodes = (String) temporaryMap.get("NC_CODE_NO");
            if (StringUtils.isBlank(ncCodes)) {
                maps.get(i).put("NC_CODE_DESCRIPTION", "");
                continue;
            }
            String[] ncCodeArray = ncCodes.split(",");
            String ncCodeDesc = "";
            for (int j = 0; j < ncCodeArray.length; j++) {
                if (ncCodeMap.get(ncCodeArray[j]) != null) {
                    ncCodeDesc += ncCodeMap.get(ncCodeArray[j]).get(0).getDescription() + ";";
                }
            }
            maps.get(i).put("NC_CODE_DESCRIPTION", ncCodeDesc);
        }


        for (int i = 0; i < maps.size(); i++) {
            Map<String, Object> temporaryMap = maps.get(i);
            String repairUser = (String) temporaryMap.get("REPAIR_USERS");
            if (StringUtils.isBlank(repairUser)) {
                maps.get(i).put("REPAIR_USERS_DESCRIPTION", "");
                continue;
            }
            String[] repairUserArray = repairUser.split(",");
            String repairUserDescription = "";
            for (int j = 0; j < repairUserArray.length; j++) {
                if (userMap.get(repairUserArray[j]) != null) {
                    repairUserDescription += userMap.get(repairUserArray[j]).get(0).getFullName() + ";";
                }
            }
            maps.get(i).put("REPAIR_USERS_DESCRIPTION", repairUserDescription);
        }
        return maps;
    }

    /**
     * 检验记录报表
     *
     * @param paramMap
     * @return
     */
    @Override
    public List<Map<String, Object>> findInspectionTask(Map paramMap) {
        return reportMapper.findInspectionTask(paramMap);
    }

    /**
     * 查询质检问题统计表
     *
     * @param paramMap
     * @return
     */
    @Override
    public List<Map<String, Object>> findQualityInspectionReport(Map paramMap) {
        String site = CommonMethods.getSite();
        paramMap.put("site", site);
        paramMap.put("locale", LocaleContextHolder.getLocale().getLanguage());
        List<Map<String, Object>> qualityInspectionReport = reportMapper.findQualityInspectionReport(paramMap);


        //原因分类
        Map<String, List<DataFieldList>> dataFieldListMap = new HashMap<>();
        DataField dataField = new DataField();
        dataField.setSite(site);
        dataField.setDataField("REASON_CATEGORY");
        List<DataFieldList> dataFieldList = dataFieldListMapper.findDataFieldList(dataField);
        if (dataFieldList != null && dataFieldList.size() > 0) {
            dataFieldListMap = dataFieldList.stream().collect(Collectors.groupingBy(DataFieldList::getDataValue));
        }
        for (int i = 0; i < qualityInspectionReport.size(); i++) {
            Map<String, Object> temporaryMap = qualityInspectionReport.get(i);
            String dutyCauseType = (String) temporaryMap.get("DUTY_CAUSE_TYPE");
            if (StringUtils.isBlank(dutyCauseType)) {
                qualityInspectionReport.get(i).put("DUTY_CAUSE_TYPE_DESCRIPTION", "");
                continue;
            }
            String[] dutyCauseTypeSplit = dutyCauseType.split(",");
            StringBuilder dutyCauseTypeDescription = new StringBuilder();
            for (int j = 0; j < dutyCauseTypeSplit.length; j++) {
                if (dataFieldListMap.get(dutyCauseTypeSplit[j]) != null) {
                    dutyCauseTypeDescription.append(dataFieldListMap.get(dutyCauseTypeSplit[j]).get(0).getDescription()).append(";");
                }
            }
            qualityInspectionReport.get(i).put("DUTY_CAUSE_TYPE_DESCRIPTION", dutyCauseTypeDescription);
        }

        //返修工序内容
        Map map = new HashMap();
        map.put("site", site);
        for (int i = 0; i < qualityInspectionReport.size(); i++) {
            Map<String, Object> temporaryMap = qualityInspectionReport.get(i);
            String abnormalBo = (String) temporaryMap.get("ABNORMAL_NO");
            String routerBo = (String) temporaryMap.get("ROUTER_BO");


            if(StringUtil.notBlank(routerBo)){
                //  map.put("abnormalBo", abnormalBo);
                map.put("routerBo", routerBo);
                List<Map<String, Object>> result = routerStepcontentService.selectDisRouterInfo(map);
                String ReapirContent = "";
                if (result != null && result.size() != 0) {
                    for (int j = 0; j < result.size(); j++) {
                        ReapirContent = ReapirContent + "【" + (String) result.get(j).get("STEP_ID") + "】" +
                                (String) result.get(j).get("ROUTER_STEP_CONTENT");
                    }
                }
                qualityInspectionReport.get(i).put("ROUTER_STEP_CONTENT", ReapirContent);
            }

        }


        //责任分类
        dataField.setDataField("RESPONSIBILITY_TYPE");
        dataFieldList = dataFieldListMapper.findDataFieldList(dataField);
        if (dataFieldList != null && dataFieldList.size() > 0) {
            dataFieldListMap = dataFieldList.stream().collect(Collectors.groupingBy(DataFieldList::getDataValue));
        }
        for (int i = 0; i < qualityInspectionReport.size(); i++) {
            Map<String, Object> temporaryMap = qualityInspectionReport.get(i);
            String dutyType = (String) temporaryMap.get("DUTY_TYPE");
            if (StringUtils.isBlank(dutyType)) {
                qualityInspectionReport.get(i).put("DUTY_TYPE_DESCRIPTION", "");
                continue;
            }
            String[] dutyTypeSplit = dutyType.split(",");
            StringBuilder dutyTypeDescription = new StringBuilder();
            for (int j = 0; j < dutyTypeSplit.length; j++) {
                if (dataFieldListMap.get(dutyTypeSplit[j]) != null) {
                    dutyTypeDescription.append(dataFieldListMap.get(dutyTypeSplit[j]).get(0).getDescription()).append(";");
                }
            }
            qualityInspectionReport.get(i).put("DUTY_TYPE_DESCRIPTION", dutyTypeDescription);
        }


        //责任部门
        dataField.setDataField("RESPONSIBILITY_DEPARTMENT");
        dataFieldList = dataFieldListMapper.findDataFieldList(dataField);
        if (dataFieldList != null && dataFieldList.size() > 0) {
            dataFieldListMap = dataFieldList.stream().collect(Collectors.groupingBy(DataFieldList::getDataValue));
        }
        for (int i = 0; i < qualityInspectionReport.size(); i++) {
            Map<String, Object> temporaryMap = qualityInspectionReport.get(i);
            String dutyDepart = (String) temporaryMap.get("DUTY_DEPART");
            if (StringUtils.isBlank(dutyDepart)) {
                qualityInspectionReport.get(i).put("DUTY_DEPART_DESCRIPTION", "");
                continue;
            }
            String[] dutyDepartSplit = dutyDepart.split(",");
            StringBuilder dutyDepartDescription = new StringBuilder();
            for (int j = 0; j < dutyDepartSplit.length; j++) {
                if (dataFieldListMap.get(dutyDepartSplit[j]) != null) {
                    dutyDepartDescription.append(dataFieldListMap.get(dutyDepartSplit[j]).get(0).getDescription()).append(";");
                }
            }
            qualityInspectionReport.get(i).put("DUTY_DEPART_DESCRIPTION", dutyDepartDescription);
        }

        return qualityInspectionReport;
    }

    @Override
    public List<Map<String, Object>> findAbnormalRecord(Map paramMap) {
        String site = CommonMethods.getSite();
        paramMap.put("site", site);
        paramMap.put("locale", LocaleContextHolder.getLocale().getLanguage());
        List<Map<String, Object>> findAbnormalRecord = reportMapper.findAbnormalRecord(paramMap);


        Map<String, List<DataFieldList>> dataFieldListMap = new HashMap<>();
        DataField dataField = new DataField();
        dataField.setSite(site);
        //责任部门
        dataField.setDataField("RESPONSIBILITY_DEPARTMENT");
        List<DataFieldList> dataFieldList = dataFieldListMapper.findDataFieldList(dataField);
        if (dataFieldList != null && dataFieldList.size() > 0) {
            dataFieldListMap = dataFieldList.stream().collect(Collectors.groupingBy(DataFieldList::getDataValue));
        }
        for (int i = 0; i < findAbnormalRecord.size(); i++) {
            Map<String, Object> temporaryMap = findAbnormalRecord.get(i);
            String dutyDepart = (String) temporaryMap.get("DUTY_DEPART");
            if (StringUtils.isBlank(dutyDepart)) {
                findAbnormalRecord.get(i).put("DUTY_DEPART_DESCRIPTION", "");
                continue;
            }
            String[] dutyDepartSplit = dutyDepart.split(",");
            StringBuilder dutyDepartDescription = new StringBuilder();
            for (int j = 0; j < dutyDepartSplit.length; j++) {
                if (dataFieldListMap.get(dutyDepartSplit[j]) != null) {
                    dutyDepartDescription.append(dataFieldListMap.get(dutyDepartSplit[j]).get(0).getDescription()).append(";");
                }
            }
            findAbnormalRecord.get(i).put("DUTY_DEPART_DESCRIPTION", dutyDepartDescription);
        }
        return findAbnormalRecord;
    }

    @Override
    public List<Map<String, Object>> shopOrderSchedule(Map paramMap) {
        String site = CommonMethods.getSite();
        paramMap.put("site", site);
        paramMap.put("locale", LocaleContextHolder.getLocale().getLanguage());
        return reportMapper.shopOrderSchedule(paramMap);
    }

    @Override
    public List<Map<String, Object>> findOperatorScheduleReport(Map paramMap) {
        String site = CommonMethods.getSite();
        paramMap.put("site", site);
        paramMap.put("locale", LocaleContextHolder.getLocale().getLanguage());
        List<Map<String, Object>> result = reportMapper.findOperatorScheduleReport(paramMap);
        for (Map<String, Object> map : result) {
            if (String.valueOf(map.get("STEP_ID")).substring(0,2).equals("90") && !map.containsKey("ABNORMAL_NO")){
                String sfc = String.valueOf(map.get("SFC"));
                String stepId = String.valueOf(map.get("STEP_ID"));
                //检验是否拆单
                Map<String, String> sfcAbnormalNoMap = reportMapper.findAbnormalDutyBySFCAndStepId(site,sfc,stepId);
                if (sfcAbnormalNoMap != null){
                    map.put("ABNORMAL_NO",sfcAbnormalNoMap.get("ABNORMAL_NO"));
                    map.put("DUTY_TYPE",sfcAbnormalNoMap.get("DUTY_TYPE"));
                    map.put("DUTY_DEPART",sfcAbnormalNoMap.get("DUTY_DEPART"));
                    map.put("PRINCIPAL_USER",sfcAbnormalNoMap.get("PRINCIPAL_USER"));
                    map.put("NC_CODE_DESCRIPTION",sfcAbnormalNoMap.get("NC_CODE_DESCRIPTION"));
                }
                String abnormalNo = String.valueOf(map.get("SHOP_ORDER"));
                if (abnormalNo.substring(0,1).equals("Q")){
                    Map<String, String> abnormalNoMap = reportMapper.findAbnormalDutyByAbnormalNo(site,abnormalNo);
                    if (abnormalNoMap != null){
                    map.put("ABNORMAL_NO",abnormalNoMap.get("ABNORMAL_NO"));
                    map.put("DUTY_TYPE",abnormalNoMap.get("DUTY_TYPE"));
                    map.put("DUTY_DEPART",abnormalNoMap.get("DUTY_DEPART"));
                    map.put("PRINCIPAL_USER",abnormalNoMap.get("PRINCIPAL_USER"));
                    map.put("NC_CODE_DESCRIPTION",abnormalNoMap.get("NC_CODE_DESCRIPTION"));
                    }
                }
            }
        }

        return result;
    }

    @Override
    public List<Map<String, Object>> findWorkOrderSchedule(Map paramMap) {
        String site = CommonMethods.getSite();
        paramMap.put("site", site);
        paramMap.put("locale", LocaleContextHolder.getLocale().getLanguage());
        return reportMapper.findWorkOrderSchedule(paramMap);
    }

    /**
     * 零件生产过程追溯报表
     *
     * @param paramMap
     * @return
     */
    @Override
    public List<Map<String, Object>> partsTraceScheduleReport(Map paramMap) {
        String site = CommonMethods.getSite();
        paramMap.put("site", site);
        paramMap.put("locale", LocaleContextHolder.getLocale().getLanguage());
        return reportMapper.partsTraceScheduleReport(paramMap);
    }

    /**
     * 原材料追溯报表
     *
     * @param paramMap
     * @return
     */
    @Override
    public List<Map<String, Object>> itemTraceScheduleReport(Map paramMap) {
        String site = CommonMethods.getSite();
        paramMap.put("site", site);
        paramMap.put("locale", LocaleContextHolder.getLocale().getLanguage());
        return reportMapper.itemTraceScheduleReport(paramMap);
    }

    /**
     * 资源任务明细报表
     *
     * @param paramMap
     * @return
     */
    @Override
    public List<Map<String, Object>> resourceTaskDetailReport(Map paramMap) {
        String site = CommonMethods.getSite();
        paramMap.put("site", site);
        paramMap.put("locale", LocaleContextHolder.getLocale().getLanguage());
        return reportMapper.resourceTaskDetailReport(paramMap);
    }

    /**
     * 项目进度报表
     *
     * @param paramMap
     * @return
     */
    @Override
    public Map<String, Object> ProjectScheduleReport(Map paramMap) {
        String site = CommonMethods.getSite();
        paramMap.put("site", site);
        paramMap.put("locale", LocaleContextHolder.getLocale().getLanguage());
        List<Map<String, Object>> maps = reportMapper.ProjectScheduleReport(paramMap);

        //根据item、workOrder、shopOrder进行多重分组存数据
        List<HashMap<String, Object>> clothingJSONObject = new ArrayList<>();
        Map<String, List<Map<String, Object>>> items = maps.stream().
                collect(Collectors.groupingBy(e -> e.get("item").toString()));
        for (String item : items.keySet()) {
            HashMap<String, Object> itemsMap = new HashMap<>();
            //项目号汇总
            List<HashMap<String, Object>> itemListJSONObject = new ArrayList<>();
            itemsMap.put("item", item);
            Integer shopOrderQTY = 0;
            Integer doneQTY = 0;
            Float rate = 0F;
            Integer nextOperationNum = 0;
            String endPlanTime = String.valueOf(items.get(item).get(0).get("endPlanTime"));
            for (Map<String, Object> map : items.get(item)) {
                shopOrderQTY +=  Integer.valueOf(String.valueOf(map.get("shopOrderQTY")));
                doneQTY += Integer.valueOf(String.valueOf(map.get("doneQTY")));
                nextOperationNum += Integer.valueOf(String.valueOf(map.get("nextOperationNum")));
                rate += Float.valueOf(String.valueOf(map.get("rate")).substring(0,String.valueOf(map.get("rate")).length() - 1));
                if (endPlanTime.compareTo(String.valueOf(map.get("endPlanTime"))) < 0){
                    endPlanTime = String.valueOf(map.get("endPlanTime"));
                }
            }
            if (!StringUtils.isBlank(String.valueOf(paramMap.get("finished"))) && "false".equals(String.valueOf(paramMap.get("finished")))){
                if ((rate / items.get(item).size()) == 100.0){
                    continue;
                }
            }else if (!StringUtils.isBlank(String.valueOf(paramMap.get("finished"))) && "true".equals(String.valueOf(paramMap.get("finished")))){
                if ((rate / items.get(item).size()) != 100.0){
                    continue;
                }
            }
            itemsMap.put("shopOrderQTY", shopOrderQTY);
            itemsMap.put("doneQTY", doneQTY);
            itemsMap.put("nextOperationNum", nextOperationNum);
            itemsMap.put("rate", String.format("%.2f",rate / items.get(item).size()) + "%");

            itemsMap.put("endPlanTime", endPlanTime.substring(0,endPlanTime.length() - 2));

            List<Map<String, Object>> itemsList = items.get(item);
            Map<String, List<Map<String, Object>>> workOrders = itemsList.stream().
                    collect(Collectors.groupingBy(e -> e.get("workOrder").toString()));


            for (String workOrder : workOrders.keySet()) {
                HashMap<String, Object> workOrderMap = new HashMap<>();
                List<HashMap<String, Object>> workOrderListJSONObject = new ArrayList<>();
                workOrderMap.put("workOrder", workOrder);
                //工作令汇总
                Integer shopOrderQTY1 = 0;
                Integer doneQTY1 = 0;
                Float rate1 = 0F;
                Integer nextOperationNum1 = 0;
                String endPlanTime1 = String.valueOf(workOrders.get(workOrder).get(0).get("endPlanTime"));

                for (Map<String, Object> map : workOrders.get(workOrder)) {
                    shopOrderQTY1 +=  Integer.valueOf(String.valueOf(map.get("shopOrderQTY")));
                    doneQTY1 += Integer.valueOf(String.valueOf(map.get("doneQTY")));
                    nextOperationNum1 += Integer.valueOf(String.valueOf(map.get("nextOperationNum")));
                    rate1 += Float.valueOf(String.valueOf(map.get("rate")).substring(0,String.valueOf(map.get("rate")).length() - 1));
                    if (endPlanTime1.compareTo(String.valueOf(map.get("endPlanTime"))) < 0){
                        endPlanTime1 = String.valueOf(map.get("endPlanTime"));
                    }
                }
                workOrderMap.put("shopOrderQTY", shopOrderQTY1);
                workOrderMap.put("doneQTY", doneQTY1);
                workOrderMap.put("nextOperationNum", nextOperationNum1);
                workOrderMap.put("rate", String.format("%.2f",rate1 / workOrders.get(workOrder).size()) + "%");
                workOrderMap.put("endPlanTime", endPlanTime1.substring(0,endPlanTime1.length() - 2));

                List<Map<String, Object>> workOrderList = workOrders.get(workOrder);
                /*Map<String, List<Map<String, Object>>> shopOrders = workOrderList.stream()
                        .collect(Collectors.groupingBy(e -> e.get("shopOrder").toString(),
                                CollectionUtil.toSortedList(Comparator.comparing(e ->
                                        Float.valueOf(e.get("rate").toString().substring(0,e.get("rate").toString().length() - 1) )))));*/
                Map<String, List<Map<String, Object>>> shopOrders = workOrderList.stream()
                        .collect(Collectors.groupingBy(e -> e.get("shopOrder").toString()));
                for (String shopOrder : shopOrders.keySet()) {
                    HashMap<String, Object> shopOrderMap = new HashMap<>();
                    shopOrderMap.put("shopOrder", shopOrder);

                    List<JSONObject> shopOrderListJSONObject = new ArrayList<>();
                    List<Map<String, Object>> shopOrderMaps = shopOrders.get(shopOrder);
                    //存入订单号对应数据
                    for (Map<String, Object> shopOrderMap1 : shopOrderMaps) {
                        JSONObject map1 = new JSONObject();
                        map1.put("SFC", shopOrderMap1.get("SFC"));
                        map1.put("itemCode", shopOrderMap1.get("itemCode"));
                        map1.put("itemDescription", shopOrderMap1.get("itemDescription"));
                        map1.put("shopOrderQTY", shopOrderMap1.get("shopOrderQTY"));
                        map1.put("doneQTY", shopOrderMap1.get("doneQTY"));
                        map1.put("rate", shopOrderMap1.get("rate"));
                        map1.put("currentOperation", shopOrderMap1.get("currentOperation"));
                        map1.put("nextOperationNum", shopOrderMap1.get("nextOperationNum"));
                        map1.put("endPlanTime", shopOrderMap1.get("endPlanTime"));
                        map1.put("currentTimeoutDay", shopOrderMap1.get("currentTimeoutDay"));
                        map1.put("estimationRange", shopOrderMap1.get("estimationRange"));
                        map1.put("isOverTime", shopOrderMap1.get("isOverTime"));
                        map1.put("rework", shopOrderMap1.get("rework"));
                        if (String.valueOf(shopOrderMap1.get("isOverTime")).equals("true")){
                            map1.put("Status", "Error");
                        }else if (String.valueOf(shopOrderMap1.get("isOverTime")).equals("false")){
                            map1.put("Status", "Success");
                        }

                        if (shopOrderMap1.get("isOverTime").equals("true")) {
                            map1.put("currentPlanTime", shopOrderMap1.get("currentPlanTime") + "(已超时)");
                        } else {
                            map1.put("currentPlanTime", shopOrderMap1.get("currentPlanTime"));
                        }
                       /* if (shopOrderMap1.get("rework").equals("true")) {
                            Map<String, Object> timeMap = reportMapper.selectCurrentPlanTime(shopOrderMap1);
                            map1.put("currentPlanTime", timeMap.get("planEndCompDate"));
                            if (shopOrderMap1.get("isOverTime").equals("true")){
                                map1.put("currentPlanTime", timeMap.get("planEndCompDate")+ "(已超时)");
                            }
                        }*/
                        shopOrderListJSONObject.add(map1);
                    }
                    shopOrderMap.put("categories", shopOrderListJSONObject);
                    workOrderListJSONObject.add(shopOrderMap);
                }
                workOrderMap.put("categories", workOrderListJSONObject);
                itemListJSONObject.add(workOrderMap);
            }
            itemsMap.put("categories", itemListJSONObject);
            clothingJSONObject.add(itemsMap);

        }
        HashMap<String, Object> objectHashMap = new HashMap<>();
        /*List<HashMap<String, Object>> clothingJSONObjectNew = new ArrayList<>();
        clothingJSONObjectNew.addAll(clothingJSONObject);
        if (!StringUtils.isBlank(String.valueOf(paramMap.get("finished"))) && "false".equals(String.valueOf(paramMap.get("finished")))){
        for (int i = 0; i < clothingJSONObject.size(); i++) {
            if ("100.00%".equals(clothingJSONObject.get(i).get("rate"))){
                clothingJSONObjectNew.remove(i);
            }
        }
        }*/

        objectHashMap.put("categories", clothingJSONObject);
        HashMap<String, Object> result = new HashMap<>();
        result.put("clothing", objectHashMap);
        result.put("workOrder", paramMap.get("workOrder"));
        result.put("oldData", maps);
        return result;
    }

    @Override
    public Map<String, Object> findUserGroupList(Map paramMap) {
        String site = CommonMethods.getSite();
        //查询用户对应用户组
        String user = CommonMethods.getUser();
        String userBo = HandleEnum.USER.getHandle(site, user);
        List<UserGroup> userGroupList = userGroupService.getUserGroupListByUserBo(userBo);
        HashMap<String, Object> result = new HashMap<>();
        result.put("userGroupList", userGroupList);
        return result;
    }

    /**
     * 质量返修报表
     * @param paramMap
     * @return
     */
    @Override
    public List<Map<String, Object>> qualityRepairReport(Map paramMap) {
        String site = CommonMethods.getSite();
        paramMap.put("site", site);
        paramMap.put("locale", LocaleContextHolder.getLocale().getLanguage());
        return reportMapper.qualityRepairReport(paramMap);
    }

    /**
     * 资源负荷率报表(周维度)
     * @param paramMap
     * @return
     */
    @Override
    public List<Map<String, String>> weekLoadRateReport(Map paramMap) {
        String site = CommonMethods.getSite();
        paramMap.put("site", site);
        paramMap.put("locale", LocaleContextHolder.getLocale().getLanguage());
        if (String.valueOf(paramMap.get("week")).equals("true")){
            LocalDateTime now = LocalDateTime.now();
            DateTimeFormatter fmt = DateTimeFormatter.ofPattern("yyyy-MM-dd");
            int dayOfWeek = now.getDayOfWeek().getValue();
            LocalDateTime weekStart = now.minusDays(dayOfWeek - 1).with(LocalTime.MIN);
            LocalDateTime weekEnd = now.plusDays(7 * 8 - dayOfWeek).with(LocalTime.MAX);
            List<Map<String, String>> weekOfDate = DateReportUtils.getWeekOfDate3(weekStart.format(fmt), weekEnd.format(fmt));
            return weekOfDate;
        }
        List<Map<String, String>> resultList = reportMapper.weekLoadRateReport(paramMap);
        HashMap<String, String> countMap = new HashMap<>();
        countMap.put("workCenterDescription","总计:");
        Float tardinessWork = 0.00f;
        Float weekCapacity = 0.00f;
        Float unWorkHours1 = 0.00f;
        Float unWorkHours2 = 0.00f;
        Float unWorkHours3 = 0.00f;
        Float unWorkHours4 = 0.00f;
        Float unWorkHours5 = 0.00f;
        Float unWorkHours6 = 0.00f;
        Float unWorkHours7 = 0.00f;
        Float unWorkHours8 = 0.00f;

        for (Map<String, String> map : resultList) {
            tardinessWork += Float.valueOf(String.valueOf(map.get("tardinessWork")));
            if (!StringUtils.isBlank(map.get("weekCapacity"))){
                weekCapacity += Float.valueOf(String.valueOf(map.get("weekCapacity")));
            }
            unWorkHours1 += Float.valueOf(String.valueOf(map.get("unWorkHours1")));
            unWorkHours2 += Float.valueOf(String.valueOf(map.get("unWorkHours2")));
            unWorkHours3 += Float.valueOf(String.valueOf(map.get("unWorkHours3")));
            unWorkHours4 += Float.valueOf(String.valueOf(map.get("unWorkHours4")));
            unWorkHours5 += Float.valueOf(String.valueOf(map.get("unWorkHours5")));
            unWorkHours6 += Float.valueOf(String.valueOf(map.get("unWorkHours6")));
            unWorkHours7 += Float.valueOf(String.valueOf(map.get("unWorkHours7")));
            unWorkHours8 += Float.valueOf(String.valueOf(map.get("unWorkHours8")));

        }
        countMap.put("tardinessWork",String.format("%.2f",tardinessWork));
        countMap.put("weekCapacity",String.format("%.2f",weekCapacity));
        countMap.put("unWorkHours1",String.format("%.2f",unWorkHours1));
        countMap.put("unWorkHours2",String.format("%.2f",unWorkHours2));
        countMap.put("unWorkHours3",String.format("%.2f",unWorkHours3));
        countMap.put("unWorkHours4",String.format("%.2f",unWorkHours4));
        countMap.put("unWorkHours5",String.format("%.2f",unWorkHours5));
        countMap.put("unWorkHours6",String.format("%.2f",unWorkHours6));
        countMap.put("unWorkHours7",String.format("%.2f",unWorkHours7));
        countMap.put("unWorkHours8",String.format("%.2f",unWorkHours8));
        countMap.put("weekRate1",String.format("%.2f",unWorkHours1 / weekCapacity * 100 ) + "%");
        countMap.put("weekRate2",String.format("%.2f",unWorkHours2 / weekCapacity * 100 ) + "%");
        countMap.put("weekRate3",String.format("%.2f",unWorkHours3 / weekCapacity * 100 ) + "%");
        countMap.put("weekRate4",String.format("%.2f",unWorkHours4 / weekCapacity * 100 ) + "%");
        countMap.put("weekRate5",String.format("%.2f",unWorkHours5 / weekCapacity * 100 ) + "%");
        countMap.put("weekRate6",String.format("%.2f",unWorkHours6 / weekCapacity * 100 ) + "%");
        countMap.put("weekRate7",String.format("%.2f",unWorkHours7 / weekCapacity * 100 ) + "%");
        countMap.put("weekRate8",String.format("%.2f",unWorkHours8 / weekCapacity * 100 ) + "%");
        resultList.add(countMap);

        return resultList;

    }

    /**
     * 资源负荷率报表(月维度)
     * @param paramMap
     * @return
     */
    @Override
    public List<Map<String, String>> monthLoadRateReport(Map paramMap) {
        String site = CommonMethods.getSite();
        paramMap.put("site", site);
        paramMap.put("locale", LocaleContextHolder.getLocale().getLanguage());
        if (!StringUtils.isBlank(String.valueOf(paramMap.get("month")))){
            paramMap.put("month",Integer.valueOf(String.valueOf(paramMap.get("month"))));
        }
        int month = Calendar.getInstance().get(Calendar.MONTH) + 1;
        List<Map<String, String>> result = reportMapper.monthLoadRateReport(paramMap);
        Integer monthRemainingDays = getMonthRemainingDays();
        for (Map<String, String> map : result) {
            Float number =  Float.valueOf(String.valueOf(map.get("workHours" + month)))
                    + Float.valueOf(String.valueOf(map.get("tardinessWork")))
                    - Float.valueOf(String.valueOf(map.get("monthToNow")));
            map.put("workHours" + month, String.format("%.2f",number));
            if ((map.containsKey("dayCapacity"))){
                Float dayCapacity= (Float.valueOf(String.valueOf(map.get("dayCapacity")))) * monthRemainingDays;
                map.put("monthRate" + month, String.format("%.2f",(number / dayCapacity) * 100) + "%");
            }else {
                map.put("monthRate" + month, "0.00%");
            }
        }
        HashMap<String, String> countMap = new HashMap<>();
        countMap.put("workCenterDescription","总计:");
        Float monthCapacity = 0.00f;
        Float dayCapacity = 0.00f;
        Float tardinessWork = 0.00f;
        Float workHours1 = 0.00f;
        Float workHours2 = 0.00f;
        Float workHours3 = 0.00f;
        Float workHours4 = 0.00f;
        Float workHours5 = 0.00f;
        Float workHours6 = 0.00f;
        Float workHours7 = 0.00f;
        Float workHours8 = 0.00f;
        Float workHours9 = 0.00f;
        Float workHours10 = 0.00f;
        Float workHours11= 0.00f;
        Float workHours12 = 0.00f;
        Float currentCompletedWorkHours = 0.00f;
        Float currentPlanWorkHours = 0.00f;
        for (Map<String, String> map : result) {
            tardinessWork += Float.valueOf(String.valueOf(map.get("tardinessWork")));
            if (!StringUtils.isBlank(map.get("monthCapacity"))){
                monthCapacity += Float.valueOf(String.valueOf(map.get("monthCapacity")));
            }
            if (!StringUtils.isBlank(map.get("dayCapacity"))){
                dayCapacity += Float.valueOf(String.valueOf(map.get("dayCapacity")));
            }
            workHours1 += Float.valueOf(String.valueOf(map.get("workHours1")));
            workHours2 += Float.valueOf(String.valueOf(map.get("workHours2")));
            workHours3 += Float.valueOf(String.valueOf(map.get("workHours3")));
            workHours4 += Float.valueOf(String.valueOf(map.get("workHours4")));
            workHours5 += Float.valueOf(String.valueOf(map.get("workHours5")));
            workHours6 += Float.valueOf(String.valueOf(map.get("workHours6")));
            workHours7 += Float.valueOf(String.valueOf(map.get("workHours7")));
            workHours8 += Float.valueOf(String.valueOf(map.get("workHours8")));
            workHours9 += Float.valueOf(String.valueOf(map.get("workHours9")));
            workHours10 += Float.valueOf(String.valueOf(map.get("workHours10")));
            workHours11 += Float.valueOf(String.valueOf(map.get("workHours11")));
            workHours12 += Float.valueOf(String.valueOf(map.get("workHours12")));
            currentCompletedWorkHours += Float.valueOf(String.valueOf(map.get("currentCompletedWorkHours")));
            currentPlanWorkHours += Float.valueOf(String.valueOf(map.get("currentPlanWorkHours")));
        }
        countMap.put("tardinessWork",String.format("%.2f",tardinessWork));
        countMap.put("monthCapacity",String.format("%.2f",monthCapacity));
        countMap.put("dayCapacity",String.format("%.2f",dayCapacity));
        countMap.put("workHours1",String.format("%.2f",workHours1));
        countMap.put("workHours2",String.format("%.2f",workHours2));
        countMap.put("workHours3",String.format("%.2f",workHours3));
        countMap.put("workHours4",String.format("%.2f",workHours4));
        countMap.put("workHours5",String.format("%.2f",workHours5));
        countMap.put("workHours6",String.format("%.2f",workHours6));
        countMap.put("workHours7",String.format("%.2f",workHours7));
        countMap.put("workHours8",String.format("%.2f",workHours8));
        countMap.put("workHours9",String.format("%.2f",workHours9));
        countMap.put("workHours10",String.format("%.2f",workHours10));
        countMap.put("workHours11",String.format("%.2f",workHours11));
        countMap.put("workHours12",String.format("%.2f",workHours12));
        countMap.put("monthRate1",String.format("%.2f",workHours1 /  monthCapacity  * 100 )+ "%");
        countMap.put("monthRate2",String.format("%.2f",workHours2 /  monthCapacity  * 100 )+ "%");
        countMap.put("monthRate3",String.format("%.2f",workHours3 /  monthCapacity  * 100 )+ "%");
        countMap.put("monthRate4",String.format("%.2f",workHours4 /  monthCapacity  * 100 )+ "%");
        countMap.put("monthRate5",String.format("%.2f",workHours5 /  monthCapacity  * 100 )+ "%");
        countMap.put("monthRate6",String.format("%.2f",workHours6 /  monthCapacity  * 100 )+ "%");
        countMap.put("monthRate7",String.format("%.2f",workHours7 /  monthCapacity  * 100 )+ "%");
        countMap.put("monthRate8",String.format("%.2f",workHours8 /  monthCapacity  * 100 )+ "%");
        countMap.put("monthRate9",String.format("%.2f",workHours9 /  monthCapacity  * 100 )+ "%");
        countMap.put("monthRate10",String.format("%.2f",workHours10 / monthCapacity * 100 )  + "%");
        countMap.put("monthRate11",String.format("%.2f",workHours11 / monthCapacity * 100 )  + "%");
        countMap.put("monthRate12",String.format("%.2f",workHours12 / monthCapacity * 100 )  + "%");
        countMap.put("currentCompletedWorkHours",String.format("%.2f",currentCompletedWorkHours));
        countMap.put("currentCompletedRate",String.format("%.2f",currentCompletedWorkHours / monthCapacity * 100 )  + "%");
        countMap.put("currentPlanWorkHours",String.format("%.2f",currentPlanWorkHours));
        countMap.put("monthRate" + month,String.format("%.2f",(Float.valueOf(String.valueOf(countMap.get("workHours" + month))) /
                (monthRemainingDays * dayCapacity)) * 100 )  + "%");
        result.add(countMap);
        return result;
    }

    /**
     * 交付及时率报表(周维度)
     * @param paramMap
     * @return
     */
    @Override
    public  List<Map<String, Object>> weekDeliveryRateReport(Map paramMap) {
        String site = CommonMethods.getSite();
        paramMap.put("site", site);
        paramMap.put("locale", LocaleContextHolder.getLocale().getLanguage());
        HashMap<String, List<Map<String, String>>> result = new HashMap<>();

/*        QueryWrapper<Object> wrapper = new QueryWrapper<>();
        //columnData
        ArrayList<Map<String,String>> columnDataList = new ArrayList<>();
        HashMap<String, String> columnMap0 = new LinkedHashMap<>();
        columnMap0.put("colName","序号");
        columnMap0.put("width","30");
        columnDataList.add(columnMap0);
        HashMap<String, String> columnMap1 = new LinkedHashMap<>();
        columnMap1.put("colName","车间描述");
        columnDataList.add(columnMap1);
        HashMap<String, String> columnMap2 = new LinkedHashMap<>();
        columnMap2.put("colName","资源编号");
        columnDataList.add(columnMap2);
        HashMap<String, String> columnMap3 = new LinkedHashMap<>();
        columnMap3.put("colName","工序描述");
        columnDataList.add(columnMap3);
        HashMap<String, String> columnMap4 = new LinkedHashMap<>();
        columnMap4.put("colName","资源描述");
        columnDataList.add(columnMap4);
        List<Map<String, String>> weekOfDate = DateReportUtils.getWeekOfDate(String.valueOf(paramMap.get("startDate")), String.valueOf(paramMap.get("endDate")));

        //rowData
        ArrayList<Map<String,String>> rowDataList = new ArrayList<>();
        int week = 1;
        for (Map<String, String> weekOfDateMap : weekOfDate) {
            HashMap<String, String> columnMap5 = new LinkedHashMap<>();
            columnMap5.put("colName",week + "计划数量");
            columnDataList.add(columnMap5);
            HashMap<String, String> columnMap6 = new LinkedHashMap<>();
            columnMap6.put("colName",week + "按期完工数量");
            columnDataList.add(columnMap6);
            HashMap<String, String> columnMap7 = new LinkedHashMap<>();
            columnMap7.put("colName",week + "零件交付及时率");
            columnDataList.add(columnMap7);
            HashMap<String, String> columnMap8 = new LinkedHashMap<>();
            columnMap8.put("colName",week + "工时量");
            columnDataList.add(columnMap8);
            HashMap<String, String> columnMap9 = new LinkedHashMap<>();
            columnMap9.put("colName",week + "按期完工工时");
            columnDataList.add(columnMap9);
            HashMap<String, String> columnMap10 = new LinkedHashMap<>();
            columnMap10.put("colName",week + "工时交付及时率");
            columnDataList.add(columnMap10);



            paramMap.put("startDate",weekOfDateMap.get("start"));
            paramMap.put("endDate",weekOfDateMap.get("end"));
            List<Map<String, String>> weekList = reportMapper.weekDeliveryRateReport(paramMap);

            int i = 1;
            for (Map<String, String> weekResultMap : weekList) {
                HashMap<String, String> rowMap = new LinkedHashMap<>();
                rowMap.put("序号",String.valueOf(i));
                rowMap.put("车间描述",weekResultMap.get("workCenter"));
                rowMap.put("资源编号",weekResultMap.get("resrceCode"));
                rowMap.put("工序描述",weekResultMap.get("operationDescription"));
                rowMap.put("资源描述",weekResultMap.get("resrceDescription"));
                rowMap.put(week + "计划数量",weekResultMap.get("sumQTY"));
                rowMap.put(week + "按期完工数量",weekResultMap.get("sumDoneQTY"));
                rowMap.put(week + "零件交付及时率",weekResultMap.get("QTYrate"));
                rowMap.put(week + "工时量",weekResultMap.get("sumHours"));
                rowMap.put(week + "按期完工工时",weekResultMap.get("sumDoneHours"));
                rowMap.put(week + "工时交付及时率",weekResultMap.get("HOURSrate"));

                i++;
                rowDataList.add(rowMap);
            }
            week++;
        }

        result.put("columnData",columnDataList);
        result.put("rowData",rowDataList);*/
        List<Map<String, Object>> resultList = reportMapper.weekDeliveryRateReport(paramMap);
        HashMap<String, Object> countMap = new HashMap<>();
        countMap.put("workCenter","总计:");
        Float sumQTY = 0.00f;
        Float sumDoneQTY = 0.00f;
        Float QTYrate = 0.00f;
        Float sumHours = 0.00f;
        Float sumDoneHours = 0.00f;
        Float HOURSrate = 0.00f;
        for (Map<String, Object> map : resultList) {
            sumQTY += Float.valueOf(String.valueOf(map.get("sumQTY")));
            sumDoneQTY += Float.valueOf(String.valueOf(map.get("sumDoneQTY")));
            QTYrate += Float.valueOf(String.valueOf(map.get("QTYrate")).replace("%",""));
            sumHours += Float.valueOf(String.valueOf(map.get("sumHours")));
            sumDoneHours += Float.valueOf(String.valueOf(map.get("sumDoneHours")));
            HOURSrate += Float.valueOf(String.valueOf(map.get("HOURSrate")).replace("%",""));

        }
        countMap.put("sumQTY",String.format("%.2f",sumQTY));
        countMap.put("sumDoneQTY",String.format("%.2f",sumDoneQTY));
        countMap.put("QTYrate",String.format("%.2f",sumDoneQTY / sumQTY * 100) + "%");
        countMap.put("sumHours",String.format("%.2f",sumHours));
        countMap.put("sumDoneHours",String.format("%.2f",sumDoneHours));
        countMap.put("HOURSrate",String.format("%.2f",sumDoneHours / sumHours * 100) + "%");
        resultList.add(countMap);
        return resultList;
    }

    /**
     * 交付及时率报表(月维度)
     * @param paramMap
     * @return
     */
    @Override
    public List<Map<String, Object>> monthDeliveryRateReport(Map paramMap) {
        String site = CommonMethods.getSite();
        paramMap.put("site", site);
        paramMap.put("locale", LocaleContextHolder.getLocale().getLanguage());

        List<Map<String, Object>> resultList = reportMapper.monthDeliveryRateReport(paramMap);
        HashMap<String, Object> countMap = new HashMap<>();
        countMap.put("workCenter","总计:");
        Float sumQTY = 0.00f;
        Float sumDoneQTY = 0.00f;
        Float QTYrate = 0.00f;
        Float sumHours = 0.00f;
        Float sumDoneHours = 0.00f;
        Float HOURSrate = 0.00f;
        for (Map<String, Object> map : resultList) {
            sumQTY += Float.valueOf(String.valueOf(map.get("sumQTY")));
            sumDoneQTY += Float.valueOf(String.valueOf(map.get("sumDoneQTY")));
            QTYrate += Float.valueOf(String.valueOf(map.get("QTYrate")).replace("%",""));
            sumHours += Float.valueOf(String.valueOf(map.get("sumHours")));
            sumDoneHours += Float.valueOf(String.valueOf(map.get("sumDoneHours")));
            HOURSrate += Float.valueOf(String.valueOf(map.get("HOURSrate")).replace("%",""));

        }
        countMap.put("sumQTY",String.format("%.2f",sumQTY));
        countMap.put("sumDoneQTY",String.format("%.2f",sumDoneQTY));
        countMap.put("QTYrate",String.format("%.2f",sumDoneQTY / sumQTY * 100) + "%");
        countMap.put("sumHours",String.format("%.2f",sumHours));
        countMap.put("sumDoneHours",String.format("%.2f",sumDoneHours));
        countMap.put("HOURSrate",String.format("%.2f",sumDoneHours / sumHours * 100) + "%");

        resultList.add(countMap);
        return resultList;
    }

    /**
     * 项目生产加工周期报表
     * @param paramMap
     * @return
     */
    @Override
    public List<Map<String, Object>> itemProductProgressReport(Map paramMap) {
        String site = CommonMethods.getSite();
        paramMap.put("site", site);
        paramMap.put("locale", LocaleContextHolder.getLocale().getLanguage());
        List<Map<String, Object>> mapList = reportMapper.itemProductProgressReport(paramMap);

        for (Map<String, Object> objectMap : mapList) {
            String workOrder = String.valueOf(objectMap.get("workOrder"));
            if (workOrder.indexOf("/") != -1){
                if (workOrder.split("/").length > 2){
                    objectMap.put("career",workOrder.split("/")[1]);
                }else {
                    objectMap.put("career","");
                }
            }else {
                objectMap.put("career","");
            }
        }
        return mapList;
    }

    /**
     * 质量检验响应时间报表
     * @param paramMap
     * @return
     */
    @Override
    public List<Map<String, Object>> qualityInspectionResponseReport(Map paramMap) {
        String site = CommonMethods.getSite();
        paramMap.put("site", site);
        paramMap.put("locale", LocaleContextHolder.getLocale().getLanguage());
        return reportMapper.qualityInspectionResponseReport(paramMap);
    }

    /**
     * 项目生产加工周期看板
     * @param userGroup
     * @return
     */
    @Override
    public Map<String, Object> projectPlanningProgress(String userGroup) {
        List<Map<String, String>> abnormalQualityList = reportMapper.projectPlanningProgress(userGroup);
        Map<String, Object> result = new HashMap<>();
        Map<String, List<Map<String, String>>> workOrderMap = abnormalQualityList.stream()
                .collect(Collectors.groupingBy(e -> {
                    String career =e.get("workOrder");
                    try {
                        if (career.indexOf("硫化") != -1) {
                            return "硫化";
                        }else if (career.indexOf("全钢") != -1) {
                            return "全钢";
                        }else if (career.indexOf("半钢") != -1) {
                            return "半钢";
                        }else if (career.indexOf("鼓") != -1) {
                            return "鼓";
                        }else if (career.indexOf("裁断") != -1) {
                            return "裁断";
                        }else if (career.indexOf("检测") != -1) {
                            return "检测";
                        }else if (career.indexOf("橡塑") != -1) {
                            return "橡塑";
                        } else if (career.indexOf("非公路") != -1) {
                            return "非公路";
                        }else if (career.indexOf("研究院") != -1) {
                            return "研究院";
                        }else if (career.indexOf("外部") != -1) {
                            return "外部";
                        }
                    } catch (Exception exception) {
                        exception.printStackTrace();
                    }
                    return "";
                }));

        for (String workOrder : workOrderMap.keySet()) {
            HashMap<String, Object> resultMap = new HashMap<>();
            ArrayList<String> items = new ArrayList<>();
            ArrayList<String> shopOrderQTYs = new ArrayList<>();
            ArrayList<String> doneQTY = new ArrayList<>();
            ArrayList<String> rate = new ArrayList<>();
            ArrayList<String> workOrderList = new ArrayList<>();
            ArrayList<String> isLongTime = new ArrayList<>();
            List<Map<String, String>> maps = workOrderMap.get(workOrder);
            maps.forEach(e -> {
                items.add(e.get("item"));
                shopOrderQTYs.add(e.get("shopOrderQTY"));
                doneQTY.add(e.get("doneQTY"));
                rate.add(e.get("rate"));
                workOrderList.add(e.get("workOrder"));
                isLongTime.add(e.get("isLongTime"));
            });
            resultMap.put("items",items);
            resultMap.put("shopOrderQTY",shopOrderQTYs);
            resultMap.put("doneQTY",doneQTY);
            resultMap.put("rate",rate);
            resultMap.put("workOrder",workOrderList);
            resultMap.put("isLongTime",isLongTime);
            result.put(workOrder,resultMap);
        }
        return result;
    }

    /**
     * 设备产能负荷看板
     * @param workCenter
     * @return
     */
    @Override
    public Map<String, Object> capacityLoad(String workCenter) {
        HashMap<String, Object> result = new HashMap<>();

        List<Map<String, String>> capacityLoadList = null;
        try {
            if (workCenter != null){
                capacityLoadList = reportMapper.capacityLoadAllInfo(workCenter);
            }else {
                capacityLoadList = reportMapper.capacityLoad(workCenter);
            }
            result.put("capacityLoadList",capacityLoadList);
            Float planCapacity = 0f;
            Float standardCapacity = 0f;
            Float standardMonthCapacity = 0f;
            ArrayList<HashMap<String, Object>> eightList = new ArrayList<>();
            ArrayList<Float> planHoursList = new ArrayList<>();
            ArrayList<Float> planUndoneHoursList = new ArrayList<>();
            for (Map<String, String> map : capacityLoadList) {
                HashMap<String, Object> resourceMap = new HashMap<>();
                //计划工时
                ArrayList<Float> planList = new ArrayList<>();
                planList.add(Float.valueOf(String.valueOf(map.get("tardinessWork"))));
                planList.add(Float.valueOf(String.valueOf(map.get("planCapacity1"))));
                planList.add(Float.valueOf(String.valueOf(map.get("planCapacity2"))));
                planList.add(Float.valueOf(String.valueOf(map.get("planCapacity3"))));
                planList.add(Float.valueOf(String.valueOf(map.get("planCapacity4"))));
                planList.add(Float.valueOf(String.valueOf(map.get("planCapacity5"))));
                planList.add(Float.valueOf(String.valueOf(map.get("planCapacity6"))));
                planList.add(Float.valueOf(String.valueOf(map.get("planCapacity7"))));
//                planList.add(Float.valueOf(String.valueOf(map.get("planCapacity8"))));
                resourceMap.put("planCapacityList",planList);
                //标准产能
                resourceMap.put("standardWeekCapacity",map.get("standardWeekCapacity"));
                //未完成工时
                //resourceMap.put("tardinessWork",map.get("tardinessWork"));
                //设备描述
                resourceMap.put("resourceDescription",map.get("resourceDescription"));
                //总标准产能、总计划产能
                for (Float et : planList) {
                    planCapacity += Float.parseFloat(String.valueOf(et));
                }
                standardCapacity += Float.valueOf(String.valueOf(map.get("standardWeekCapacity")));
                standardMonthCapacity += Float.valueOf(String.valueOf(map.get("standardMonthCapacity")));
                //比率  计划工时 / 标准产能 * 100
                ArrayList<String> rateList = new ArrayList<>();
                rateList.add("0.00");
                rateList.add(String.format("%.2f", Float.valueOf(String.valueOf(map.get("planCapacity1"))) / Float.valueOf(String.valueOf(Float.valueOf(map.get("standardWeekCapacity")))) * 100));
                rateList.add(String.format("%.2f", Float.valueOf(String.valueOf(map.get("planCapacity2"))) / Float.valueOf(String.valueOf(Float.valueOf(map.get("standardWeekCapacity")))) * 100));
                rateList.add(String.format("%.2f", Float.valueOf(String.valueOf(map.get("planCapacity3"))) / Float.valueOf(String.valueOf(Float.valueOf(map.get("standardWeekCapacity")))) * 100));
                rateList.add(String.format("%.2f", Float.valueOf(String.valueOf(map.get("planCapacity4"))) / Float.valueOf(String.valueOf(Float.valueOf(map.get("standardWeekCapacity")))) * 100));
                rateList.add(String.format("%.2f", Float.valueOf(String.valueOf(map.get("planCapacity5"))) / Float.valueOf(String.valueOf(Float.valueOf(map.get("standardWeekCapacity")))) * 100));
                rateList.add(String.format("%.2f", Float.valueOf(String.valueOf(map.get("planCapacity6"))) / Float.valueOf(String.valueOf(Float.valueOf(map.get("standardWeekCapacity")))) * 100));
                rateList.add(String.format("%.2f", Float.valueOf(String.valueOf(map.get("planCapacity7"))) / Float.valueOf(String.valueOf(Float.valueOf(map.get("standardWeekCapacity")))) * 100));
                resourceMap.put("planCapacityRateList",rateList);

                planHoursList.add(Float.valueOf(String.valueOf(map.get("planHours"))));
                planUndoneHoursList.add(Float.valueOf(String.valueOf(map.get("planUndoneHours"))));
                eightList.add(resourceMap);
            }
//            result.put("planCapacity",Math.round(planCapacity));
            Map<String, String> capacityLoadUNPlanHours = reportMapper.getCapacityLoadUNPlanHours(workCenter);
            result.put("planCapacity",capacityLoadUNPlanHours.get("tardinessWork"));

            result.put("standardCapacity",Math.round(standardCapacity));
            result.put("standardMonthCapacity",Math.round(standardMonthCapacity));
            //八个柱状图
            result.put("eightList",eightList);
            //中心 计划工时、未完成工时List
            result.put("planHoursList",planHoursList);
            result.put("planUndoneHoursList",planUndoneHoursList);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return result;
    }

    /**
     * 制造中心检验看板
     * @param workCenter
     * @return
     */
    @Override
    public Map<String, Object> manufacturingPlanning(String workCenter) {
        HashMap<String, Object> result = new HashMap<>();
        LinkedHashMap<String, String> sixMap = reportMapper.sixShopOrderQTY(workCenter);
        ArrayList<Integer> sixList = new ArrayList<>();
        for (String key : sixMap.keySet()) {
            sixList.add(Integer.valueOf(String.valueOf(sixMap.get(key))));
        }
        result.put("sixList",sixList);
        List<Map<String, String>> nearRateList = reportMapper.nearFifteenDayRate(workCenter);
        ArrayList<Float> rateList = new ArrayList<>();
        ArrayList<Float> itemsList = new ArrayList<>();
        for (Map<String, String> nearRateMap : nearRateList) {
            rateList.add(Float.parseFloat(String.valueOf(nearRateMap.get("qualifiedRate"))));
            itemsList.add(Float.parseFloat(String.valueOf(nearRateMap.get("ItemsQTY"))));
        }
        result.put("rateList",rateList);
        result.put("itemsList",itemsList);
        //不合格责任部门分布
        List<Map<String, String>> departmentList = reportMapper.departmentForNonconformity(workCenter);
        List<String> monthOldList = departmentList.stream().map(e -> { return e.get("month"); })
                .distinct().sorted().collect(Collectors.toList());
        ArrayList<String> monthList = new ArrayList<>();
        monthList.addAll(monthOldList);
        Map<String, List<Map<String, String>>> descriptionMap = departmentList.stream().
                collect(Collectors.groupingBy(e -> e.get("description")));
        ArrayList<HashMap<String, Object>> descriptionValueList = new ArrayList<>();
        for (String description : descriptionMap.keySet()) {

            List<Map<String, String>> maps = descriptionMap.get(description);
            HashMap<String, Object> map = new HashMap<>();
            ArrayList<String> QTYList = new ArrayList<>();
            maps.forEach(e -> {
                QTYList.add(String.valueOf(e.get("QTY")));
            });
            map.put("name",description);
            map.put("value",QTYList);
            descriptionValueList.add(map);
        }
        result.put("monthList",monthList);
        result.put("descriptionValueList",descriptionValueList);
        //一次性交付合格率
        List<Map<String, String>> deliveryRateList = reportMapper.workshopDeliveryRateList(workCenter);
        HashMap<String, List<String>> deliveryRateMap = new HashMap<>();
        ArrayList<String> xDataNameList = new ArrayList<>();
        ArrayList<String> yDataList = new ArrayList<>();
        for (Map<String, String> map : deliveryRateList) {
            xDataNameList.add(String.valueOf(map.get("month")));
            yDataList.add(String.valueOf(map.get("rate")));
        }
        deliveryRateMap.put("xDataName",xDataNameList);
        deliveryRateMap.put("yData",yDataList);
        result.put("deliveryRate",deliveryRateMap);
        //不合格TOP5比例图
        List<Map<String, String>> ncTop5RateOldList = reportMapper.NCTop5RateList(workCenter).subList(0,5);
        ArrayList<Map<String, Object>> ncTop5RateList = new ArrayList<>();
        ncTop5RateOldList.forEach(e -> {
            HashMap<String, Object> ncMap = new HashMap<>();
            ncMap.put("name",String.valueOf(e.get("description")));
            ncMap.put("y",Integer.valueOf(String.valueOf(e.get("QTY"))));
            ncMap.put("h",Integer.valueOf(String.valueOf(e.get("QTY"))));
            ncTop5RateList.add(ncMap);
        });
        result.put("ncTop5Rate",ncTop5RateList);
        return result;
    }

    /**
     * 入库查询报表
     * @param paramMap
     * @return
     */
    @Override
    public List<Map<String, Object>> importStoreQueryReports(Map paramMap) {
        String site = CommonMethods.getSite();
        paramMap.put("site", site);
        paramMap.put("locale", LocaleContextHolder.getLocale().getLanguage());
        return reportMapper.importStoreQueryReports(paramMap);
    }

    /**
     * 项目空闲时间统计报表
     * @param paramMap
     * @return
     */
    @Override
    public List<Map<String, Object>> projectIdleTimeStatisticsReport(Map paramMap) {
        String site = CommonMethods.getSite();
        paramMap.put("site", site);
        paramMap.put("locale", LocaleContextHolder.getLocale().getLanguage());
        return reportMapper.projectIdleTimeStatisticsReport(paramMap);
    }

    /**
     * 项目加工周期报表
     * @param paramMap
     * @return
     */
    @Override
    public List<Map<String, Object>> projectProcessingCycleReport(Map paramMap) {
        String site = CommonMethods.getSite();
        paramMap.put("site", site);
        paramMap.put("locale", LocaleContextHolder.getLocale().getLanguage());
        return reportMapper.projectProcessingCycleReport(paramMap);
    }

    /**
     * 上工序已完成报表
     * @param paramMap
     * @return
     */
    @Override
    public List<Map<String, Object>> lastOperationCompletedReport(Map paramMap) {
        String site = CommonMethods.getSite();
        paramMap.put("site", site);
        paramMap.put("locale", LocaleContextHolder.getLocale().getLanguage());
        return reportMapper.lastOperationCompletedReport(paramMap);
    }

    /**
     * 订单查询报表
     * @param paramMap
     * @return
     */
    @Override
    public List<Map<String, Object>> orderQueryReportReport(Map paramMap) {
        String site = CommonMethods.getSite();
        paramMap.put("site", site);
        paramMap.put("locale", LocaleContextHolder.getLocale().getLanguage());
        List<Map<String, Object>> result = reportMapper.orderQueryReportReport(paramMap);
        HashMap<String, Object> countMap = new HashMap<>();
        countMap.put("ITEM","总计:");
        Long dispatchQTY = 0L ;
        Long doneQTY = 0L ;
        for (Map<String, Object> map : result) {
            dispatchQTY += Long.valueOf(String.valueOf(map.get("DISPATCH_QTY")));
            doneQTY += Long.valueOf(String.valueOf(map.get("DONE_QTY")));
        }
        countMap.put("DISPATCH_QTY",dispatchQTY);
        countMap.put("DONE_QTY",doneQTY);
        result.add(countMap);
        return result;
    }

    /**
     * 已派工工单明细报表
     * @param paramMap
     * @return
     */
    @Override
    public List<Map<String, Object>> assignedWorkersOrdersReport(Map paramMap) {
        String site = CommonMethods.getSite();
        paramMap.put("site", site);
        paramMap.put("locale", LocaleContextHolder.getLocale().getLanguage());
        return reportMapper.assignedWorkersOrdersReport(paramMap);
    }
    /**
     * 工作令有效生产报表
     * @param paramMap
     * @return
     */
    @Override
    public List<Map<String, Object>> workOrderEffectiveProductionReport(Map paramMap) {
        String site = CommonMethods.getSite();
        paramMap.put("site", site);
        paramMap.put("locale", LocaleContextHolder.getLocale().getLanguage());
        return reportMapper.workOrderEffectiveProductionReport(paramMap);
    }

    /**
     * 机台已完工工时报表
     * @param paramMap
     * @return
     */
    @Override
    public List<Map<String, String>> machineFinishedTimeReport(Map paramMap) {
        String site = CommonMethods.getSite();
        paramMap.put("site", site);
        paramMap.put("locale", LocaleContextHolder.getLocale().getLanguage());
        List<Map<String, String>> result = reportMapper.machineFinishedTimeReport(paramMap);
        HashMap<String, String> countMap = new HashMap<>();
        countMap.put("workCenterDescription","总计:");
        Float monthCapacity = 0.00f;
        Float tardinessWork = 0.00f;
        Float workHours1 = 0.00f;
        Float workHours2 = 0.00f;
        Float workHours3 = 0.00f;
        Float workHours4 = 0.00f;
        Float workHours5 = 0.00f;
        Float workHours6 = 0.00f;
        Float workHours7 = 0.00f;
        Float workHours8 = 0.00f;
        Float workHours9 = 0.00f;
        Float workHours10 = 0.00f;
        Float workHours11= 0.00f;
        Float workHours12 = 0.00f;
        Float totalWorkingHours = 0.00f;
        for (Map<String, String> map : result) {
            if (!StringUtils.isBlank(map.get("monthCapacity"))){
                monthCapacity += Float.valueOf(String.valueOf(map.get("monthCapacity")));
            }
            workHours1 += Float.valueOf(String.valueOf(map.get("workHours1")));
            workHours2 += Float.valueOf(String.valueOf(map.get("workHours2")));
            workHours3 += Float.valueOf(String.valueOf(map.get("workHours3")));
            workHours4 += Float.valueOf(String.valueOf(map.get("workHours4")));
            workHours5 += Float.valueOf(String.valueOf(map.get("workHours5")));
            workHours6 += Float.valueOf(String.valueOf(map.get("workHours6")));
            workHours7 += Float.valueOf(String.valueOf(map.get("workHours7")));
            workHours8 += Float.valueOf(String.valueOf(map.get("workHours8")));
            workHours9 += Float.valueOf(String.valueOf(map.get("workHours9")));
            workHours10 += Float.valueOf(String.valueOf(map.get("workHours10")));
            workHours11 += Float.valueOf(String.valueOf(map.get("workHours11")));
            workHours12 += Float.valueOf(String.valueOf(map.get("workHours12")));
            tardinessWork += Float.valueOf(String.valueOf(map.get("tardinessWork")));
            totalWorkingHours += Float.valueOf(String.valueOf(map.get("totalWorkingHours")));
        }
        countMap.put("tardinessWork",String.format("%.2f",tardinessWork));
        countMap.put("monthCapacity",String.format("%.2f",monthCapacity));
        countMap.put("workHours1",String.format("%.2f",workHours1));
        countMap.put("workHours2",String.format("%.2f",workHours2));
        countMap.put("workHours3",String.format("%.2f",workHours3));
        countMap.put("workHours4",String.format("%.2f",workHours4));
        countMap.put("workHours5",String.format("%.2f",workHours5));
        countMap.put("workHours6",String.format("%.2f",workHours6));
        countMap.put("workHours7",String.format("%.2f",workHours7));
        countMap.put("workHours8",String.format("%.2f",workHours8));
        countMap.put("workHours9",String.format("%.2f",workHours9));
        countMap.put("workHours10",String.format("%.2f",workHours10));
        countMap.put("workHours11",String.format("%.2f",workHours11));
        countMap.put("workHours12",String.format("%.2f",workHours12));
        countMap.put("totalWorkingHours",String.format("%.2f",totalWorkingHours));
        result.add(countMap);
        return result;
    }

    /**
     * 客户维度已完工工时报表
     * @param paramMap
     * @return
     */
    @Override
    public List<Map<String, String>> customerDimensionFinishedTimeReport(Map paramMap) {
        String site = CommonMethods.getSite();
        paramMap.put("site", site);
        paramMap.put("locale", LocaleContextHolder.getLocale().getLanguage());
        List<Map<String, String>> result = null;

        if (paramMap.containsKey("workCenter") && !"".equals(paramMap.get("workCenter"))){
            result = reportMapper.customerDimensionFinishedTimeReport(paramMap);
        }else {
            result = reportMapper.customerDimensionFinishedTimeReportNotWorkCenter(paramMap);
        }
        HashMap<String, String> countMap = new HashMap<>();
        countMap.put("workCenterDescription","总计:");
        Float tardinessWork = 0.00f;
        Float workHours1 = 0.00f;
        Float workHours2 = 0.00f;
        Float workHours3 = 0.00f;
        Float workHours4 = 0.00f;
        Float workHours5 = 0.00f;
        Float workHours6 = 0.00f;
        Float workHours7 = 0.00f;
        Float workHours8 = 0.00f;
        Float workHours9 = 0.00f;
        Float workHours10 = 0.00f;
        Float workHours11= 0.00f;
        Float workHours12 = 0.00f;
        Float totalWorkingHours = 0.00f;
        for (Map<String, String> map : result) {
            workHours1 += Float.valueOf(String.valueOf(map.get("workHours1")));
            workHours2 += Float.valueOf(String.valueOf(map.get("workHours2")));
            workHours3 += Float.valueOf(String.valueOf(map.get("workHours3")));
            workHours4 += Float.valueOf(String.valueOf(map.get("workHours4")));
            workHours5 += Float.valueOf(String.valueOf(map.get("workHours5")));
            workHours6 += Float.valueOf(String.valueOf(map.get("workHours6")));
            workHours7 += Float.valueOf(String.valueOf(map.get("workHours7")));
            workHours8 += Float.valueOf(String.valueOf(map.get("workHours8")));
            workHours9 += Float.valueOf(String.valueOf(map.get("workHours9")));
            workHours10 += Float.valueOf(String.valueOf(map.get("workHours10")));
            workHours11 += Float.valueOf(String.valueOf(map.get("workHours11")));
            workHours12 += Float.valueOf(String.valueOf(map.get("workHours12")));
            tardinessWork += Float.valueOf(String.valueOf(map.get("tardinessWork")));
            totalWorkingHours += Float.valueOf(String.valueOf(map.get("totalWorkingHours")));
        }
        countMap.put("tardinessWork",String.format("%.2f",tardinessWork));
        countMap.put("workHours1",String.format("%.2f",workHours1));
        countMap.put("workHours2",String.format("%.2f",workHours2));
        countMap.put("workHours3",String.format("%.2f",workHours3));
        countMap.put("workHours4",String.format("%.2f",workHours4));
        countMap.put("workHours5",String.format("%.2f",workHours5));
        countMap.put("workHours6",String.format("%.2f",workHours6));
        countMap.put("workHours7",String.format("%.2f",workHours7));
        countMap.put("workHours8",String.format("%.2f",workHours8));
        countMap.put("workHours9",String.format("%.2f",workHours9));
        countMap.put("workHours10",String.format("%.2f",workHours10));
        countMap.put("workHours11",String.format("%.2f",workHours11));
        countMap.put("workHours12",String.format("%.2f",workHours12));
        countMap.put("totalWorkingHours",String.format("%.2f",totalWorkingHours));
        result.add(countMap);
        return result;
    }

    /**
     * 工时看板
     * @param workCenter
     * @return
     */
    @Override
    public Map<String, Object> workHoursBoard(String workCenter) {
        HashMap<String, Object> result = new HashMap<>();

        LinkedHashMap<String, String> sixMap = reportMapper.sixWorkCenterHoursQTY(workCenter);
        ArrayList<Float> sixList = new ArrayList<>();
        for (String key : sixMap.keySet()) {
            sixList.add(Float.valueOf(String.valueOf(sixMap.get(key))));
        }
        result.put("sixList",sixList);

        List<Map<String, String>>  thisYearMapList = reportMapper.totalWorkingHoursCompletedThisYear(workCenter);
        ArrayList<String> monthYearList = new ArrayList<>();
        ArrayList<Float> workHorsList = new ArrayList<>();
        ArrayList<Float> planHorsList = new ArrayList<>();
        for (Map<String, String> map : thisYearMapList) {
            monthYearList.add(map.get("MONTH_MONTH"));
            planHorsList.add(Float.valueOf(String.valueOf(map.get("PLAN_QTY"))));
            workHorsList.add(Float.valueOf(String.valueOf(map.get("QTY"))));
        }
        result.put("monthYearList",monthYearList);
        result.put("planHorsList",planHorsList);
        result.put("workHorsList",workHorsList);

        //各车间总工时完成情况(单轴多折线图)
        List<Map<String, String>> WorkshopList = reportMapper.workingHoursEachWorkshop(workCenter);
        List<String> monthOldList = WorkshopList.stream().map(e -> { return e.get("MONTH"); })
                .distinct().sorted().collect(Collectors.toList());
        ArrayList<String> monthList = new ArrayList<>();
        monthList.addAll(monthOldList);

        Map<String, List<Map<String, String>>> descriptionMap = WorkshopList.stream().
                collect(Collectors.groupingBy(e -> e.get("WORK_CENTER")));
        ArrayList<HashMap<String, Object>> descriptionValueList = new ArrayList<>();
        for (String description : descriptionMap.keySet()) {
            List<Map<String, String>> maps = descriptionMap.get(description);
            HashMap<String, Object> map = new HashMap<>();
            ArrayList<Float> QTYList = new ArrayList<>();
            maps.forEach(e -> {
                QTYList.add(Float.valueOf(String.valueOf(e.get("QTY"))));
            });
            map.put("name",description);
            map.put("value",QTYList);
            descriptionValueList.add(map);
        }
        result.put("monthList",monthList);
        result.put("descriptionValueList",descriptionValueList);

        //事业部完工工时占比(双环形图)
        HashMap<String, Object> dutyCauseMap = new HashMap<>();
        List<Map<String, String>> workHoursCompletedList = reportMapper.workHoursCompletedBusinessUnit(workCenter);
        ArrayList<String> nameList = new ArrayList<>();
        ArrayList<HashMap<String, Object>> nameAndValueList = new ArrayList<>();
        for (Map<String, String> map : workHoursCompletedList) {
            nameList.add(map.get("workOrder"));
            HashMap<String, Object> NVMap = new HashMap<>();
            NVMap.put("name",map.get("workOrder"));
            NVMap.put("value",map.get("workHours"));
            nameAndValueList.add(NVMap);
        }
        dutyCauseMap.put("causeTypeDescription",nameList);
        dutyCauseMap.put("distributionMapList",nameAndValueList);
        result.put("dutyCause",dutyCauseMap);

        //年度对比图(单轴多折线图)
        List<Map<String, String>> contrastList = reportMapper.workingHoursYearContrast(workCenter);
        List<String> yearOldList = contrastList.stream().map(e -> { return e.get("M_MONTH"); })
                .distinct().sorted().collect(Collectors.toList());
        ArrayList<String> yearList = new ArrayList<>();
        yearList.addAll(yearOldList);

        Map<String, List<Map<String, String>>> contrastMap = contrastList.stream().
                collect(Collectors.groupingBy(e -> e.get("CON_YEAR")));
        ArrayList<HashMap<String, Object>> contrastMapValueList = new ArrayList<>();
        for (String description : contrastMap.keySet()) {
            List<Map<String, String>> maps = contrastMap.get(description);
            HashMap<String, Object> map = new HashMap<>();
            ArrayList<Float> QTYList = new ArrayList<>();
            maps.forEach(e -> {
                QTYList.add(Float.valueOf(String.valueOf(e.get("QTY"))));
            });
            map.put("name",description);
            map.put("value",QTYList);
            contrastMapValueList.add(map);
        }
        result.put("yearList",yearList);
        result.put("contrastMapValueList",contrastMapValueList);
        return result;
    }

    /**
     * 项目齐套率报表
     * @param paramMap
     * @return
     */
    @Override
    public List<Map<String, String>> projectAlignmentRateReport(Map paramMap) {
        String site = CommonMethods.getSite();
        paramMap.put("site", site);
        paramMap.put("locale", LocaleContextHolder.getLocale().getLanguage());
        return reportMapper.projectAlignmentRateReport(paramMap);
    }

    /**
     * 机加车间零部件工序效率报表
     * @param paramMap
     * @return
     */
    @Override
    public List<Map<String, String>> JJComponentProcessEfficiencyReport(Map paramMap) {
        String site = CommonMethods.getSite();
        paramMap.put("site", site);
        paramMap.put("locale", LocaleContextHolder.getLocale().getLanguage());
        return reportMapper.JJComponentProcessEfficiencyReport(paramMap);
    }

    /**
     * 工时统计报表(客户维度)
     * @param paramMap
     * @return
     */
    @Override
    public List<Map<String, String>> timeStatisticsCustomerReport(Map paramMap) {
        String site = CommonMethods.getSite();
        paramMap.put("site", site);
        paramMap.put("locale", LocaleContextHolder.getLocale().getLanguage());
        return reportMapper.timeStatisticsCustomerReport(paramMap);
    }

    /**
     * 工时统计报表(车间维度)
     * @param paramMap
     * @return
     */
    @Override
    public List<Map<String, String>> timeStatisticsWorkshopReport(Map paramMap) {
        String site = CommonMethods.getSite();
        paramMap.put("site", site);
        paramMap.put("locale", LocaleContextHolder.getLocale().getLanguage());
        return reportMapper.timeStatisticsWorkshopReport(paramMap);
    }

    /**
     * 工时统计-车间维度看板
     * @param paramMap
     * @return
     */
    @Override
    public Map<String, Object> financialWorkHoursBoard(Map paramMap) {
        HashMap<String, Object> result = new HashMap<>();
        List<Map<String, String>> mapList = reportMapper.timeStatisticsWorkshopReport(paramMap);
        Map<String, List<Map<String, String>>> workCenterList =
                mapList.stream().collect(Collectors.groupingBy(e -> e.get("WORK_CENTER_DESCRIPTION")));
        ArrayList<Object> workCenterDescList = new ArrayList<>();
        workCenterDescList.add("workCenter");
        ArrayList<Object> workHoursList = new ArrayList<>();
        workHoursList.add("加工工时");
        ArrayList<Object> ZTFWorkHoursList = new ArrayList<>();
        ZTFWorkHoursList.add("临时修复工时");
        ArrayList<Object> totalHoursList = new ArrayList<>();
        totalHoursList.add("总工时");

        workCenterList.remove("外协虚拟车间");
        workCenterList.remove("装配车间");
        for (String workCenter : workCenterList.keySet()) {
            workCenterDescList.add(workCenter);
            List<Map<String, String>>workCenterMapList = workCenterList.get(workCenter);
            Float workHours = 0.00f;
            Float ZTFWorkHours = 0.00f;
            Float totalHours = 0.00f;
            for (Map<String, String> map : workCenterMapList) {
                workHours += Float.valueOf(String.valueOf(map.get("WORK_HOURS")));
                ZTFWorkHours += Float.valueOf(String.valueOf(map.get("ZTF_WORK_HOURS")));
                totalHours += Float.valueOf(String.valueOf(map.get("TOTAL_HOURS")));
            }
            workHoursList.add(workHours);
            ZTFWorkHoursList.add(ZTFWorkHours);
            totalHoursList.add(totalHours);
        }
        ArrayList<Object> resultList = new ArrayList<>();
        resultList.add(workCenterDescList);
        resultList.add(workHoursList);
        resultList.add(ZTFWorkHoursList);
        resultList.add(totalHoursList);
        result.put("resultList",resultList);
        return result;
    }

    /**
     * 资源负荷率报表(上工序已完成)
     * @param paramMap
     * @return
     */
    @Override
    public List<Map<String, String>> loadRatePreviousCompletedReport(Map paramMap) {
        String site = CommonMethods.getSite();
        paramMap.put("site", site);
        paramMap.put("locale", LocaleContextHolder.getLocale().getLanguage());
        Integer restOfTheWeek = 1 ;
        if (String.valueOf(paramMap.get("week")).equals("true")){
            LocalDateTime now = LocalDateTime.now();
            DateTimeFormatter fmt = DateTimeFormatter.ofPattern("yyyy-MM-dd");
            int dayOfWeek = now.getDayOfWeek().getValue();
            LocalDateTime weekStart = now.minusDays(dayOfWeek - 1).with(LocalTime.MIN);
            LocalDateTime weekEnd = now.plusDays(7 * 8 - dayOfWeek).with(LocalTime.MAX);
            List<Map<String, String>> weekOfDate = DateReportUtils.getWeekOfDate3(weekStart.format(fmt), weekEnd.format(fmt));
            return weekOfDate;
        }else {
            LocalDateTime now = LocalDateTime.now();
            int dayOfWeek = now.getDayOfWeek().getValue();
            restOfTheWeek = 7 - dayOfWeek + 1;
        }
        List<Map<String, String>> resultList = reportMapper.loadRatePreviousCompletedReport(paramMap);
        HashMap<String, String> countMap = new HashMap<>();
        countMap.put("workCenterDescription","总计:");
        Float tardinessWork = 0.00f;
        Float weekCapacity = 0.00f;
        Float dayCapacity = 0.00f;
        Float OverdueAddWeek = 0.00f;
        Float unWorkHours1 = 0.00f;
        Float unWorkHours2 = 0.00f;
        Float unWorkHours3 = 0.00f;
        Float unWorkHours4 = 0.00f;
        Float unWorkHours5 = 0.00f;
        Float unWorkHours6 = 0.00f;
        Float unWorkHours7 = 0.00f;
        Float unWorkHours8 = 0.00f;

        for (Map<String, String> map : resultList) {
            tardinessWork += Float.valueOf(String.valueOf(map.get("tardinessWork")));
            if (!StringUtils.isBlank(map.get("weekCapacity"))){
                weekCapacity += Float.valueOf(String.valueOf(map.get("weekCapacity")));
            }
            if (!StringUtils.isBlank(map.get("dayCapacity"))){
                dayCapacity += Float.valueOf(String.valueOf(map.get("dayCapacity")));
            }
            OverdueAddWeek += Float.valueOf(String.valueOf(map.get("OverdueAddWeek")));
            unWorkHours1 += Float.valueOf(String.valueOf(map.get("unWorkHours1")));
            unWorkHours2 += Float.valueOf(String.valueOf(map.get("unWorkHours2")));
            unWorkHours3 += Float.valueOf(String.valueOf(map.get("unWorkHours3")));
            unWorkHours4 += Float.valueOf(String.valueOf(map.get("unWorkHours4")));
            unWorkHours5 += Float.valueOf(String.valueOf(map.get("unWorkHours5")));
            unWorkHours6 += Float.valueOf(String.valueOf(map.get("unWorkHours6")));
            unWorkHours7 += Float.valueOf(String.valueOf(map.get("unWorkHours7")));
            unWorkHours8 += Float.valueOf(String.valueOf(map.get("unWorkHours8")));

        }
        countMap.put("tardinessWork",String.format("%.2f",tardinessWork));
        countMap.put("weekCapacity",String.format("%.2f",weekCapacity));
        countMap.put("dayCapacity",String.format("%.2f",dayCapacity));
        countMap.put("OverdueAddWeek",String.format("%.2f",OverdueAddWeek));
        countMap.put("unWorkHours1",String.format("%.2f",unWorkHours1));
        countMap.put("unWorkHours2",String.format("%.2f",unWorkHours2));
        countMap.put("unWorkHours3",String.format("%.2f",unWorkHours3));
        countMap.put("unWorkHours4",String.format("%.2f",unWorkHours4));
        countMap.put("unWorkHours5",String.format("%.2f",unWorkHours5));
        countMap.put("unWorkHours6",String.format("%.2f",unWorkHours6));
        countMap.put("unWorkHours7",String.format("%.2f",unWorkHours7));
        countMap.put("unWorkHours8",String.format("%.2f",unWorkHours8));
        countMap.put("weekRate1",String.format("%.2f", (OverdueAddWeek + unWorkHours1)  / (dayCapacity * restOfTheWeek) * 100 ) + "%");
        countMap.put("weekRate2",String.format("%.2f",unWorkHours2 / weekCapacity * 100 ) + "%");
        countMap.put("weekRate3",String.format("%.2f",unWorkHours3 / weekCapacity * 100 ) + "%");
        countMap.put("weekRate4",String.format("%.2f",unWorkHours4 / weekCapacity * 100 ) + "%");
        countMap.put("weekRate5",String.format("%.2f",unWorkHours5 / weekCapacity * 100 ) + "%");
        countMap.put("weekRate6",String.format("%.2f",unWorkHours6 / weekCapacity * 100 ) + "%");
        countMap.put("weekRate7",String.format("%.2f",unWorkHours7 / weekCapacity * 100 ) + "%");
        countMap.put("weekRate8",String.format("%.2f",unWorkHours8 / weekCapacity * 100 ) + "%");
        resultList.add(countMap);

        return resultList;
    }

    /**
     * 事业部项目进度明细看板
     * @param paramMap
     * @return
     */
    @Override
    public Map<String, Object> projectScheduleReportBoard(Map paramMap) {
        HashMap<String, Object> result = new HashMap<>();
        if (paramMap.containsKey("item")){
            List<Map<String, String>> maps = reportMapper.projectScheduleReportBoard(paramMap);
            result.put("oldData", maps);
            result.put("item", paramMap.get("item"));
            return result;
        }
        String user = null;
        if (paramMap.containsKey("career")){
            user = String.valueOf(paramMap.get("career"));
        }
        result.put("career", user);
        List<Map<String, String>> projectScheduleList = reportMapper.projectScheduleShowBoard(user);

        //项目进度圆圈circle
        ArrayList<HashMap<String, String>> projectCircleList = new ArrayList<>();
        Map<String, List<Map<String, String>>> itemNMap = projectScheduleList.stream().collect(Collectors.groupingBy(e -> e.get("item")));
        for (String items : itemNMap.keySet()) {
            List<Map<String, String>> maps = itemNMap.get(items);
            String name = items;
            Float rate = 0.00f;
            Float projectRate = 0.00f;
            for (Map<String, String> map : maps) {
                rate += Float.valueOf(map.get("rate"));
                projectRate += Float.valueOf(map.get("planRate"));
            }
            HashMap<String, String> objectObjectHashMap = new HashMap<>();
            objectObjectHashMap.put("name",name);
            objectObjectHashMap.put("rate",String.format("%.2f",rate / maps.size()));
            objectObjectHashMap.put("projectRate",String.format("%.2f",projectRate / maps.size()));
            projectCircleList.add(objectObjectHashMap);
        }
        result.put("projectCircleList",projectCircleList);
        return result;
    }

    /**
     * 上下料表维护
     * @param paramMap
     * @return
     */
    @Override
    public List<Map<String, String>> inventoryTableSave(Map paramMap) {
        String site = CommonMethods.getSite();
        paramMap.put("site", site);
        paramMap.put("locale", LocaleContextHolder.getLocale().getLanguage());
        return reportMapper.inventoryTableSave(paramMap);
    }

    /**
     * 设备稼动率报表
     * @param paramMap
     * @return
     */
    @Override
    public List<Map<String, String>> operationRateOfEquipmentReport(Map paramMap) {
        String site = CommonMethods.getSite();
        paramMap.put("site", site);
        paramMap.put("locale", LocaleContextHolder.getLocale().getLanguage());
        return reportMapper.operationRateOfEquipmentReport(paramMap);
    }

    /**
     * 焊前件周转报表
     * @param paramMap
     * @return
     */
    @Override
    public List<Map<String, String>> weldingForepartsTurnoverReport(Map paramMap) {
        String site = CommonMethods.getSite();
        paramMap.put("site", site);
        paramMap.put("locale", LocaleContextHolder.getLocale().getLanguage());
        return reportMapper.weldingForepartsTurnoverReport(paramMap);
    }

    /**
     * 工时统计报表
     * @param paramMap
     * @return
     */
    @Override
    public List<Map<String, Object>> monthTimeStatisticsReport(Map paramMap) {
        String site = CommonMethods.getSite();
        paramMap.put("site", site);
        paramMap.put("locale", LocaleContextHolder.getLocale().getLanguage());
        return reportMapper.monthTimeStatisticsReport(paramMap);
    }

    /**
     * 质量问题统计表
     * @param paramMap
     * @return
     */
    @Override
    public List<Map<String, Object>> qualityProblemsInfoReport(Map paramMap) {
        String site = CommonMethods.getSite();
        paramMap.put("site", site);
        paramMap.put("locale", LocaleContextHolder.getLocale().getLanguage());
        List<Map<String, Object>> qualityInspectionReport = reportMapper.qualityProblemsInfoReport(paramMap);
        //原因分类
        Map<String, List<DataFieldList>> dataFieldListMap = new HashMap<>();
        DataField dataField = new DataField();
        dataField.setSite(site);
        dataField.setDataField("REASON_CATEGORY");
        List<DataFieldList> dataFieldList = dataFieldListMapper.findDataFieldList(dataField);
        if (dataFieldList != null && dataFieldList.size() > 0) {
            dataFieldListMap = dataFieldList.stream().collect(Collectors.groupingBy(DataFieldList::getDataValue));
        }
        for (int i = 0; i < qualityInspectionReport.size(); i++) {
            Map<String, Object> temporaryMap = qualityInspectionReport.get(i);
            String dutyCauseType = (String) temporaryMap.get("DUTY_CAUSE_TYPE");
            if (StringUtils.isBlank(dutyCauseType)) {
                qualityInspectionReport.get(i).put("DUTY_CAUSE_TYPE_DESCRIPTION", "");
                continue;
            }
            String[] dutyCauseTypeSplit = dutyCauseType.split(",");
            StringBuilder dutyCauseTypeDescription = new StringBuilder();
            for (int j = 0; j < dutyCauseTypeSplit.length; j++) {
                if (dataFieldListMap.get(dutyCauseTypeSplit[j]) != null) {
                    dutyCauseTypeDescription.append(dataFieldListMap.get(dutyCauseTypeSplit[j]).get(0).getDescription()).append(";");
                }
            }
            qualityInspectionReport.get(i).put("DUTY_CAUSE_TYPE_DESCRIPTION", dutyCauseTypeDescription);
        }

        //返修工序内容
        Map map = new HashMap();
        map.put("site", site);
        for (int i = 0; i < qualityInspectionReport.size(); i++) {
            Map<String, Object> temporaryMap = qualityInspectionReport.get(i);
            String routerBo = (String) temporaryMap.get("ROUTER_BO");
            if(StringUtil.notBlank(routerBo)){
                //  map.put("abnormalBo", abnormalBo);
                map.put("routerBo", routerBo);
                List<Map<String, Object>> result = routerStepcontentService.selectDisRouterInfo(map);
                String ReapirContent = "";
                if (result != null && result.size() != 0) {
                    for (int j = 0; j < result.size(); j++) {
                        ReapirContent = ReapirContent + "【" + (String) result.get(j).get("STEP_ID") + "】" +
                                (String) result.get(j).get("ROUTER_STEP_CONTENT");
                    }
                }
                qualityInspectionReport.get(i).put("ROUTER_STEP_CONTENT", ReapirContent);
            }

        }
        //责任分类
        dataField.setDataField("RESPONSIBILITY_TYPE");
        dataFieldList = dataFieldListMapper.findDataFieldList(dataField);
        if (dataFieldList != null && dataFieldList.size() > 0) {
            dataFieldListMap = dataFieldList.stream().collect(Collectors.groupingBy(DataFieldList::getDataValue));
        }
        for (int i = 0; i < qualityInspectionReport.size(); i++) {
            Map<String, Object> temporaryMap = qualityInspectionReport.get(i);
            String dutyType = (String) temporaryMap.get("DUTY_TYPE");
            if (StringUtils.isBlank(dutyType)) {
                qualityInspectionReport.get(i).put("DUTY_TYPE_DESCRIPTION", "");
                continue;
            }
            String[] dutyTypeSplit = dutyType.split(",");
            StringBuilder dutyTypeDescription = new StringBuilder();
            for (int j = 0; j < dutyTypeSplit.length; j++) {
                if (dataFieldListMap.get(dutyTypeSplit[j]) != null) {
                    dutyTypeDescription.append(dataFieldListMap.get(dutyTypeSplit[j]).get(0).getDescription()).append(";");
                }
            }
            qualityInspectionReport.get(i).put("DUTY_TYPE_DESCRIPTION", dutyTypeDescription);
        }
        //责任部门
        dataField.setDataField("RESPONSIBILITY_DEPARTMENT");
        dataFieldList = dataFieldListMapper.findDataFieldList(dataField);
        if (dataFieldList != null && dataFieldList.size() > 0) {
            dataFieldListMap = dataFieldList.stream().collect(Collectors.groupingBy(DataFieldList::getDataValue));
        }
        for (int i = 0; i < qualityInspectionReport.size(); i++) {
            Map<String, Object> temporaryMap = qualityInspectionReport.get(i);
            String dutyDepart = (String) temporaryMap.get("DUTY_DEPART");
            if (StringUtils.isBlank(dutyDepart)) {
                qualityInspectionReport.get(i).put("DUTY_DEPART_DESCRIPTION", "");
                continue;
            }
            String[] dutyDepartSplit = dutyDepart.split(",");
            StringBuilder dutyDepartDescription = new StringBuilder();
            for (int j = 0; j < dutyDepartSplit.length; j++) {
                if (dataFieldListMap.get(dutyDepartSplit[j]) != null) {
                    dutyDepartDescription.append(dataFieldListMap.get(dutyDepartSplit[j]).get(0).getDescription()).append(";");
                }
            }
            qualityInspectionReport.get(i).put("DUTY_DEPART_DESCRIPTION", dutyDepartDescription);
        }
        return qualityInspectionReport;
    }



    /**
     * 事业部质量看板数据接口
     * @param user
     * @return
     */
    @Override
    public HashMap<String, Object> qualityShows(String user) {
        HashMap<String, Object> result = new HashMap<>();
        result.put("user",CommonMethods.getUser()+ ":" + user);

        //在运行质量返修明细
        List<Map<String, String>> qualityShows = reportMapper.qualityShows(user ,CommonMethods.getSite());
        ArrayList<List<String>> qualityShowsList = new ArrayList<>();
        for (Map<String, String> qualityShow : qualityShows) {
            ArrayList<String> quality = new ArrayList<>();
            quality.add(qualityShow.get("abnormalNo"));
            quality.add(qualityShow.get("itemDescription"));
            quality.add(qualityShow.get("item"));
            quality.add(qualityShow.get("createdDateTime"));
            quality.add(qualityShow.get("resolveDateTime"));
            quality.add(qualityShow.get("planEndCompDate"));
            quality.add(qualityShow.get("reworkSum"));
            quality.add(qualityShow.get("lastReworkSum"));
            quality.add(qualityShow.get("rate"));
            qualityShowsList.add(quality);
        }
        result.put("qualityShows",qualityShowsList);
        //近半年单据信息 订单总零件数(orderedTotalNumber) 质量异常总零件数(abnormalQualityNumber) 合格率(rate)%
        Map<String, String> documentInfo = reportMapper.documentInformation(user);
        result.put("documentInfo",documentInfo);
        //近半年异常原因分布 dutyCauseType
        HashMap<String, Object> dutyCauseMap = new HashMap<>();
        List<Map<String, String>> dutyCauseTypeDescription = reportMapper.dutyCauseTypeDescription();
        List<String> descriptionList = dutyCauseTypeDescription.stream().map(e -> e.get("DESCRIPTION")).collect(Collectors.toList());

        List<String> numbersList = new LinkedList<>();
        List<String> dutyCauseType = reportMapper.dutyCauseType(user);
        for (String type : dutyCauseType) {
            if (type.indexOf(",") != -1){
                String[] split = type.split(",");
                for (String s : split) {
                    numbersList.add(s);
                }
            }else {
                numbersList.add(type);
            }
        }
        List<HashMap<String, Object>> distributionMapList = new ArrayList<>();
        for (int i = 0; i < descriptionList.size(); i++) {
            HashMap<String, Object> distributionMap = new HashMap<>();
            Integer integers = 0;
            for (String numbers : numbersList) {
                if (numbers.equals(String.valueOf(i + 1))){
                    integers++;
                }
            }
            distributionMap.put("name", descriptionList.get(i));
            distributionMap.put("value", integers);
            distributionMapList.add(distributionMap);
        }
        //过滤前十的异常原因
        distributionMapList = distributionMapList.stream()
                .sorted(Comparator.comparing(e -> Integer.parseInt(String.valueOf(e.get("value"))))).collect(Collectors.toList());
        List<HashMap<String, Object>> distributionMapListTop = distributionMapList.subList(8,18);
        descriptionList.clear();
        for (HashMap<String, Object> hashMap : distributionMapListTop) {
            descriptionList.add(String.valueOf(hashMap.get("name")));
        }

        dutyCauseMap.put("causeTypeDescription",descriptionList);
        dutyCauseMap.put("distributionMapList",distributionMapListTop);
        result.put("dutyCause",dutyCauseMap);
        //近半年质量异常情况
        HashMap<String, Object> abnormalQualityMap = new HashMap<>();
        List<String> dateList = new ArrayList<>();
        ArrayList<String> abnormalQualityNumbers = new ArrayList<>();

        //获取近半年月份
        List<Map<String, String>> abnormalQualityList = reportMapper.abnormalQuality(user);
        for (Map<String, String> map : abnormalQualityList) {
            dateList.add(map.get("DATE_TIME"));
            abnormalQualityNumbers.add(map.get("NC_QTY"));
        }
        abnormalQualityMap.put("month",dateList);
        abnormalQualityMap.put("qualityNumbers",abnormalQualityNumbers);
        result.put("abnormalQuality",abnormalQualityMap);
        return result;
    }

    /**
     * 事业部项目进度看板数据接口
     * @param userGroup
     * @return
     */
    @Override
    public HashMap<String, Object> projectScheduleShow(String userGroup) {
        HashMap<String, Object> result = new HashMap<>();
        result.put("userGroup", userGroup);
        List<Map<String, String>> abnormalQualityList = reportMapper.projectScheduleShow(userGroup);
       /* if (abnormalQualityList.size() > 20){
            abnormalQualityList = abnormalQualityList.subList(0, 20);
        }*/
        result.put("originalData",abnormalQualityList);

        //项目进度圆圈circle
        ArrayList<HashMap<String, String>> projectCircleList = new ArrayList<>();
        Map<String, List<Map<String, String>>> itemNMap = abnormalQualityList.stream().collect(Collectors.groupingBy(e -> e.get("item")));
        for (String items : itemNMap.keySet()) {
            List<Map<String, String>> maps = itemNMap.get(items);
            String name = items;
            Float rate = 0.00f;
            Float projectRate = 0.00f;
            for (Map<String, String> map : maps) {
                rate += Float.valueOf(map.get("rate"));
                projectRate += Float.valueOf(map.get("planRate"));
            }
            HashMap<String, String> objectObjectHashMap = new HashMap<>();
            objectObjectHashMap.put("name",name);
            objectObjectHashMap.put("rate",String.valueOf(rate / maps.size()));
            objectObjectHashMap.put("projectRate",String.valueOf(projectRate / maps.size()));
            projectCircleList.add(objectObjectHashMap);
        }
       /* for (Map<String, String> abnormalQualityMap : abnormalQualityList) {
            HashMap<String, String> objectObjectHashMap = new HashMap<>();
            objectObjectHashMap.put("name",abnormalQualityMap.get("item"));
            objectObjectHashMap.put("rate",abnormalQualityMap.get("rate"));
            objectObjectHashMap.put("projectRate",abnormalQualityMap.get("planRate"));
            projectCircleList.add(objectObjectHashMap);
        }*/

        result.put("projectCircleList",projectCircleList);

        //项目进度生产甘特图GanttChart 项目号、部套、零件总数量
        ArrayList<HashMap<String, String>> firstThreeList = new ArrayList<>();
        for (Map<String, String> abnormalQualityMap : abnormalQualityList) {
            LinkedHashMap<String, String> firstThreeMap = new LinkedHashMap<>();
            if (abnormalQualityMap.get("item").length() > 30){
                firstThreeMap.put("value1",abnormalQualityMap.get("item").substring(0,29));
            }else {
                firstThreeMap.put("value1",abnormalQualityMap.get("item"));
            }

            if (abnormalQualityMap.get("workOrder").indexOf("/") != -1){
                if (abnormalQualityMap.get("workOrder").split("/").length > 2){
                    firstThreeMap.put("value2",abnormalQualityMap.get("workOrder").split("/")[2]);
                }else {
                    firstThreeMap.put("value2",abnormalQualityMap.get("workOrder").split("/")[1]);
                }
            }else {
                firstThreeMap.put("value2",abnormalQualityMap.get("workOrder"));
            }
            firstThreeMap.put("value3",abnormalQualityMap.get("shopOrderQTY"));
            firstThreeList.add(firstThreeMap);
        }
        result.put("firstThreeList",firstThreeList);

        // rate2 计划进度（浅绿色）/ rate1 计划完成（深蓝色） 为百分比 rate3深黄色() 实际工时进度。start开始时间   0 -- 70
        ArrayList<HashMap<String, Object>> projectGanttChartList = new ArrayList<>();
       for (Map<String, String> abnormalQualityMap : abnormalQualityList) {
            /*HashMap<String, String> projectGanttChartHashMap = new HashMap<>();
            String startDate = dateConvert(String.valueOf(abnormalQualityMap.get("startDate")));
            projectGanttChartHashMap.put("start",startDate);
            // rate1 计划完成（深蓝色）
            String planRate = dateConvert(String.valueOf(abnormalQualityMap.get("endPlanDate")));
           if ((Float.valueOf(planRate) - Float.valueOf(startDate)) <= 0){
               projectGanttChartHashMap.put("rate1", String.valueOf(1.00));
               projectGanttChartHashMap.put("rate2", String.valueOf(0.00));
               projectGanttChartHashMap.put("rate3", String.valueOf(0.00));
               projectGanttChartHashMap.put("start", String.valueOf(0.00));
               projectGanttChartList.add(projectGanttChartHashMap);
               continue;
           }
            planRate = String.valueOf(Float.valueOf(planRate) - Float.valueOf(startDate));
            if (Float.valueOf(planRate) < 1){
                projectGanttChartHashMap.put("rate1", "1");
            }else {
                    projectGanttChartHashMap.put("rate1", planRate);
                }

            String rate = String.valueOf(abnormalQualityMap.get("rate"));
            if ((Float.valueOf(rate) * 0.7 - Float.valueOf(startDate)) < 0){
                projectGanttChartHashMap.put("rate3",String.valueOf(0.00));
            }else {
                projectGanttChartHashMap.put("rate3",String.valueOf(Float.valueOf(rate) * 0.7 - Float.valueOf(startDate)));
            }
            //String now = dateConvert(DateUtil.formatDate(new Date()));
            projectGanttChartHashMap.put("rate2",String.valueOf(Float.valueOf(abnormalQualityMap.get("planRate")) * 0.7  - Float.valueOf(startDate)));
            projectGanttChartList.add(projectGanttChartHashMap);*/
           HashMap<String, Object> projectGanttChartHashMap = new HashMap<>();
           Date startDate = DateReportUtils.stringToDate(String.valueOf(abnormalQualityMap.get("startDate")), "yyyy-MM-dd");
           Date endPlanDate = DateReportUtils.stringToDate(String.valueOf(abnormalQualityMap.get("endPlanDate")), "yyyy-MM-dd");
           projectGanttChartHashMap.put("start",startDate.getTime());
           projectGanttChartHashMap.put("end",endPlanDate.getTime());
           projectGanttChartHashMap.put("rate", Float.valueOf(String.format("%.2f", Float.valueOf(String.valueOf(abnormalQualityMap.get("rate"))))));
           projectGanttChartHashMap.put("rate1",Float.valueOf(String.format("%.2f", Float.valueOf(String.valueOf(abnormalQualityMap.get("planRate"))))));
           projectGanttChartHashMap.put("rate2",Float.valueOf(String.format("%.2f", Float.valueOf(String.valueOf(abnormalQualityMap.get("rate"))))));
           projectGanttChartList.add(projectGanttChartHashMap);
        }
        result.put("projectGanttChartList",projectGanttChartList);
        //System.out.println(JSON.toJSONString(projectGanttChartList));
        return result;
    }


    /**
     * 看板日期格式转换 任务开始时间
     * @param date
     * @return
     */
    private String dateConvert(String date){

        long oneDate = 1000 * 60 * 60 * 24;

        long tenWeek = 1000 * 60 * 60 * 24 * 7 * 10;
        //任务开始时间
        long dateTime = DateUtil.parseDate(date).getTime();
        //十个周 70天 | 当前一周开始时间为基准 前四周 + 后六周
        long currentTime  = DateUtil.getBeginDateByWeek().getTime();
        // 四周前 毫秒数
        long firstFourWeeksTime = currentTime - (oneDate * ( 4 * 7 ));
        //  天数 = (任务开始时间 - 四周前) / (1000 * 60 * 60 * 24)
        double timeDifference = (double)(dateTime - firstFourWeeksTime) / (double)oneDate;
        if (timeDifference > 70){
            return "70.00";
        }else if(timeDifference < 0){
            return "0.00";
        }
        return String.format("%.2f", timeDifference);
    }

}

