package com.foreverwin.mesnac.anomaly.service.impl;

import com.foreverwin.mesnac.common.util.ExcelUtils;
import com.foreverwin.mesnac.common.util.StringUtil;
import com.foreverwin.mesnac.meapi.util.StringUtils;
import com.foreverwin.modular.core.util.CommonMethods;
import com.foreverwin.modular.core.util.FrontPage;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.foreverwin.mesnac.anomaly.model.TemporaryFix;
import com.foreverwin.mesnac.anomaly.mapper.TemporaryFixMapper;
import com.foreverwin.mesnac.anomaly.service.TemporaryFixService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.foreverwin.modular.core.util.R;
import org.apache.commons.fileupload.FileItem;
import org.springframework.stereotype.Service;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.commons.CommonsMultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.websocket.server.PathParam;
import java.time.LocalDateTime;
import java.util.List;

/**
 * <p>
 * 临时修复表 服务实现类
 * </p>
 *
 * @author Y.Q
 * @since 2022-09-08
 */
@Service("TemporaryFixService")
@Transactional(rollbackFor = Exception.class)
public class TemporaryFixServiceImpl extends ServiceImpl<TemporaryFixMapper, TemporaryFix> implements TemporaryFixService {


    @Autowired
    private TemporaryFixMapper temporaryFixMapper;

    @Override
    public IPage<TemporaryFix> selectPage(FrontPage<TemporaryFix> frontPage, TemporaryFix temporaryFix) {
        QueryWrapper<TemporaryFix> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(temporaryFix);
        return super.page(frontPage.getPagePlus(), queryWrapper);
    }

    @Override
    public List<TemporaryFix> selectList(TemporaryFix temporaryFix) {
        QueryWrapper<TemporaryFix> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(temporaryFix);
        return super.list(queryWrapper);
    }

    /**
     * 文件导入
     *
     * @param temporaryFixList
     * @return
     */
    @Override
    public List<TemporaryFix> temporaryFixImportFile(List<TemporaryFix> temporaryFixList) {
        String user = CommonMethods.getUser();
        String site = CommonMethods.getSite();
        LocalDateTime now = LocalDateTime.now();
        try {
            for (TemporaryFix temporaryFix : temporaryFixList) {
                String handle = StringUtil.createQUID();
                temporaryFix.setHandle(handle);
                temporaryFix.setSite(site);
                temporaryFix.setCreatedTime(now);
                temporaryFix.setCreatedBy(user);
                temporaryFix.setUpdatedBy(user);
                temporaryFix.setUpdatedTime(now);
                if (temporaryFix.getAmount() != null && temporaryFix.getSingleWorkHours() != null && temporaryFix.getAmount() != "" && temporaryFix.getSingleWorkHours() != ""){
                    temporaryFix.setTotalWorkHours(String.valueOf(Float.valueOf(temporaryFix.getAmount()) * Float.valueOf(temporaryFix.getSingleWorkHours())));
                }
            }
            return temporaryFixList;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 临时维修导入
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public R temporaryFixImportFile(FileItem fileItem, HttpServletRequest req) {
        String fileType = req.getParameter("fileType");
        String site = req.getParameter("site");
        String user = req.getParameter("user");
        CommonMethods.setSite(site);
        CommonMethods.setUser(user);
        try {
            CommonsMultipartFile multipartFile = new CommonsMultipartFile(fileItem);
            List<TemporaryFix> temporaryFixList = ExcelUtils.readExcel(fileType, TemporaryFix.class, multipartFile, null);
            if (temporaryFixList.size() == 0){
                return R.failed("导出数据失败：0条数据");
            }
            temporaryFixList = this.temporaryFixImportFile(temporaryFixList);
            for (TemporaryFix temporaryFix : temporaryFixList) {
                if (StringUtils.isBlank(temporaryFix.getUploadCenter()) || StringUtils.isBlank(temporaryFix.getUploadUser())){
                    return R.failed("请导入正确的上传车间或上传人员且不为空");
                }else if (!StringUtils.isBlank(temporaryFix.getEntrustUnit())){
                    if (temporaryFix.getEntrustUnit().equals("制造中心") || temporaryFix.getEntrustUnit().equals("市场部") || temporaryFix.getEntrustUnit().equals("外协")){
                        if (StringUtils.isBlank(temporaryFix.getDutyPerson())){
                            return R.failed("委托/联系单位:" + temporaryFix.getEntrustUnit() + ",请填写“责任主体/人/原因”后再进行导入" );
                        }else if (StringUtils.isBlank(temporaryFix.getProjectNumber())){
                            return R.failed("委托/联系单位:" + temporaryFix.getEntrustUnit() + ",请填写“项目号”后再进行导入" );
                        }
                    }
                }else if (StringUtils.isBlank(temporaryFix.getQualityReviewNumber()) && StringUtils.isBlank(temporaryFix.getWorkContactNumber())
                        && StringUtils.isBlank(temporaryFix.getOtherKindsNumber())){
                    return R.failed("请至少填写一个再进行导入:质量评审单号、工作联络单号、其他种类单据号" );
                }
            }
            boolean result = this.saveBatch(temporaryFixList);
            return R.ok(result);
        } catch (Exception e) {
            e.printStackTrace();
            return R.failed(e.getMessage());
        }
    }

}