package com.foreverwin.mesnac.anomaly.service.impl;

import com.foreverwin.mesnac.anomaly.model.TemporaryFix;
import com.foreverwin.mesnac.common.util.ExcelUtils;
import com.foreverwin.mesnac.common.util.StringUtil;
import com.foreverwin.mesnac.meapi.util.StringUtils;
import com.foreverwin.modular.core.util.CommonMethods;
import com.foreverwin.modular.core.util.FrontPage;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.foreverwin.mesnac.anomaly.model.WorkHourStatistics;
import com.foreverwin.mesnac.anomaly.mapper.WorkHourStatisticsMapper;
import com.foreverwin.mesnac.anomaly.service.WorkHourStatisticsService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.foreverwin.modular.core.util.R;
import org.apache.commons.fileupload.FileItem;
import org.springframework.stereotype.Service;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.commons.CommonsMultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.time.LocalDateTime;
import java.util.List;
/**
 * <p>
 * 工时统计汇总表 服务实现类
 * </p>
 *
 * @author YinQ
 * @since 2022-12-16
 */
@Service("WorkHourStatisticsService")
@Transactional(rollbackFor = Exception.class)
public class WorkHourStatisticsServiceImpl extends ServiceImpl<WorkHourStatisticsMapper, WorkHourStatistics> implements WorkHourStatisticsService {


    @Autowired
    private WorkHourStatisticsMapper workHourStatisticsMapper;

    @Override
    public IPage<WorkHourStatistics> selectPage(FrontPage<WorkHourStatistics> frontPage, WorkHourStatistics workHourStatistics) {
        QueryWrapper<WorkHourStatistics> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(workHourStatistics);
        return super.page(frontPage.getPagePlus(), queryWrapper);
    }

    @Override
    public List<WorkHourStatistics> selectList(WorkHourStatistics workHourStatistics) {
        QueryWrapper<WorkHourStatistics> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(workHourStatistics);
        return super.list(queryWrapper);
    }

    /**
     * 文件导入
     *
     * @param workHourStatisticsList
     * @return
     */
    @Override
    public List<WorkHourStatistics> workHourStatisticsFile(List<WorkHourStatistics> workHourStatisticsList) {
        String user = CommonMethods.getUser();
        String site = CommonMethods.getSite();
        LocalDateTime now = LocalDateTime.now();
        try {
            for (WorkHourStatistics workHourStatistics : workHourStatisticsList) {
                String handle = StringUtil.createQUID();
                workHourStatistics.setHandle(handle);
                workHourStatistics.setSite(site);
                workHourStatistics.setCreatedTime(now);
                workHourStatistics.setCreatedBy(user);
                workHourStatistics.setUpdatedBy(user);
                workHourStatistics.setUpdatedTime(now);
            }
            return workHourStatisticsList;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 临时维修导入
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public R workHourStatisticsFile(FileItem fileItem, HttpServletRequest req) {
        String fileType = req.getParameter("fileType");
        String site = req.getParameter("site");
        String user = req.getParameter("user");
        CommonMethods.setSite(site);
        CommonMethods.setUser(user);
        try {
            CommonsMultipartFile multipartFile = new CommonsMultipartFile(fileItem);
            List<WorkHourStatistics> workHourStatisticsList = ExcelUtils.readExcel(fileType, WorkHourStatistics.class, multipartFile, null);
            if (workHourStatisticsList.size() == 0){
                return R.failed("导出数据失败：0条数据");
            }
            workHourStatisticsList = this.workHourStatisticsFile(workHourStatisticsList);
            boolean result = this.saveBatch(workHourStatisticsList);
            return R.ok(result);
        } catch (Exception e) {
            e.printStackTrace();
            return R.failed(e.getMessage());
        }
    }

}