package com.foreverwin.mesnac.anomaly.service.impl;

import com.foreverwin.mesnac.anomaly.model.WxOrderDto;
import com.foreverwin.mesnac.anomaly.service.WxOrderService;
import com.foreverwin.mesnac.common.util.ExcelUtils;
import com.foreverwin.modular.core.exception.BaseException;
import com.foreverwin.modular.core.util.CommonMethods;
import com.foreverwin.modular.core.util.FrontPage;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.foreverwin.mesnac.anomaly.model.WxOrder;
import com.foreverwin.mesnac.anomaly.mapper.WxOrderMapper;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.foreverwin.modular.core.util.R;
import org.apache.commons.fileupload.FileItem;
import org.springframework.stereotype.Service;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.commons.CommonsMultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.util.*;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author YANG.WL
 * @since 2022-10-19
 */
@Service("wxOrderService")
@Transactional(rollbackFor = Exception.class)
public class WxOrderServiceImpl extends ServiceImpl<WxOrderMapper, WxOrder> implements WxOrderService {


    @Autowired
    private WxOrderMapper wxOrderMapper;

    @Override
    public IPage<WxOrder> selectPage(FrontPage<WxOrder> frontPage, WxOrder wxOrder) {
        QueryWrapper<WxOrder> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(wxOrder);
        return super.page(frontPage.getPagePlus(), queryWrapper);
    }

    @Override
    public List<WxOrder> selectList(WxOrder wxOrder) {
        QueryWrapper<WxOrder> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(wxOrder);
        return super.list(queryWrapper);
    }

    @Override
    public List<WxOrderDto> selectReceiveNumber() {
        return wxOrderMapper.selectReceiveNumber();
    }

    /**
     * 出库导入
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public R wxOrderListImportFile(FileItem fileItem, HttpServletRequest req) throws Exception {
        String fileType = req.getParameter("fileType");
        String site = req.getParameter("site");
        String user = req.getParameter("user");
        CommonMethods.setSite(site);
        CommonMethods.setUser(user);
        try {
            CommonsMultipartFile multipartFile = new CommonsMultipartFile(fileItem);
            List<WxOrder> wxOrderList = ExcelUtils.readExcel(fileType, WxOrder.class, multipartFile, null);
            if (wxOrderList.size() == 0){
                return R.failed("导入数据失败：0条数据");
            }

            if (this.saveBatch(wxOrderList)){
                return R.ok("导入数据成功");
            }else {
                return R.failed("导入数据失败");
            }
        } catch (Exception e) {
            e.printStackTrace();
            return R.failed("导入数据失败" + e.getMessage());
        }
    }

    @Override
    public boolean saveBatch(Collection<WxOrder> entityList) {
        String user=CommonMethods.getUser();
        LocalDateTime now = LocalDateTime.now();
        try {
            List<WxOrder> wxOrderList=new ArrayList<>();
            for (WxOrder wo: entityList) {
                wo.setHandle(UUID.randomUUID().toString());
                wo.setCreateUser(user);
                wo.setCreatedDateTime(now);
                wxOrderList.add(wo);
            }
            super.saveBatch(wxOrderList);
            return true;
        }catch (Exception e){
            e.printStackTrace();
        }
        return false;
    }
}