package com.foreverwin.mesnac.anomaly.service.impl;

import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.ValueFilter;
import com.foreverwin.mesnac.anomaly.dto.QualitySystemDto;
import com.foreverwin.mesnac.anomaly.model.AbnormalBill;
import com.foreverwin.mesnac.anomaly.model.AbnormalBillDispose;
import com.foreverwin.mesnac.anomaly.service.YunBpmService;
import com.foreverwin.mesnac.anomaly.utils.*;
import com.foreverwin.mesnac.common.constant.Constants;
import com.foreverwin.mesnac.common.util.DateUtil;
import com.foreverwin.mesnac.common.util.StringUtil;
import com.foreverwin.mesnac.meapi.util.StringUtils;
import com.foreverwin.modular.core.exception.BaseException;
import org.apache.http.Consts;
import org.apache.http.HttpEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.mime.HttpMultipartMode;
import org.apache.http.entity.mime.MultipartEntityBuilder;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.text.DecimalFormat;
import java.util.HashMap;
import java.util.Map;

/**
 * @Author YinQ
 * @create 2022-08-04 14:16
 */
@Service
public class YunBpmServiceImpl implements YunBpmService {
    private final Logger logger = LoggerFactory.getLogger(YunBpmServiceImpl.class);

    private final static String DEFAULT_ACTION_CODE = "submit";
    private final static Boolean DEFAULT_AGREE = true;
    private final static String DEFAULT_FORM_TYPE = "1";

    private static ValueFilter valueFilter = (o, s, data) -> data == "" ? null : data;
    @Override
    public YunBpmApiRes saveForm(Object formData, String owner, String ownerDeptId, String sheetCode, String workflowCode, String replayToken, String accessToken) {
        return saveOrUpdateForm(formData, owner, ownerDeptId, sheetCode, workflowCode, null, null, null, replayToken, accessToken);
    }

    @Override
    public YunBpmApiRes updateForm(Object formData, String sheetCode, String workflowCode, String workflowFormId, String workflowInstanceId, String workItemId, String replayToken, String accessToken) {
        return saveOrUpdateForm(formData, null, null, sheetCode, workflowCode, workflowFormId, workflowInstanceId, workItemId, replayToken, accessToken);
    }

    /**
     * 质量异常提报给质量bpm系统
     *
     * @param abnormalBill
     * @param ncGroup
     * @param ncCode
     * @return
     */
    public YunBpmApiRes saveQualitySystem(AbnormalBill abnormalBill, AbnormalBillDispose abnormalBillDispose, String ncGroup, String ncCode) {

        QualitySystemDto qualitySystemDto = QualitySystemDto.builder()
                //异常主数据
                .abnormalbill_no(abnormalBill.getAbnormalNo()).abnormalbill_workshop(abnormalBill.getWorkCenter())
                .material_code(abnormalBill.getItemBo().split(",")[1]).drawing_number(abnormalBill.getMapNo())
                .material_description(abnormalBill.getItemDescription()).shop_order(abnormalBill.getShopOrder())
                .product_barcode(abnormalBill.getSfc()).work_order(abnormalBill.getWorkOrder()).submit_date(DateUtil.getCurrentDate())
                .item_number(abnormalBill.getItemNumber()).product_category(abnormalBill.getProductCategory())
                .submission_users(abnormalBill.getReportSendUserGroup()).working_procedure_code(abnormalBill.getOperation())
                .submission_man_employee_no(abnormalBill.getPbUser()).test_user_employeeNo(abnormalBill.getInspector())
                .texture(abnormalBill.getTexture())
                //不良明细
                .problem_description(abnormalBill.getPbDescription()).problem_number(abnormalBill.getPbQty())
                .discover_link(abnormalBill.getDiscover()).entity_location(abnormalBill.getEntityLocation())
                .report_from(abnormalBill.getReportFrom()).rejects_number(abnormalBill.getNcQty())
                .bad_code_group(ncGroup).bad_code(ncCode).problem_level(abnormalBill.getPbGrade())
                // .problem_photos(null)
                 //责任划分
                 .cause_analysis(abnormalBillDispose.getDutyCauseDescription()).reason_type(abnormalBillDispose.getDutyCauseTypeDescription())
                 .responsibility_type(abnormalBillDispose.getDutyTypeDescription()).problem_owner_depart(abnormalBillDispose.getDutyDepartDescription())
                 .problem_owner_name(abnormalBillDispose.getPrincipalUser())
                 //.send_user_group()
                 /*//解决方案
                 .abnormalbill_programme().repair_process_route().repair_process_version()
                 .repair_workorder().repair_process_content().programme_remark()
                 //纠正预防
                 .abnormalbill_reason().preventive_measure().rectification_record_photos(null)*/
                 .build();

        String jsonString = JSONObject.toJSONString(qualitySystemDto,valueFilter);
        JSONObject qualitySystem = JSONObject.parseObject(jsonString);
        OAuthTokenInfo auth = getTokenByUserNameAndPassword(yunBpmConfig.userName, yunBpmConfig.passWord);
        if(null == auth){
            logger.error("连通bpm系统失败，账号：" + yunBpmConfig.userName +",密码："+ yunBpmConfig.passWord );
            throw new BaseException("连通bpm系统失败");
        }
        YunBpmApiRes tokenRes = getReplayToken(auth.getAccess_token());
        if (auth.getSuccess()) {
            YunBpmApiRes result = null;
            if (abnormalBill.getStatus().equals("N")){
                  //mes->bpm 提报
                try {
                    result = saveForm(qualitySystem, auth.getUser_id(), yunBpmConfig.deptId,
                          yunBpmConfig.workCode, yunBpmConfig.workCode, (String)tokenRes.getData(), auth.getAccess_token());
                    //保存操作日志
                    StringUtil.saveOperLog("BPM质量系统暂存接口，单号："  + abnormalBill.getAbnormalNo(),"saveForm",
                            qualitySystem.toString(),result.toString(),null);
                    String workflowInstanceId = JSON.parseObject(String.valueOf(result.getData())).getString("workflowInstanceId");
                    abnormalBill.setBpmFlowId(workflowInstanceId);
                    //mes->bpm 提交
                    String url = yunBpmConfig.getApiBaseUrl() + "/api/api/runtime/form/load?workflowInstanceId=" + abnormalBill.getBpmFlowId()
                            + "&access_token=" + auth.getAccess_token();
                    String res = HttpUtil.get(url);
                    JSONObject jsonObject = JSON.parseObject(res);
                    JSONObject jsonData = jsonObject.getJSONObject("data");
                    String workItemId = jsonData.getString("workItemId");
                    String workflowFormId = jsonData.getJSONObject("bizObject").getJSONObject("data").getString("id");
                    result = defaultSubmitForm(yunBpmConfig.workCode, yunBpmConfig.workCode,workflowFormId, abnormalBill.getBpmFlowId(),
                            workItemId,(String)tokenRes.getData(), auth.getAccess_token());
                    //保存操作日志
                    StringUtil.saveOperLog("BPM质量系统提交接口，单号："  + abnormalBill.getAbnormalNo(),"defaultSubmitForm",
                            "workflowFormId:" + workflowFormId + ",bpmFlowId:" + abnormalBill.getBpmFlowId()
                                    + ",workItemId:" + workItemId,result.toString(),null);
                } catch (Exception e) {
                    logger.error("bpm表单返回错误结果：" + e.toString(),result);
                }
            }else {
                String url = yunBpmConfig.getApiBaseUrl() + "/api/api/runtime/form/load?workflowInstanceId=" + abnormalBill.getBpmFlowId()
                        + "&access_token=" + auth.getAccess_token();
                String res = HttpUtil.get(url);
                JSONObject jsonObject = JSON.parseObject(res);
                JSONObject jsonData = jsonObject.getJSONObject("data");
                String workItemId = jsonData.getString("workItemId");
                String workflowFormId = jsonData.getJSONObject("bizObject").getJSONObject("data").getString("id");
                if (!abnormalBill.getStatus().equals(Constants.SHUT_DOWN)){
                    //保存
                    result = updateForm(qualitySystem, yunBpmConfig.workCode, yunBpmConfig.workCode,
                            workflowFormId, abnormalBill.getBpmFlowId(), workItemId,
                            (String) tokenRes.getData(), auth.getAccess_token());
                    //保存操作日志
                    StringUtil.saveOperLog("BPM质量系统保存接口，单号："  + qualitySystem.toString(),"updateForm",
                            qualitySystem.toString(),result.toString(),null);
                }else {
                    //闭关关闭
                    JSONObject jsonObj = new JSONObject();
                    jsonObj.put("abnormalbill_reason",abnormalBillDispose.getAbnormalReason());
                    jsonObj.put("preventive_measure",abnormalBillDispose.getBeforeMeasure());
                    jsonObj.put("mes_status",0);
                    result = updateForm(jsonObj, yunBpmConfig.workCode, yunBpmConfig.workCode,
                            workflowFormId, abnormalBill.getBpmFlowId(), workItemId,
                            (String) tokenRes.getData(), auth.getAccess_token());
                    //保存操作日志
                    StringUtil.saveOperLog("BPM质量系统关闭接口，单号："  + qualitySystem.toString(),"updateFormDown",
                            qualitySystem.toString(),result.toString(),null);
                }
            }
            logger.info("bpm表单返回结果：" + result.toString());
            return result;
        }
        return null;
    }

    /**
     * 保存或更新表单信息
     *
     * @param formData     表单信息 业务数据对象(建议在原本的实体类属性上挂@Alias()注解，直接转JSONObject传输)
     * @param owner        流程所有者 用户id
     * @param ownerDeptId  所有者部门 部门id
     * @param sheetCode    表单Code 固定值manufacture_cores
     * @param workflowCode 工作流Code 固定值manufacture_cores
     * @param replayToken  ReplayToken 调用接口时实时获取
     * @param accessToken  AccessToken 调用接口时实时获取
     * @return com.foreverwin.mesnac.anomaly.utils.YunBpmApis
     **/
    private YunBpmApiRes saveOrUpdateForm(Object formData, String owner, String ownerDeptId, String sheetCode,
                                          String workflowCode, String workflowFormId,
                                          String workflowInstanceId, String workItemId,
                                          String replayToken, String accessToken) {
        BizObjectModel bizObjectModel = BizObjectModel.builder()
                .data(formData)
                .owner(owner)
                .ownerDeptId(ownerDeptId)
                .sheetCode(sheetCode)
                .schemaCode(sheetCode)
                .workflowInstanceId(workflowInstanceId)
                .id(workflowFormId)
                .build();

        SaveFormModel saveFormModel = SaveFormModel.builder()
                .replayToken(replayToken)
                .workflowCode(workflowCode)
                .bizObject(bizObjectModel)
                .saveBizObject(true)
                .workItemId(workItemId)
                .workflowInstanceId(workflowInstanceId)
                .build();
        String requestUrl = yunBpmConfig.getApiBaseUrl() +
                String.format(YunBpmApis.SAVE_OR_UPDATE_FORM, accessToken);
        String jsons = JSONUtil.toJsonStr(saveFormModel);
        logger.info("保存或更新BPM表单信息：" + jsons);
        String respBody = HttpUtil.post(requestUrl, JSONUtil.toJsonStr(saveFormModel));
        return JSONUtil.toBean(respBody, YunBpmApiRes.class);
    }

    @Override
    public YunBpmApiRes defaultSubmitForm(String sheetCode, String workflowCode, String workflowFormId, String workflowInstanceId, String workItemId, String replayToken, String accessToken) {
        return submitForm(DEFAULT_ACTION_CODE,DEFAULT_AGREE,DEFAULT_FORM_TYPE, sheetCode, workflowCode, workflowFormId, workflowInstanceId, workItemId, replayToken, accessToken);
    }

    /**
     * 提交待办任务
     * @author donglt
     * @date 2021/8/14 23:55
     * @param actionCode 操作按钮编码，提交submit，同意agree，不同意disAgree
     * @param agree 是否同意, true:同意。false:不同意
     * @param formType 表单类型： 1：流程表单，2：业务表单
     * @param sheetCode 表单Code
     * @param workflowCode 工作流Code
     * @param workflowFormId 表单ID
     * @param workflowInstanceId 流程实例ID
     * @param workItemId 任务ID
     * @param replayToken ReplayToken
     * @param accessToken AccessToken
     * @return org.jeecg.modules.pq.yunbpm.bean.YunBpmApiRes
     **/
    private YunBpmApiRes submitForm(String actionCode,Boolean agree,String formType, String sheetCode,String workflowCode, String workflowFormId, String workflowInstanceId, String workItemId, String replayToken, String accessToken) {
        String requestUrl = yunBpmConfig.getApiBaseUrl() + String.format(YunBpmApis.FORM_SUBMIT,accessToken);
        Map<String, String> formData = new HashMap<>();
        formData.put("id", workflowFormId);

        BizObjectModel bizObjectModel = BizObjectModel.builder()
                .data(formData)
                .id(workflowFormId)
                .sheetCode(sheetCode)
                .schemaCode(sheetCode)
                .workflowInstanceId(workflowInstanceId)
                .build();

        SubmitFormModel submitFormModel = SubmitFormModel.builder()
                .actionCode(actionCode)
                .agree(agree)
                .formType(formType)
                .replayToken(replayToken)
                .workflowCode(workflowCode)
                .workItemId(workItemId)
                .workflowInstanceId(workflowInstanceId)
                .bizObject(bizObjectModel)
                .build();

        String respBody = HttpUtil.post(requestUrl, JSONUtil.toJsonStr(submitFormModel));
        return JSONUtil.toBean(respBody, YunBpmApiRes.class);
    }

    /**
     * 用户鉴权
     * @param userName
     * @param password
     * @return
     */
    public OAuthTokenInfo getTokenByUserNameAndPassword(String userName, String password) {
        YunBpmApiRes yunBpmApiRes = this.getOauthCode(userName, password, null);//获取授权码
        if (yunBpmConfig.SUCCESS_CODE.equals(yunBpmApiRes.getErrCode())) {
            return getTokenByOauthCode(yunBpmApiRes.getCode());//根据授权码获取token
        }
        return null;
    }

    public YunBpmApiRes getOauthCode(String userName, String password, String url) {
        String requestUrl = yunBpmConfig.getApiBaseUrl() + "/api/login/Authentication/get_code";
        if (StringUtils.isEmpty(url)) {
            url = this.getDefaultOauthCodeUrl();
        }

        Map<String, Object> params = new HashMap<>();
        params.put("username", userName);
        params.put("password", password);
        params.put("url", url);

        String respBody = HttpUtil.post(requestUrl, JSONUtil.toJsonStr(params));
        return JSONUtil.toBean(respBody, YunBpmApiRes.class);
    }

    private String getDefaultOauthCodeUrl() {
        return yunBpmConfig.getYunBpmUrl() + "/api/login?redirect_uri="
                + yunBpmConfig.getYunBpmUrl() +
                "/api/oauth/authorize?client_id=api&response_type=code&scope=read&redirect_uri="
                + yunBpmConfig.getYunBpmUrl() + "/oauth";
    }

    public OAuthTokenInfo getTokenByOauthCode(String oAuthCode) {
        String url = yunBpmConfig.getYunBpmUrl() + "/api";
        String redirectUri = yunBpmConfig.getYunBpmUrl() + "/oauth";
        String requestUrl = yunBpmConfig.getApiBaseUrl() + "/api/login/Authentication/get_token?code="
                + oAuthCode + "&client_secret=" + yunBpmConfig.getClientSecret() +
                "&client_id=" + yunBpmConfig.getClientId() + "&url=" + url + "&redirect_uri=" + redirectUri;
        String respBody = HttpUtil.get(requestUrl);
        return JSONUtil.toBean(respBody, OAuthTokenInfo.class);
    }


    /**
     * 上传附件
     *
     * @param urlStr http://yunbpmcs.mesnac.com:8080/api/api/aliyun/upload
     * @param file   文件类
     * @param token  accessToken 上传文件不同于其他接口，需要在请求头中使用authorization：Bearer 的方式挂载，注意Bearer和token中间有空格
     * @date 2021/8/14 22:34
     **/
    public String postFile(String urlStr, MultipartFile file, String token) throws IOException, InterruptedException {

        String prefix = file.getOriginalFilename();
        assert prefix != null;
        int lastIndex = prefix.lastIndexOf('.');
        String suffix = prefix.substring(lastIndex);
        CloseableHttpClient httpClient = HttpClients.createDefault();
        HttpPost uploadFile = new HttpPost(urlStr);

        uploadFile.setHeader("authorization", "Bearer " + token);
        DecimalFormat df = new DecimalFormat("#.##");
        MultipartEntityBuilder builder = MultipartEntityBuilder.create().setMode(HttpMultipartMode.RFC6532);

        builder.addTextBody("name", file.getOriginalFilename(), ContentType.create("text/plain", Consts.UTF_8));
        builder.addTextBody("size", df.format((double) file.getSize() / 1024), ContentType.TEXT_PLAIN);
        builder.addTextBody("suffix", suffix, ContentType.TEXT_PLAIN);

        builder.addBinaryBody(
                "file",
                file.getInputStream(),
                // new FileInputStream(f),
                ContentType.DEFAULT_BINARY,
                file.getOriginalFilename()
        );

        HttpEntity multipart = builder.build();
        uploadFile.setEntity(multipart);
        CloseableHttpResponse response = httpClient.execute(uploadFile);
        HttpEntity responseEntity = response.getEntity();
        String sResponse = EntityUtils.toString(responseEntity, "UTF-8");
        JSONObject jsonObject = JSONObject.parseObject(sResponse);

        return JSON.toJSONString(jsonObject);
    }

    /**
     * 获取replayToken接口
     *
     * @param accessToken
     * @return
     */
    private YunBpmApiRes getReplayToken(String accessToken) {
        String requestUrl = yunBpmConfig.getApiBaseUrl()
                + "/api/api/runtime/form/getReplayToken?access_token="
                + accessToken;
        HashMap<String, Object> map = new HashMap<>();
        map.put("access_token", accessToken);
        String respBody = HttpUtil.get(requestUrl, map);
        return JSONUtil.toBean(respBody, YunBpmApiRes.class);
    }


}
