package com.foreverwin.mesnac.common.controller;

import com.foreverwin.mesnac.common.model.OperLog;
import com.foreverwin.modular.core.util.R;
import com.foreverwin.modular.core.util.FrontPage;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import com.foreverwin.mesnac.common.service.OperLogService;
import java.util.List;

/**
 *
 * @author YinQ
 * @since 2023-02-01
 */
@RestController
@RequestMapping("/Z-OPER-LOG")
public class OperLogController {

    @Autowired
    public OperLogService operLogService;

    /**
    * 根据id查询
    *
    * @param id 主键
    * @return
    */
    @ResponseBody
    @GetMapping("/{id:.+}")
    public R getOperLogById(@PathVariable String id) {
        return R.ok( operLogService.getById(id));
    }

    /**
     * 查询所有数据
     *
     * @return
     */
    @ResponseBody
    @GetMapping("")
    public R getOperLogList(OperLog operLog){
        List<OperLog> result;
        QueryWrapper<OperLog> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(operLog);
        result = operLogService.list(queryWrapper);
        return R.ok(result);
    }

    /**
     * 分页查询数据
     *
     * @param frontPage  分页信息
     * @return
     */
    @ResponseBody
    @GetMapping("/page")
    public R page(FrontPage<OperLog> frontPage, OperLog operLog){
        IPage result;
        QueryWrapper<OperLog> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(operLog);
        result = operLogService.page(frontPage.getPagePlus(), queryWrapper);
        return R.ok(result);
    }

    /**
     * 新增
     * @param operLog  传递的实体
     * @return  null 失败  实体成功
     */
    @PostMapping
    public R save(@RequestBody OperLog operLog) {
        return R.ok(operLogService.save(operLog));
    }

    /**
     * 修改
     * @param operLog  传递的实体
     * @return  null 失败  实体成功
     */
    @PutMapping
    public R updateById(@RequestBody OperLog operLog) {
        return R.ok(operLogService.updateById(operLog));
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     * @return 0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.DELETE, value = "/{id:.+}")
    public R removeById(@PathVariable("id") String id){
        return R.ok(operLogService.removeById(id));
    }

    /**
     * 批量删除对象
     * @param ids 实体集合ID
     * @return  0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.POST, value = "/delete-batch")
    public R removeByIds(List<String> ids){
        return R.ok(operLogService.removeByIds(ids));
    }
}