package com.foreverwin.mesnac.common.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.foreverwin.mesnac.common.enums.HandleEnum;
import com.foreverwin.mesnac.common.ftp.DrawingFtpClient;
import com.foreverwin.mesnac.common.ftp.JSchClient;
import com.foreverwin.mesnac.common.ftp.WorkmanshipCardFtpClient;
import com.foreverwin.mesnac.common.service.FileService;
import com.foreverwin.mesnac.common.util.ExceptionUtil;
import com.foreverwin.mesnac.common.util.GetDrawingVersionUtil;
import com.foreverwin.mesnac.common.util.HttpUtils;
import com.foreverwin.mesnac.common.util.StringUtil;
import com.foreverwin.mesnac.meapi.model.Sfc;
import com.foreverwin.mesnac.meapi.model.SfcDispatchDrawing;
import com.foreverwin.mesnac.meapi.service.SfcDispatchDrawingService;
import com.foreverwin.mesnac.meapi.service.SfcService;
import com.foreverwin.modular.core.exception.BaseException;
import com.foreverwin.modular.core.util.CommonMethods;
import com.jcraft.jsch.Channel;
import com.jcraft.jsch.ChannelSftp;
import com.jcraft.jsch.Session;
import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPFile;
import org.jsoup.Connection;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.io.InputStream;
import java.net.InetAddress;
import java.util.*;
import java.util.regex.Pattern;

import static com.foreverwin.mesnac.common.util.GetDrawingVersionUtil.GetDrawingVersionSFTP;

@Service
public class FileServiceImpl implements FileService {
    @Autowired
    private SfcService sfcService;
    @Autowired
    private SfcDispatchDrawingService sfcDispatchDrawingService;
    @Autowired
    private DrawingFtpClient drawingFtpClient;
    @Autowired
    private JSchClient jSchClient;

    @Autowired
    private WorkmanshipCardFtpClient workmanshipCardFtpClient;


    @Override
    public Map<String, String> getFilePaths(String site, String sfc, String operation, String step) throws IOException {

        Sfc sfcServiceById = sfcService.getById(HandleEnum.SFC.getHandle(CommonMethods.getSite(), sfc));
        if (sfcServiceById == null) {
            throw new BaseException("未找到产品条码" + sfc);
        }
        SfcDispatchDrawing sfcDispatchDrawing = sfcDispatchDrawingService.findSfcDispatch(site, sfc, operation, step);
        if (sfcDispatchDrawing == null || sfcDispatchDrawing.getDrawingsRevision() == null) {
            throw new BaseException("未找到图纸相应信息");
        }
        String itemBo = sfcServiceById.getItemBo();
        String path = "/" + StringUtil.trimHandle(itemBo) + "_" + sfcDispatchDrawing.getDrawingsRevision() + "/";
        FTPClient connect = null;
        List<Integer> list = new ArrayList<Integer>();
        Map<String, String> pathMap = new HashMap<>();
        try {
            connect = drawingFtpClient.login();
            boolean flag = connect.changeWorkingDirectory(path);
            connect.enterLocalPassiveMode();
            FTPFile[] files = connect.listFiles();
            if (files != null && flag) {
                for (FTPFile ftpFile : files) {
                    String tmp = ftpFile.getName().substring(ftpFile.getName().lastIndexOf("_") + 1, ftpFile.getName().lastIndexOf("_") + 2);
                    Pattern pattern = Pattern.compile("^[-\\+]?[\\d]*$");
                    if (pattern.matcher(tmp).matches()) {
                        list.add(Integer.parseInt(tmp));
                    }
                    if (ftpFile.getName().equals(path.substring(1, path.length() - 1) + ".pdf") && ftpFile.isFile()) {
                        pathMap.put(ftpFile.getName(), path + ftpFile.getName());
                    }
                }
                int max = (int) Collections.max(list);
                for (FTPFile ftpFile : files) {
                    if (ftpFile.getName().substring(ftpFile.getName().lastIndexOf("_") + 1, ftpFile.getName().lastIndexOf("_") + 2).equals(String.valueOf(max)) && ftpFile.isFile()) {
                        pathMap.put(ftpFile.getName(), path + ftpFile.getName());
                    }
                }
            } else {
                throw new BaseException("未找到当前工序的图纸版本号和工艺卡片相应信息,请联系计划部门确认");
            }
            connect.disconnect();
        } catch (Exception e) {
            ExceptionUtil.throwException(e);
        } finally {
            if (connect != null && connect.isConnected()) {
                connect.logout();
                connect.disconnect();
            }
        }
        if (pathMap == null) {
            throw new BaseException("未找到当前工序的图纸版本号和工艺卡片相应信息,请联系计划部门确认");
        }
        return pathMap;
    }

    @Override
    public Map<String, String> getFilePathsByItemBo(String itemBo) throws IOException {
        FTPClient connect = null;
        Map<String, String> pathMap = new HashMap<>();
        List<Integer> list = new ArrayList<Integer>();
//        try {
//            connect = drawingFtpClient.login();
//            String vision = GetDrawingVersionUtil.GetDrawingVerson(itemBo, connect);
//            String path = "/" + StringUtil.trimHandle(itemBo) + "_" + vision + "/";
//            boolean flag = connect.changeWorkingDirectory(path);
//            connect.enterLocalPassiveMode();
//            FTPFile[] files = connect.listFiles();
//            if (files != null && flag) {
//                for (FTPFile ftpFile : files) {
//                    String tmp = ftpFile.getName().substring(ftpFile.getName().lastIndexOf("_") + 1, ftpFile.getName().lastIndexOf("_") + 2);
//                    Pattern pattern = Pattern.compile("^[-\\+]?[\\d]*$");
//                    if (pattern.matcher(tmp).matches()) {
//                        list.add(Integer.parseInt(tmp));
//                    }
//                    if (ftpFile.getName().equals(path.substring(1, path.length() - 1) + ".pdf") && ftpFile.isFile()) {
//                        pathMap.put(ftpFile.getName(), path + ftpFile.getName());
//                    }
//                }
//                int max = (int) Collections.max(list);
//                for (FTPFile ftpFile : files) {
//                    if (ftpFile.getName().substring(ftpFile.getName().lastIndexOf("_") + 1, ftpFile.getName().lastIndexOf("_") + 2).equals(String.valueOf(max)) && ftpFile.isFile()) {
//                        pathMap.put(ftpFile.getName(), path + ftpFile.getName());
//                    }
//                }
//            } else {
//                throw new BaseException("未找到当前工序的图纸版本号和工艺卡片相应信息,请联系计划部门确认");
//            }
//            connect.disconnect();
//        } catch (Exception e) {
//            ExceptionUtil.throwException(e);
//        } finally {
//            if (connect != null && connect.isConnected()) {
//                connect.logout();
//                connect.disconnect();
//            }
//        }
//        if (pathMap == null) {
//            throw new BaseException("未找到当前工序的图纸版本号和工艺卡片相应信息,请联系计划部门确认");
//        }
        return pathMap;
    }

    /**
     * 获取文件路径SFTP
     * 最新版本
     *
     * @param itemBo
     * @return
     * @throws Exception
     */
    @Override
    public Map<String, String> getFilePathsByItemBoSFTP(String itemBo,String type,String version) throws Exception {


        Map<String, String> pathMap = new HashMap<>();
        List<Integer> list = new ArrayList<Integer>();
        if (type.equals("CARD")){
            FTPClient connect = null;
            String[] itemBos = null;
            try {

               connect = workmanshipCardFtpClient.login();
                //获取路径
                itemBos = itemBo.split(",");
                for (String item:itemBos) {
                    if (version==null || version.equals("")){
                        version = GetDrawingVersionUtil.GetDrawingVerson(item, connect);
                    }
                    String path =   StringUtil.trimHandle(item) + "_" + version + "/";
                    boolean flag = connect.changeWorkingDirectory(path);
                    connect.enterLocalPassiveMode();
                    FTPFile[] files = connect.listFiles();
                    if (files.length!=0&& flag){
                        for (FTPFile ftpFile : files){
                            pathMap.put(ftpFile.getName(), path + ftpFile.getName());
                        }
                    }
                    else {
                        throw new BaseException("未找到工艺卡片相应信息,请联系工艺部门确认");
                    }

                }
                connect.disconnect();

            }catch (Exception e){
                ExceptionUtil.throwException(e);
            }
            finally {
                if (connect != null && connect.isConnected()) {
                    connect.logout();
                    connect.disconnect();
                }
            }

        }
        //看图纸
         if (type.equals("DRAWING")){
//            //初始化SFTP参数
//             int  timeOut =  3000 ;  //超时应该在3钞以上
//             boolean status = InetAddress.getByName(drawingFtpClient.getAddress()).isReachable(timeOut);
//             InetAddress localHost = InetAddress.getLocalHost();
//             System.out.println("主机名："+localHost.getHostName());
//             System.out.println("本地ip地址："+localHost.getHostAddress());
//             // 当返回值是true时，说明host是可用的，false则不可。
//           if (status){
//               throw new BaseException("主机名："+localHost.getHostName()+"本地ip地址："+localHost.getHostAddress()+"执行ping:"+drawingFtpClient.getAddress()+"的结果为"+status);
//           }else if (!status){
//               throw new BaseException("主机名："+localHost.getHostName()+"本地ip地址："+localHost.getHostAddress()+"执行ping:"+drawingFtpClient.getAddress()+"的结果为"+status);
//           }

            jSchClient = new JSchClient(drawingFtpClient.getAddress(), drawingFtpClient.getUserName(),
                    drawingFtpClient.getPassword(), drawingFtpClient.getPort());
            String[] itemBos = null;
            //多物料编码传入
            if (itemBo.contains(",")) {
                itemBos = itemBo.split(",");

                for (String item : itemBos) {
                    //通过SFTP连接服务器
                    jSchClient.connectSFTP();
                    if (version==null || version.equals("")){
                        version =GetDrawingVersionSFTP(item, jSchClient, drawingFtpClient.getDownloadDir());
                    }
                    if (null != version) {
                        //拼接文件路径
                        String ftpFilePath = drawingFtpClient.getDownloadDir() + "/" + item + "_" + version + "/" +
                                item + "_" + version + ".PDF";
                        pathMap.put(item + "_" + version + ".PDF", ftpFilePath);
                    }
                }

            } else {
                //单物料编码传入
                try {
                    //通过SFTP连接服务器
                    jSchClient.connectSFTP();
                    //获取版本号
                    if (version==null || version.equals("")){
                        version =GetDrawingVersionSFTP(itemBo, jSchClient, drawingFtpClient.getDownloadDir());
                    }
                    if (null != version) {
                        //拼接文件路径
                        String ftpFilePath = drawingFtpClient.getDownloadDir() + "/" + itemBo + "_" + version + "/" +
                                itemBo + "_" + version + ".PDF";
                        pathMap.put(itemBo + "_" + version + ".PDF", ftpFilePath);
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                } finally {
                    jSchClient.close();
                }
            }
            if (jSchClient != null){
                jSchClient.close();
            }
        }
         if (pathMap == null || pathMap.size() == 0) {
            throw new BaseException("未找到图纸相应信息,请在图纸管理系统申请！");
        }

        return pathMap;
    }

    /**
     * 获取指定文件流
     *
     * @param path
     * @return
     * @throws Exception
     */
    @Override
    public InputStream getSFTP(String path) throws Exception {
        //初始化输入流
        InputStream in;
        try {
            //初始化SFTP参数
//            jSchClient = new JSchClient(drawingFtpClient.getAddress(), drawingFtpClient.getUserName(),
//                    drawingFtpClient.getPassword(), drawingFtpClient.getPort());
            //通过SFTP连接服务器
            jSchClient.connectSFTP();
            in = jSchClient.getSFTP(path);
            return in;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

//    public static void main(String[] args) {
//        InputStream in;
//        try {
//            JSchClient jSchClient=new JSchClient("172.16.77.182", "drawing",
//                    "mesnac1104", 22);
//            jSchClient.connectSFTP();
//            in=jSchClient.downloadStream("/drawing/020801010100048_A/","020801010100048_A.PDF");
//          //  in = jSchClient.getSFTP("/drawing/020801010100048_A/020801010100048_A.PDF");
//
//        }catch (Exception e){
//            e.printStackTrace();
//        }
//
//    }


}
