package com.foreverwin.mesnac.dispatch.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.foreverwin.mesnac.common.dto.SfcDispatchDDto;
import com.foreverwin.mesnac.common.dto.SfcDispatchDto;
import com.foreverwin.mesnac.dispatch.dto.ResultData;
import com.foreverwin.mesnac.dispatch.dto.RouterDTO;
import com.foreverwin.mesnac.dispatch.model.SfcDispatch;
import com.foreverwin.mesnac.dispatch.service.SfcDispatchService;
import com.foreverwin.mesnac.meapi.dto.UploadPicturesDto;
import com.foreverwin.mesnac.meapi.service.RouterStepcontentService;
import com.foreverwin.modular.core.util.CommonMethods;
import com.foreverwin.modular.core.util.FrontPage;
import com.foreverwin.modular.core.util.R;
import org.apache.ibatis.annotations.Param;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.websocket.server.PathParam;
import java.util.*;
import java.util.stream.Collectors;

/**
 *
 * ClassName: SfcDispatchController
 *
 * @author Leon.L
 *
 * Copyright (C) 2021-2022 上海昊声电子信息技术有限公司.
 *
 * All Rights Reserved
 *
 * http://www.foreverwin.com.cn
 *
 * Create Time: 2021/06/02
 */
@RestController
@RequestMapping("/Z-SFC-DISPATCH")
public class SfcDispatchController {

    @Autowired
    public SfcDispatchService sfcDispatchService;
    @Autowired
    private RouterStepcontentService routerStepcontentService;


    /**
     * 查询派工数据
     *
     * @param paramMap
     * @return
     */
    @ResponseBody
    @GetMapping("findSfcDispatch")
    public R findSfcDispatch(@RequestParam(required = false) Map paramMap) {
        List<SfcDispatchDto> list;
        try {
            paramMap.put("site", CommonMethods.getSite());

            list = sfcDispatchService.findSfcDispatchList(paramMap);
        } catch (Exception e) {
            return R.failed(e.getMessage());
        }
        return R.ok(list);
    }

    @GetMapping("findSfcDispatchDD")
    public R findSfcDispatchDD(String sfc){
        List<SfcDispatchDDto> list;
        try {
            list=sfcDispatchService.findSfcDispatchDD(sfc);
            return R.ok(list);
        } catch (Exception e) {
            return R.failed(e.getMessage());
        }
    }


    @ResponseBody
    @PostMapping("/saveDispatch")
    public R saveDispatch(@RequestBody List<SfcDispatchDto> sfcDispatchList) {
        Boolean flag;
        StringBuffer message = new StringBuffer();
        try {
            String site = CommonMethods.getSite();
            String user = CommonMethods.getUser();
            flag = sfcDispatchService.saveDispatch(site, user, message, sfcDispatchList);
        } catch (Exception e) {
            return R.failed(e.getMessage());
        }
        if (flag) {
            message.insert(0, "派工保存成功");
            return R.ok(message.toString());
        } else {
            message.insert(0, "派工保存失败：");
            return R.failed(message.toString());
        }
    }

    /**
     * 派工发布
     *
     * @param sfcDispatchList
     * @return
     */
    @ResponseBody
    @PostMapping("/releaseDispatch")
    public R releaseDispatch(@RequestBody List<SfcDispatchDto> sfcDispatchList) {
        Boolean flag;
        StringBuffer message = new StringBuffer();
        try {
            String site = CommonMethods.getSite();
            String user = CommonMethods.getUser();
            flag = sfcDispatchService.releaseDispatch(site, user, message, sfcDispatchList);
        } catch (Exception e) {
            return R.failed("派工发布失败：" + e.getMessage());
        }



        if (flag) {
            message.insert(0, "派工发布成功");
            return R.ok(message.toString());
        } else {
            message.insert(0, "派工发布失败：");
            return R.failed(message.toString());
        }
    }

    /**
     * 返回新建状态
     *
     * @param sfcDispatchList
     * @return
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.POST, value = "/returnToNew")
    public R returnToNew(@RequestBody List<SfcDispatchDto> sfcDispatchList) {
        String user = CommonMethods.getUser();
        try {
            sfcDispatchService.returnToNew(user, sfcDispatchList);
        } catch (Exception e) {
            return R.failed(e.getMessage());
        }

        return R.ok();
    }

    /**
     * 暂挂
     *
     * @param sfcDispatchList
     * @return
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.POST, value = "/pause")
    public R pause(@RequestBody List<SfcDispatchDto> sfcDispatchList) {
        String user = CommonMethods.getUser();
        try {
            sfcDispatchService.pause(user, sfcDispatchList);
        } catch (Exception e) {
            return R.failed(e.getMessage());
        }

        return R.ok();
    }

    /**
     * 取消
     *
     * @param sfcDispatchList
     * @return
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.POST, value = "/cancel")
    public R cancel(@RequestBody List<SfcDispatchDto> sfcDispatchList) {
        String user = CommonMethods.getUser();
        try {
            sfcDispatchService.cancel(user, sfcDispatchList);
        } catch (Exception e) {
            return R.failed(e.getMessage());
        }

        return R.ok();
    }

    @PostMapping("/import")
    public R importFile(@PathParam("fileType") String fileType, @Param("site") String site, @Param("user") String user, @RequestParam("file") MultipartFile multipartFile) {
        Boolean flag;
        StringBuffer message = new StringBuffer();
        try {
            flag = sfcDispatchService.importDispatch(site, user, fileType, message, multipartFile);
        } catch (Exception e) {
            return R.failed(e.getMessage());
        }


        if (flag) {
            message.insert(0, "派工数据导入成功");
            return R.ok(message.toString());
        } else {
            message.insert(0, "派工数据导入失败：");
            return R.failed(message.toString());
        }
    }

    @ResponseBody
    @GetMapping("/page")
    public R page(FrontPage<SfcDispatch> frontPage, SfcDispatch sfcDispatch){
        IPage result;
        QueryWrapper<SfcDispatch> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(sfcDispatch);
        sfcDispatch.setSite(CommonMethods.getSite());
        if (frontPage.getGlobalQuery() != null && !"".equals(frontPage.getGlobalQuery().trim())) {
            //TODO modify global query
            queryWrapper.lambda().and(wrapper -> wrapper
                    .or().like(SfcDispatch::getShopOrder, frontPage.getGlobalQuery())
                    .or().like(SfcDispatch::getSfc, frontPage.getGlobalQuery())
                    .or().like(SfcDispatch::getDispatchNo, frontPage.getGlobalQuery())
                    .or().like(SfcDispatch::getDispatchStatus, frontPage.getGlobalQuery())
            );
        }
        result = sfcDispatchService.page(frontPage.getPagePlus(), queryWrapper);
        return R.ok(result);
    }


    @GetMapping("/gantt-list")
    public R ganttList(@RequestParam(required = false) Map paramMap) {
        Map<String, Object> result = new HashMap<>();

        try {
            String site = CommonMethods.getSite();
            paramMap.put("site", site);
            paramMap.put("status", "('RELEASE','START','COMPLETE')");
            result = sfcDispatchService.ganttList(site, paramMap);
        } catch (Exception e) {
            return R.failed(e.getMessage());
        }

        return R.ok(result);
    }

    //查询返修工艺内容
    @ResponseBody
    @PostMapping("/findRepairContent")
    public R findRepairContent(@RequestBody List<SfcDispatchDto> sfcDispatchList) {
        try {
            Map paramMap=new HashMap();
            Map distinctClassMap=new HashMap();
            UploadPicturesDto uploadPicturesDto=new UploadPicturesDto();
            paramMap.put("site", CommonMethods.getSite());
            String msg="";
            String pbdes="";
            String url="";
            List<Map<String, Object>> result=null;
            List<String> mapList=new ArrayList<>();
            ResultData resultData=new ResultData();

          //  ResultData resultData=new ResultData();
            List<SfcDispatchDto> distinctClass = sfcDispatchList.stream().collect(Collectors.collectingAndThen(Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(o ->
                    o.getShopOrder() + ";" + o.getSfc()))), ArrayList::new));
            for (SfcDispatchDto sfcdto: distinctClass) {
                distinctClassMap.put("SHOP_ORDER",sfcdto.getShopOrder());
//                distinctClassMap.put("SFC","");
//                distinctClassMap.put("ITEM_BO",sfcdto.getItemBo());

                List<String> stringList= routerStepcontentService.pbDescription(distinctClassMap);
                uploadPicturesDto = routerStepcontentService.findUploadPictureByRouterBo(sfcdto.getRouterBo(),sfcdto.getStepId());
                if (stringList!=null&&stringList.size()!=0){
                    for (int i = 0; i < stringList.size(); i++) {
                        pbdes=pbdes+stringList.get(i);
                    }
                }
            }
            resultData.setQuestion(pbdes);
            if (uploadPicturesDto!=null){
                resultData.setPrctureAdress(uploadPicturesDto.getPictureAddress());
            }
            for (int i = 0; i < sfcDispatchList.size(); i++) {
                paramMap.put("routerStepBo", "RouterStepBO:" + sfcDispatchList.get(i).getRouterBo() + "," + sfcDispatchList.get(i).getStepId());
                result = routerStepcontentService.selectDisRouterInfo(paramMap);

                if (result != null && result.size() != 0) {

                    for (int j = 0; j < result.size(); j++) {
                        msg = msg + "【" + result.get(j).get("STEP_ID") + "】" + result.get(j).get("ROUTER_STEP_CONTENT");
                    }
                }
            }
            resultData.setRouter(msg);
            return R.ok(resultData);
        }catch (Exception e){
            return R.failed(e.getMessage());
        }

    }


}