package com.foreverwin.mesnac.meapi.controller;

import com.foreverwin.modular.core.util.R;
import com.foreverwin.modular.core.util.FrontPage;
import com.foreverwin.modular.core.util.CommonMethods;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import com.foreverwin.mesnac.meapi.service.LocationService;
import com.foreverwin.mesnac.meapi.model.Location;
import java.util.List;

/**
 *
 * @author YANG.WL
 * @since 2022-08-04
 */
@RestController
@RequestMapping("/Z-LOCATION")
public class LocationController {

    @Autowired
    public LocationService locationService;

    /**
    * 根据id查询
    *
    * @param id 主键
    * @return
    */
    @ResponseBody
    @GetMapping("/{id:.+}")
    public R getLocationById(@PathVariable String id) {
        return R.ok( locationService.getById(id));
    }

    /**
     * 查询所有数据
     *
     * @return
     */
    @ResponseBody
    @GetMapping("")
    public R getLocationList(Location location){
        List<Location> result;
        QueryWrapper<Location> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(location);
        result = locationService.list(queryWrapper);
        return R.ok(result);
    }

    /**
     * 分页查询数据
     *
     * @param frontPage  分页信息
     * @return
     */
    @ResponseBody
    @GetMapping("/page")
    public R page(FrontPage<Location> frontPage, Location location){
        IPage result;
        QueryWrapper<Location> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(location);
        if (frontPage.getGlobalQuery() != null && !"".equals(frontPage.getGlobalQuery().trim())) {
            //TODO modify global query
            queryWrapper.lambda().and(wrapper -> wrapper
                .like(Location::getHandle, frontPage.getGlobalQuery())
                .or().like(Location::getLocationCode, frontPage.getGlobalQuery())
                .or().like(Location::getWorkCenter, frontPage.getGlobalQuery())
                .or().like(Location::getDescription, frontPage.getGlobalQuery())
    );
        }
        result = locationService.page(frontPage.getPagePlus(), queryWrapper);
        return R.ok(result);
    }

    /**
     * 新增
     * @param location  传递的实体
     * @return  null 失败  实体成功
     */
    @PostMapping
    public R save(@RequestBody Location location) {
        return R.ok(locationService.save(location));
    }

    /**
     * 修改
     * @param location  传递的实体
     * @return  null 失败  实体成功
     */
    @PutMapping
    public R updateById(@RequestBody Location location) {
        return R.ok(locationService.updateById(location));
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     * @return 0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.DELETE, value = "/{id:.+}")
    public R removeById(@PathVariable("id") String id){
        return R.ok(locationService.removeById(id));
    }

    /**
     * 批量删除对象
     * @param ids 实体集合ID
     * @return  0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.POST, value = "/delete-batch")
    public R removeByIds(List<String> ids){
        return R.ok(locationService.removeByIds(ids));
    }

    @ResponseBody
    @GetMapping("/LOCATION")
    public Location getLocation(String code){
        Location location=new Location();
        location.setLocationCode(code);
        QueryWrapper<Location> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(location);
        location = locationService.getOne(queryWrapper);
        return location;
    }
}