package com.foreverwin.mesnac.meapi.controller;

import com.foreverwin.modular.core.util.R;
import com.foreverwin.modular.core.util.FrontPage;
import com.foreverwin.modular.core.util.CommonMethods;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import com.foreverwin.mesnac.meapi.service.LogisticsBillService;
import com.foreverwin.mesnac.meapi.model.LogisticsBill;
import java.util.List;

/**
 *
 * @author YANG.WL
 * @since 2022-08-05
 */
@RestController
@RequestMapping("/Z-LOGISTICS-BILL")
public class LogisticsBillController {

    @Autowired
    public LogisticsBillService logisticsBillService;

    /**
    * 根据id查询
    *
    * @param id 主键
    * @return
    */
    @ResponseBody
    @GetMapping("/{id:.+}")
    public R getLogisticsBillById(@PathVariable String id) {
        return R.ok( logisticsBillService.getById(id));
    }

    /**
     * 查询所有数据
     *
     * @return
     */
    @ResponseBody
    @GetMapping("")
    public R getLogisticsBillList(LogisticsBill logisticsBill){
        List<LogisticsBill> result;
        QueryWrapper<LogisticsBill> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(logisticsBill);
        result = logisticsBillService.list(queryWrapper);
        return R.ok(result);
    }

    /**
     * 分页查询数据
     *
     * @param frontPage  分页信息
     * @return
     */
    @ResponseBody
    @GetMapping("/page")
    public R page(FrontPage<LogisticsBill> frontPage, LogisticsBill logisticsBill){
        IPage result;
        QueryWrapper<LogisticsBill> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(logisticsBill);
        if (frontPage.getGlobalQuery() != null && !"".equals(frontPage.getGlobalQuery().trim())) {
            //TODO modify global query
            queryWrapper.lambda().and(wrapper -> wrapper
                .like(LogisticsBill::getLogisticsBillNo, frontPage.getGlobalQuery())
                .or().like(LogisticsBill::getLogisticsBo, frontPage.getGlobalQuery())
                .or().like(LogisticsBill::getOther1, frontPage.getGlobalQuery())
                .or().like(LogisticsBill::getOther2, frontPage.getGlobalQuery())
                .or().like(LogisticsBill::getOther3, frontPage.getGlobalQuery())
    );
        }
        result = logisticsBillService.page(frontPage.getPagePlus(), queryWrapper);
        return R.ok(result);
    }

    /**
     * 新增
     * @param logisticsBill  传递的实体
     * @return  null 失败  实体成功
     */
    @PostMapping
    public R save(@RequestBody LogisticsBill logisticsBill) {
        return R.ok(logisticsBillService.save(logisticsBill));
    }

    /**
     * 修改
     * @param logisticsBill  传递的实体
     * @return  null 失败  实体成功
     */
    @PutMapping
    public R updateById(@RequestBody LogisticsBill logisticsBill) {
        return R.ok(logisticsBillService.updateById(logisticsBill));
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     * @return 0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.DELETE, value = "/{id:.+}")
    public R removeById(@PathVariable("id") String id){
        return R.ok(logisticsBillService.removeById(id));
    }

    /**
     * 批量删除对象
     * @param ids 实体集合ID
     * @return  0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.POST, value = "/delete-batch")
    public R removeByIds(List<String> ids){
        return R.ok(logisticsBillService.removeByIds(ids));
    }
}