package com.foreverwin.mesnac.meapi.controller;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.foreverwin.mesnac.meapi.dto.LogisticsDto;
import com.foreverwin.mesnac.meapi.enums.HandleEnum;
import com.foreverwin.mesnac.meapi.model.Location;
import com.foreverwin.mesnac.meapi.model.LogisticsBill;
import com.foreverwin.mesnac.meapi.model.StoreSfcInfo;
import com.foreverwin.mesnac.meapi.model.UserGroup;
import com.foreverwin.mesnac.meapi.service.LogisticsBillService;
import com.foreverwin.mesnac.meapi.service.UserGroupService;
import com.foreverwin.modular.core.exception.BaseException;
import com.foreverwin.modular.core.util.R;
import com.foreverwin.modular.core.util.FrontPage;
import com.foreverwin.modular.core.util.CommonMethods;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.apache.ibatis.annotations.Param;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import com.foreverwin.mesnac.meapi.service.LogisticsTurnoverService;
import com.foreverwin.mesnac.meapi.model.LogisticsTurnover;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.UUID;


/**
 *
 * @author YANG.WL
 * @since 2022-08-08
 */
@RestController
@RequestMapping("/Z-LOGISTICS-TURNOVER")
public class LogisticsTurnoverController {

    @Autowired
    public LogisticsTurnoverService logisticsTurnoverService;
    @Autowired
    public LogisticsBillService logisticsBillService;
    @Autowired
    private UserGroupService userGroupService;
    /**
    * 根据id查询
    *
    * @param id 主键
    * @return
    */
    @ResponseBody
    @GetMapping("/{id:.+}")
    public R getLogisticsTurnoverById(@PathVariable String id) {
        return R.ok( logisticsTurnoverService.getById(id));
    }

    /**
     * 查询所有数据
     *
     * @return
     */
    @ResponseBody
    @GetMapping("")
    public R getLogisticsTurnoverList(LogisticsTurnover logisticsTurnover){
        List<LogisticsTurnover> result;
        QueryWrapper<LogisticsTurnover> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(logisticsTurnover);
        result = logisticsTurnoverService.list(queryWrapper);
        return R.ok(result);
    }

    /**
     * 分页查询数据
     *
     * @param frontPage  分页信息
     * @return
     */
    @ResponseBody
    @GetMapping("/page")
    public R page(FrontPage<LogisticsTurnover> frontPage, LogisticsTurnover logisticsTurnover){
        IPage result;
        QueryWrapper<LogisticsTurnover> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(logisticsTurnover);
        if (frontPage.getGlobalQuery() != null && !"".equals(frontPage.getGlobalQuery().trim())) {
            //TODO modify global query
            queryWrapper.lambda().and(wrapper -> wrapper
                .like(LogisticsTurnover::getHandle, frontPage.getGlobalQuery())
                .or().like(LogisticsTurnover::getSfcDispatchBo, frontPage.getGlobalQuery())
                .or().like(LogisticsTurnover::getStatus, frontPage.getGlobalQuery())
                .or().like(LogisticsTurnover::getuserId, frontPage.getGlobalQuery())
                .or().like(LogisticsTurnover::getUserName, frontPage.getGlobalQuery())
                .or().like(LogisticsTurnover::getOther1, frontPage.getGlobalQuery())
                .or().like(LogisticsTurnover::getOther2, frontPage.getGlobalQuery())
                .or().like(LogisticsTurnover::getOther3, frontPage.getGlobalQuery())
                .or().like(LogisticsTurnover::getStorageLocation, frontPage.getGlobalQuery())
                .or().like(LogisticsTurnover::getTurnoverWorkCenter, frontPage.getGlobalQuery())
                .or().like(LogisticsTurnover::getIsReceive, frontPage.getGlobalQuery())
                .or().like(LogisticsTurnover::getTurnoutWorkcenter, frontPage.getGlobalQuery())
                .or().like(LogisticsTurnover::getTurnoutItem, frontPage.getGlobalQuery())
                .or().like(LogisticsTurnover::getOther4, frontPage.getGlobalQuery())
                .or().like(LogisticsTurnover::getOther5, frontPage.getGlobalQuery())
                .or().like(LogisticsTurnover::getOther6, frontPage.getGlobalQuery())
    );
        }
        result = logisticsTurnoverService.page(frontPage.getPagePlus(), queryWrapper);
        return R.ok(result);
    }




    /**
     * 查询数据
     *
     * @param logisticsDto
     * @return
     */
    @ResponseBody
    @GetMapping("/querylogisticsturnovelist")
    public R querylogisticsturnovelist(LogisticsDto logisticsDto){
        List<LogisticsDto> result;
        result = logisticsTurnoverService.selectlogisticsturnoverlist(logisticsDto);
        return R.ok(result);
    }

    /**
     * 新增
     * @param logisticsTurnover  传递的实体
     * @return  null 失败  实体成功
     */
    @PostMapping
    public R save(@RequestBody LogisticsTurnover logisticsTurnover) {
        String site=CommonMethods.getSite();
        if (logisticsTurnover != null){
            logisticsTurnover.setHandle(UUID.randomUUID().toString());
            logisticsTurnover.setSfcDispatchBo(HandleEnum.SFC_DISPATCH.getHandle(site, logisticsTurnover.getSfc()+"-"+logisticsTurnover.getOther2()));
            logisticsTurnover.setStatus("NEW");
            logisticsTurnover.setCreatedDateTime(LocalDateTime.now());
            logisticsTurnover.setIsReceive("N");
        }
        return R.ok(logisticsTurnoverService.save(logisticsTurnover));
    }

    /**
     * 修改
     * @param logisticsTurnover  传递的实体
     * @return  null 失败  实体成功
     */
    @PutMapping
    public R updateById(@RequestBody LogisticsTurnover logisticsTurnover) {
        return R.ok(logisticsTurnoverService.updateById(logisticsTurnover));
    }

    /**
     * 根据id删除对象
     * @param   logisticsDtoList
     * @return 0 失败  1 成功
     */
    @ResponseBody
    @PostMapping("/delete")
    public R removeById(@RequestBody List<LogisticsDto> logisticsDtoList){
        if (logisticsTurnoverService.deleteLogistics(logisticsDtoList)){
            return R.ok();
        }else {
            return R.failed("删除失败");
        }




//        try {
//            LogisticsTurnover logisticsTurnover=new LogisticsTurnover();
//            List<LogisticsTurnover> logisticsTurnoverList=new ArrayList<>();
//            if (logisticsDtoList.size()>0){
//                for (LogisticsDto logisticsDto:logisticsDtoList){
//                    logisticsTurnover.setHandle(logisticsDto.getHandle());
//                    logisticsTurnover.setOther6("1");
//                    logisticsTurnoverList.add(logisticsTurnover);
//                }
//                if (logisticsTurnoverService.saveOrUpdateBatch(logisticsTurnoverList)){
//                    return R.ok();
//                }else {
//                    return R.failed();
//                }
//            }
//        }catch (Exception e){
//            throw new BaseException("删除错误:"+e.getMessage());
//        }


    }

    /**
     * 批量删除对象
     * @param ids 实体集合ID
     * @return  0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.POST, value = "/delete-batch")
    public R removeByIds(List<String> ids){
        return R.ok(logisticsTurnoverService.removeByIds(ids));
    }

    /**
     * 查询所有数据
     *
     * @return
     */
    @ResponseBody
    @GetMapping("/queryLogisticsTurnoverList")
    public String queryLogisticsTurnoverList(String  login_name){
        //查询登录人员所在的用户组

        String userBo = HandleEnum.USER.getHandle("1000", login_name);
        List<UserGroup> userGroupList = userGroupService.getUserGroupListByUserBo(userBo);
        List<LogisticsDto> result=new ArrayList<>();
        if (userGroupList != null && userGroupList.size() > 0) {
            for (UserGroup userGroupModel: userGroupList) {
                String userGroup = userGroupModel.getUserGroup();
                if ("WL-HJ-HJ".equals(userGroup)) {
                    result = logisticsTurnoverService.queryLogisticsTurnoverList(userGroup,null,null,null,null,null);
                    break;
                } else if ("WL-HJ-XL".equals(userGroup)) {
                    result = logisticsTurnoverService.queryLogisticsTurnoverList(null,userGroup,null,null,null,null);
                    break;
                } else if ("WL-JJ-JJ".equals(userGroup)) {
                    result = logisticsTurnoverService.queryLogisticsTurnoverList(null,null,userGroup,null,null,null);
                    break;
                } else if ("WL-WX-WX".equals(userGroup)) {
                    result = logisticsTurnoverService.queryLogisticsTurnoverList(null,null,null,userGroup,null,null);
                    break;
                } else if ("WL-RC-RC".equals(userGroup)) {
                    result = logisticsTurnoverService.queryLogisticsTurnoverList(null,null,null,null,userGroup,null);
                    break;
                } else if ("WL-ZP-ZP".equals(userGroup)) {
                    result = logisticsTurnoverService.queryLogisticsTurnoverList(null,null,null,null,null,userGroup);
                    break;
                }

            }
            return Optional.ofNullable(result)
                    .map(t -> JSONObject.toJSONString(t)).orElse("null");
        }else {
            return null;
        }


//        QueryWrapper<LogisticsTurnover> queryWrapper = new QueryWrapper<>();
//        queryWrapper.setEntity(logisticsTurnover);

    }





    /**
     * 批量修改对象
     * @param list 实体集合ID
     * @return  0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.POST, value = "/UPDATE-STATUS")
    public R updateStatusById(String list,String location,String transportno,String user,String username) {
        List<LogisticsDto> logisticsDtoList = JSONArray.parseArray(list, LogisticsDto.class);
        Location lt=JSONArray.parseObject(location,Location.class);
        Boolean flag;
        StringBuffer message = new StringBuffer();
        try {
            if (!logisticsDtoList.isEmpty()){
                flag=logisticsTurnoverService.updateLogistics(logisticsDtoList,lt,message,transportno,user,username);
            }else {
                flag=false;
            }
        }catch (Exception e){
            return R.failed("转运出错");
        }
        if (flag) {
            message.insert(0, "转运成功");
            return R.ok(message.toString());
        } else {
            message.insert(0, "转运出错,请检查扫描的库位与转入车间是否匹配，条码：");
            return R.failed(message.toString());
        }

    }

    /**
     * 批量修改对象
     * @param logisticsDtoList 实体集合ID
     * @return  0 失败  1 成功
     */
    @ResponseBody
    @PostMapping("/updatelogisticsTurnover")
    public R updatelogisticsTurnover(@RequestBody List<LogisticsDto> logisticsDtoList) {
//        LogisticsTurnover logisticsTurnover = new LogisticsTurnover();
//        StringBuffer message = new StringBuffer();
//        boolean flag=true;
//        if (!logisticsDtoList.isEmpty()) {
//            try {
//                int seq = 0;
//                for (LogisticsDto logisticsDto : logisticsDtoList) {
//                    if (!logisticsDto.getStatus().equals("COM")) {
//                        flag=false;
//                        message.append(seq++ % 3 == 0 ? "\n" : "|   |");
//                        message.append("SFC[" + logisticsDto.getSfc() + "]未转运完成，不允许车间接收！");
//                        continue;
//                    } else {
//                        logisticsTurnover.setHandle(logisticsDto.getHandle());
//                        logisticsTurnover.setIsReceive("Y");
//                        logisticsTurnover.setOther4(LocalDateTime.now());
//                        logisticsTurnover.setOther5(CommonMethods.getUser());
//                        logisticsTurnoverService.updateById(logisticsTurnover);
//                    }
//
//                }
//            } catch (Exception e) {
//                return R.failed(e.getMessage());
//            }
//        }

        Boolean flag;
        StringBuffer message = new StringBuffer();
        try {
            if (!logisticsDtoList.isEmpty()){
                flag = logisticsTurnoverService.Receivelogistics( message, logisticsDtoList);
            }else {
                flag=false;
            }

        } catch (Exception e) {
            return R.failed(e.getMessage());
        }
        if (flag) {
            message.insert(0, "接收物料成功");
            return R.ok(message.toString());
        } else {
            message.insert(0, "接收物料失败：");
            return R.failed(message.toString());
        }

    }

    @PostMapping("/selectSFC")
    @ResponseBody
    public String selectSFC(String sfc){
       StoreSfcInfo storeSfcInfo= logisticsTurnoverService.storeSelectSfc(sfc);
       return Optional.ofNullable(storeSfcInfo).map(t->JSONObject.toJSONString(t)).orElse("null");
    }
    @PostMapping("/submentInStore")
    @ResponseBody
    public String selectSFC(String sfc,double qty,int inStoreType,String user,
                            @RequestParam(value = "storeLocation",defaultValue = "") String storeLocation){

       return  logisticsTurnoverService.submentInStore(sfc,qty, inStoreType, user,storeLocation);
    }

}