package com.foreverwin.mesnac.meapi.service.impl;

import com.foreverwin.mesnac.meapi.dto.LogisticsDto;
import com.foreverwin.mesnac.meapi.mapper.LogisticsBillMapper;
import com.foreverwin.mesnac.meapi.model.Location;
import com.foreverwin.mesnac.meapi.model.LogisticsBill;
import com.foreverwin.mesnac.meapi.model.StoreSfcInfo;
import com.foreverwin.modular.core.exception.BaseException;
import com.foreverwin.modular.core.util.CommonMethods;
import com.foreverwin.modular.core.util.FrontPage;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.foreverwin.mesnac.meapi.model.LogisticsTurnover;
import com.foreverwin.mesnac.meapi.mapper.LogisticsTurnoverMapper;
import com.foreverwin.mesnac.meapi.service.LogisticsTurnoverService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.foreverwin.modular.core.util.R;
import org.springframework.stereotype.Service;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author YANG.WL
 * @since 2022-08-08
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class LogisticsTurnoverServiceImpl extends ServiceImpl<LogisticsTurnoverMapper, LogisticsTurnover> implements LogisticsTurnoverService {


    @Autowired
    private LogisticsTurnoverMapper logisticsTurnoverMapper;
    @Autowired
    private LogisticsBillMapper logisticsBillMapper;


    @Override
    public IPage<LogisticsTurnover> selectPage(FrontPage<LogisticsTurnover> frontPage, LogisticsTurnover logisticsTurnover) {
        QueryWrapper<LogisticsTurnover> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(logisticsTurnover);
        return super.page(frontPage.getPagePlus(), queryWrapper);
    }

    @Override
    public List<LogisticsTurnover> selectList(LogisticsTurnover logisticsTurnover) {
        QueryWrapper<LogisticsTurnover> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(logisticsTurnover);
        return super.list(queryWrapper);
    }

    @Override
    public List<LogisticsDto> queryLogisticsTurnoverList(String HJHJ,String HJXL,String JJ,String WX,String RC,String ZP) {
        return logisticsTurnoverMapper.queryLogisticsTurnoverList(HJHJ,HJXL,JJ,WX,RC,ZP);
    }

    @Override
    public LogisticsTurnover queryNewtistTurnoverTask(String sfcDispatchBo) {
        return logisticsTurnoverMapper.queryNewtistTurnoverTask(sfcDispatchBo);
    }

    @Override
    public IPage<LogisticsDto> selectLogisticsDtoPage(FrontPage<LogisticsDto> frontPage, LogisticsDto logisticsDto) {
        return logisticsTurnoverMapper.selectLogisticsDtoPage(frontPage,logisticsDto);
    }

    @Override
    public List<LogisticsDto> selectlogisticsturnoverlist(LogisticsDto logisticsDto) {
        return logisticsTurnoverMapper.selectlogisticsturnoverlist(logisticsDto);
    }

    @Override
    public Boolean Receivelogistics(StringBuffer message, List<LogisticsDto> logisticsDtoList) {
        Boolean flag = true;
        int seq = 0;
        LogisticsTurnover logisticsTurnover = new LogisticsTurnover();
        List<LogisticsTurnover> logisticsTurnoverList = new ArrayList<>();
        for (LogisticsDto logisticsDto:logisticsDtoList ) {
                   if (!logisticsDto.getStatus().equals("COM")) {
                        flag=false;
                        message.append(seq++ % 3 == 0 ? "\n" : "|   |");
                        message.append("SFC[" + logisticsDto.getSfc() + "]未转运完成，不允许车间接收！");
                        continue;
                    } else {
                        logisticsTurnover.setHandle(logisticsDto.getHandle());
                        logisticsTurnover.setIsReceive("Y");
                        logisticsTurnover.setOther4(LocalDateTime.now());
                        logisticsTurnover.setOther5(CommonMethods.getUser());
                       logisticsTurnoverList.add(logisticsTurnover);
                    }
                   this.updateById(logisticsTurnover);
        }

        return flag;
    }

    @Override
    public boolean updateLogistics(List<LogisticsDto> logisticsDtoList, Location lt,StringBuffer message, String transportno, String user, String username) {

                LogisticsTurnover logisticsTurnover = new LogisticsTurnover();
                LogisticsBill logisticsBill = new LogisticsBill();
                Boolean flag = true;
                int seq = 0;
                for (LogisticsDto logisticsDto:logisticsDtoList) {
                    logisticsTurnover.setHandle(logisticsDto.getHandle());
                    logisticsTurnover.setStatus("COM");
                    logisticsTurnover.setStorageLocation(lt.getLocationCode());
                    logisticsTurnover.setComDateTime(LocalDateTime.now());
                    logisticsTurnover.setIsReceive("N");
                    logisticsTurnover.setuserId(user);
                    logisticsTurnover.setUserName(username);
                    logisticsBill.setComTime(LocalDateTime.now());
                    logisticsBill.setLogisticsBillNo(transportno);
                    logisticsBill.setLogisticsBo(logisticsDto.getHandle());
                    if (!lt.getWorkCenter().equals(logisticsDto.getNextWorkCenterDes())) {
                        flag = false;
                        message.append(seq++ % 3 == 0 ? "\n" : "|   |");
                        message.append(logisticsDto.getSfc());
                        continue;
                    } else {
                        logisticsTurnoverMapper.updateById(logisticsTurnover);
                        logisticsBillMapper.insert(logisticsBill);
                    }
                }
        return flag;
    }
    @Override
    public StoreSfcInfo storeSelectSfc(String sfc) {
        return logisticsTurnoverMapper.storeSelectSfc(sfc);
    }

    @Override
    public String submentInStore(String sfc, double qty, int inStoreType, String user,String storeLocation) {
        int i =logisticsTurnoverMapper.insertRecordInStore(sfc,qty, inStoreType, user,storeLocation);
        if (i>0){
            if (inStoreType==0){
                int tag=logisticsTurnoverMapper.countLedgerBySfc(sfc);
                if (tag==0){
                    logisticsTurnoverMapper.insertRecordLedger(sfc,qty,storeLocation);
                }else {
                    logisticsTurnoverMapper.updateRecordLedger(sfc,qty);
                }
            }
            return "suess";
        }else {
            return "error";
        }

    }

    @Override
    public boolean deleteLogistics(List<LogisticsDto> logisticsDtoList) {
        try {
            if (logisticsDtoList.size()>0){
                LogisticsTurnover logisticsTurnover=new LogisticsTurnover();
                List<LogisticsTurnover> logisticsTurnoverList=new ArrayList<>();
                for (LogisticsDto logisticsDto:logisticsDtoList){
                    logisticsTurnover.setHandle(logisticsDto.getHandle());
                    logisticsTurnover.setOther6("1");
                    logisticsTurnover.setDeleteDateTime(LocalDateTime.now());
                    logisticsTurnover.setDeleteUser(CommonMethods.getUser());
                    this.updateById(logisticsTurnover);
                }
                return true;
            }

        }catch (Exception e){
            e.printStackTrace();
          return false;
        }
      return false;
    }

}