package com.foreverwin.mesnac.meapi.service.impl;

import com.foreverwin.mesnac.meapi.dto.AbnormalRouterDto;
import com.foreverwin.mesnac.meapi.dto.DisRouterStepDto;
import com.foreverwin.mesnac.meapi.dto.UploadPicturesDto;
import com.foreverwin.mesnac.meapi.enums.HandleEnum;
import com.foreverwin.mesnac.meapi.mapper.RouterStepMapper;
import com.foreverwin.mesnac.meapi.model.*;
import com.foreverwin.mesnac.meapi.service.*;
import com.foreverwin.modular.core.exception.BaseException;
import com.foreverwin.modular.core.exception.BusinessException;
import com.foreverwin.modular.core.meext.MEServices;
import com.foreverwin.modular.core.util.CommonMethods;
import com.foreverwin.modular.core.util.FrontPage;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.foreverwin.mesnac.meapi.mapper.RouterMapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.sap.me.productdefinition.*;
import com.sap.me.productdefinition.RouterStep;
import org.eclipse.jetty.util.StringUtil;
import org.springframework.stereotype.Service;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author Leon.L
 * @since 2021-06-02
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class RouterServiceImpl extends ServiceImpl<RouterMapper, Router> implements RouterService {


    @Autowired
    private RouterMapper routerMapper;
    @Autowired
    private RouterService routerService;
    @Autowired
    private OperationService operationService;
    @Autowired
    private RouterStepcontentService routerStepcontentService;
    @Autowired
    private CustomFieldsService customFieldsService;
    @Autowired
    private PicturesService picturesService;
    @Autowired
    private RouterStepMapper routerStepMapper;

    @Override
    public IPage<Router> selectPage(FrontPage<Router> frontPage, Router router) {
        QueryWrapper<Router> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(router);
        return super.page(frontPage.getPagePlus(), queryWrapper);
    }

    @Override
    public List<Router> selectList(Router router) {
        QueryWrapper<Router> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(router);
        return super.list(queryWrapper);
    }

    @Override
    public Router getRouterBySfcBo(String sfcBo) {
        return routerMapper.selectRouterBySfcBo(sfcBo);
    }

    @Override
    public Router getCurrentRouter(String routerBo) {
        return routerMapper.selectCurrentRouter(routerBo);
    }

    @Override
    public Router getMaxRevisionRouter(String site, String router) {
        List<Router> list = routerMapper.selectRouterList4RevisionDesc(site, router);
        if (list == null || list.size() <= 0) {
            return null;
        }

        return list.get(0);
    }

    @Override
    public void CreateDisRouter(DisRouter disRouter) throws Exception {
        //获取工艺路线标准服务
        String site = CommonMethods.getSite();
        RouterConfigurationServiceInterface routerConfigurationService = MEServices.create("com.sap.me.productdefinition", "RouterConfigurationService", site);
//        List<DisRouterStepDto> newList = disRouter.getDisRouterStepDtoList().stream().filter(a ->
//                !disRouter.getOriginRouterStepList().stream().map(b -> b.getStep()).collect(Collectors.toList()).contains(a.getStep())
//        ).collect(Collectors.toList());
//        for (DisRouterStepDto disRouterStepDto:disRouter.getDisRouterStepDtoList()){
//            if (!disRouterStepDto.getStep().startsWith("9")){
//                throw BusinessException.build("请检查返修工序"+"【"+disRouterStepDto.getStep()+"】是否为9开头");
//            }
//        }

        //物料编号+_+异常单号作为工艺路线编号
        String router=disRouter.getRouter();
        if (StringUtil.isBlank(router)) {
            throw BusinessException.build("工艺路线编号不能为空");
        }
        //工艺路线步骤是否有值
        List<DisRouterStepDto> routerStepDtoList = disRouter.getDisRouterStepDtoList();
        if (routerStepDtoList == null || routerStepDtoList.size() <= 0) {
            throw BusinessException.build("工艺路线步骤不能为空");
        }

        //校验router步骤是否重复
        List<DisRouterStepDto> stepDtoList = disRouter.getDisRouterStepDtoList();
        List<Map<String, String>> routerAllStepList = routerStepMapper.findRouterAllStep(disRouter,site);
        for (DisRouterStepDto routerStepDto : stepDtoList) {
            String stepId = routerStepDto.getStep();
            String substring = stepId.substring(0, 2);
            if (!substring.equals("90")){
                stepId = "90" + stepId;
            }
            for (Map<String, String> map : routerAllStepList) {
                String STEP_ID = String.valueOf(map.get("STEP_ID"));
                if (stepId.equals(STEP_ID)){
                    List<String> collect = routerAllStepList.stream().map(e -> { return e.get("STEP_ID"); }).collect(Collectors.toList());
                    throw new BaseException("请重新输入工艺步骤，该工艺路线已存在的返修步骤为:" + collect.toString());
                }
            }
        }

        //工艺路线BO
        Float revision = new Float(1.0);
        String routerType = RouterType.SPECIAL.value();
        String routerBo = HandleEnum.ROUTER.getHandle(site, router, routerType, revision.toString());
        //查询工艺路线是否存在, 不存在版本默认为1.0，存在最大版本+1最为当前版本
        Router routerModel = routerService.getMaxRevisionRouter(site, router);
        if (routerModel != null) {
            Float currentRevision = new Float(routerModel.getRevision());
            revision = currentRevision + 1;
            routerBo = HandleEnum.ROUTER.getHandle(site, router, routerType, revision.toString());
        }

        //工艺路线参数组装
        RouterConfiguration routerConfiguration = new RouterConfiguration();
        routerConfiguration.setRef(routerBo);
        routerConfiguration.setRouter(router);
        routerConfiguration.setDescription(router);
        routerConfiguration.setRevision(revision.toString());
        routerConfiguration.setRouterType(com.sap.me.common.RouterType.SPECIAL);
        routerConfiguration.setCurrentRevision(true);
        routerConfiguration.setStatus(RouterStatus.Releasable);

        //入口步骤
        String entryRouterStep = null;
        //工艺路线步骤集合
        List<RouterStep> routerStepList = new ArrayList<>();

        //自定义字段集合
        List<CustomFields> customFieldsList = new ArrayList<>();

        for (int i = 0; i < disRouter.getDisRouterStepDtoList().size(); i++) {
            DisRouterStepDto routerStepDto = routerStepDtoList.get(i);
            String stepId = routerStepDto.getStep();
            String substring = stepId.substring(0, 2);
            if (!substring.equals("90")){
                stepId = "90" + stepId;
            }
            if (StringUtil.isBlank(entryRouterStep)) {
                entryRouterStep = stepId;
            }

            //上一步骤添加当前步骤为下一步骤
            if (i > 0) {
                RouterNextStep routerNextStep = new RouterNextStep();
                routerNextStep.setNextStepId(stepId);
                routerStepList.get(i-1).getRouterNextStepList().add(routerNextStep);
            }

            String resource = routerStepDto.getOperation();
            /*String operation = routerStepDto.getOPERATION();
            String _revision = routerStepDto.getREVISION();
            _revision = StringUtil.notBlank(_revision) ? _revision : "1.0";
            String description = routerStepDto.getOPERATION_DESCRIPTION();
            Operation operationModel = this.createResourceOfOperation(site, resource, operation, description, _revision);
            String operationBo = operationModel.getHandle();*/
            Operation operationModel = operationService.queryOperationByOperation(site, resource);
            if (operationModel == null) {
                throw BusinessException.build("步骤标识【" + stepId + "】下【" + resource + "】对应的MES工序未维护");
            }
            String operation = operationModel.getOperation();
            String operationBo = operationModel.getHandle();

            //工艺路线步骤
            RouterStep routerStep = new RouterStep();
            routerStep.setSequence(new BigDecimal(i+1));
            routerStep.setStepId(stepId);
            routerStep.setDescription(operation);
            routerStep.setRework(true);
            routerStep.setQueueDecisionType(QueueDecisionType.COMPLETING_OPERATOR);

            //工艺路线步骤工序 RouterComponentContext
            RouterComponentContext routerComponentContext = new RouterComponentContext();
            RouterOperation routerOperation = new RouterOperation();
            //工艺路线步骤工序默认当前版本
            routerOperation.setOperationRef(operationBo);
            routerOperation.setStepType(RouterStepType.NORMAL);
            routerComponentContext.setRouterOperation(routerOperation);
            routerStep.setRouterComponentContext(routerComponentContext);

            //步骤入仓
            routerStepList.add(routerStep);

            //自定义字段
            String routerOperationBO = HandleEnum.ROUTER_OPERATION.getHandle(routerBo, stepId);
            CustomFields customFields = new CustomFields();
            customFields.setHandle(routerOperationBO);
            customFields.setAttribute("PROD_TIME");
            customFields.setValue(routerStepDto.getRepairTime());
            customFieldsList.add(customFields);

            customFields = new CustomFields();
            customFields.setHandle(routerOperationBO);
            customFields.setAttribute("OUT_SOURCE");
            customFields.setValue(stepId);
            customFieldsList.add(customFields);


            RouterStepcontent routerStepcontent=new RouterStepcontent();
            routerStepcontent.setHandle(UUID.randomUUID().toString());
            routerStepcontent.setAbnormalNo(disRouter.getAbnormalNo());
            routerStepcontent.setRepairTime(routerStepDto.getRepairTime());
            routerStepcontent.setRouterStepBo(HandleEnum.ROUTER_STEP.getHandle(routerBo,stepId));
            routerStepcontent.setRouterStepContent(routerStepDto.getOperationInfo());
            try {
                routerStepcontentService.save(routerStepcontent);
            }catch (Exception e){
                throw BusinessException.build(e.getMessage().trim());
            }
            //图片路径
            try {
                this.fileUpload(disRouter,routerBo,stepId);
            }catch (Exception e){
                throw BusinessException.build("图片保存失败");
            }
        }

        String lastStepId = "9999";
        if (router.endsWith("FX")) {
            //工艺路线编号“FX”结尾，标识返修工艺路线，添加返回原始工序步骤
            RouterStep routerStep = new RouterStep();
            routerStep.setSequence(new BigDecimal(lastStepId));
            RouterComponentContext routerComponentContext = new RouterComponentContext();
            RouterReturnStep routerReturnStep = new RouterReturnStep();
            routerReturnStep.setReturnType(ReturnType.ORIGINAL_OPERATION);
            routerReturnStep.setCompleteOriginal(true);
            routerComponentContext.setRouterReturnStep(routerReturnStep);
            routerStep.setRouterComponentContext(routerComponentContext);
            routerStep.setStepId(lastStepId);
            routerStepList.add(routerStep);
        } else {
            //添加完成工序
            RouterStep routerDoneStep = new RouterStep();
            routerDoneStep.setSequence(new BigDecimal(lastStepId));
            routerDoneStep.setStepId(lastStepId);
            RouterComponentContext routerDoneStepComponentContext = new RouterComponentContext();
            routerDoneStepComponentContext.setRouterDoneStep(true);
            routerDoneStep.setRouterComponentContext(routerDoneStepComponentContext);
            routerStepList.add(routerDoneStep);
        }

        //完成工序添加到下一步骤
        RouterNextStep routerNextStep = new RouterNextStep();
        routerNextStep.setNextStepId(lastStepId);
        routerStepList.get(routerStepList.size()-2).getRouterNextStepList().add(routerNextStep);

        routerConfiguration.setEntryRouterStep(entryRouterStep);
        routerConfiguration.setRouterStepList(routerStepList);
        //工艺路线创建
        routerConfigurationService.createRouter(routerConfiguration);


        //自定义数据保存
        if (customFieldsList != null && customFieldsList.size() > 0) {
         customFieldsService.saveOrUpdateCustomFields(customFieldsList);
        }
    }

    @Override
    public com.foreverwin.mesnac.meapi.model.RouterStep findRepairOperation(String site, String routerBo, String stepId) {
        return routerMapper.findRepairOperation(site,routerBo,stepId);
    }

    @Override
    public List<AbnormalRouterDto> selectDisrouter(AbnormalRouterDto abnormalRouterDto) {
         return routerMapper.selectDisrouter(abnormalRouterDto);
    }

    @Override
    public List<AbnormalRouterDto> selectAllDisRouter(AbnormalRouterDto abnormalRouterDto) {
        return routerMapper.selectAllDisRouter(abnormalRouterDto);
    }

    @Override
    public void fileUpload(DisRouter disRouter,String routerBo,String stepId) {
        String user = CommonMethods.getUser();
        LocalDateTime now = LocalDateTime.now();
        String site = CommonMethods.getSite();
        String status = stepId;
        List<String> name = Arrays.asList(disRouter.getFileName().split(","));
        name = name.stream().filter(item -> item.split("/")[0].equals(stepId)).collect(Collectors.toList());
        if(name.size() != 0){
            UploadPicturesDto uploadPictures = new UploadPicturesDto();
            uploadPictures.setHandle("UploadPicturesBO:"+site+","+disRouter.getRouter()+"_"+disRouter.getAbnormalNo()+","+status);
            uploadPictures.setSite(site);
            //文件路径 = 物料编号异常单号 + 步骤
            String filePath = disRouter.getFilePath() + "/" + status;
            uploadPictures.setPicturePath(filePath);
            StringBuilder pictureAddress = new StringBuilder();
            for (int i = 0; i <= name.size() -1; i++) {
                if(i != name.size() -1){
                    pictureAddress.append(filePath).append("/").append(name.get(i).split("/")[1]).append(",");
                }else{
                    pictureAddress.append(filePath).append("/").append(name.get(i).split("/")[1]);
                }
            }
            uploadPictures.setStatus(status);
            uploadPictures.setPictureAddress(pictureAddress.toString());
            uploadPictures.setObjectBo(routerBo);
            Integer num = name.size();
            uploadPictures.setNum(num.toString());
            uploadPictures.setType("Repair");
            uploadPictures.setCreatedUser(user);
            uploadPictures.setCreatedDateTime(now);
            uploadPictures.setModifiedUser(user);
            uploadPictures.setModifiedDateTime(now);
            picturesService.saveOrUpdate(uploadPictures);
        }
    }

}