package com.foreverwin.mesnac.quality.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.foreverwin.mesnac.common.constant.Constants;
import com.foreverwin.mesnac.common.constant.IntegrationTypeConstant;
import com.foreverwin.mesnac.common.dto.SfcDispatchDto;
import com.foreverwin.mesnac.common.enums.HandleEnum;
import com.foreverwin.mesnac.common.model.InspectionTask;
import com.foreverwin.mesnac.common.model.InspectionTaskDetail;
import com.foreverwin.mesnac.common.model.IntegrationLog;
import com.foreverwin.mesnac.common.service.InspectionTaskDetailService;
import com.foreverwin.mesnac.common.service.InspectionTaskService;
import com.foreverwin.mesnac.common.service.IntegrationLogService;
import com.foreverwin.mesnac.common.util.ActiveMQUtil;
import com.foreverwin.mesnac.common.util.DateUtil;
import com.foreverwin.mesnac.common.util.StringUtil;
import com.foreverwin.mesnac.dispatch.model.SfcDispatch;
import com.foreverwin.mesnac.dispatch.service.SfcDispatchService;
import com.foreverwin.mesnac.meapi.dto.SfcDto;
import com.foreverwin.mesnac.meapi.mapper.LogisticsTurnoverMapper;
import com.foreverwin.mesnac.meapi.mapper.SfcMapper;
import com.foreverwin.mesnac.meapi.model.LogisticsTurnover;
import com.foreverwin.mesnac.meapi.model.Router;
import com.foreverwin.mesnac.meapi.service.LogisticsTurnoverService;
import com.foreverwin.mesnac.meapi.service.RouterService;
import com.foreverwin.mesnac.production.mapper.SplitSfcMapper;
import com.foreverwin.mesnac.quality.dto.InspectionTaskManageRequest;
import com.foreverwin.mesnac.quality.mapper.InspectionTaskManageMapper;
import com.foreverwin.mesnac.quality.model.InspectionTaskManageDetail;
import com.foreverwin.mesnac.quality.model.InspectionTaskManage;
import com.foreverwin.mesnac.quality.service.InspectionTaskManageService;
import com.foreverwin.modular.core.exception.BaseException;
import com.foreverwin.modular.core.exception.BusinessException;
import com.foreverwin.modular.core.util.CommonMethods;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;


import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;

@Service
@Transactional(rollbackFor = Exception.class)
public class InspectionTaskManageServiceImpl implements InspectionTaskManageService {

    @Autowired
    private InspectionTaskService inspectionTaskService;
    @Autowired
    private InspectionTaskManageMapper inspectionTaskManageMapper;
    @Autowired
    private InspectionTaskDetailService inspectionTaskDetailService;
    @Autowired
    private SfcDispatchService sfcDispatchService;
    @Autowired
    private LogisticsTurnoverService logisticsTurnoverService;
    @Autowired
    private RouterService routerService;
    @Autowired
    private SplitSfcMapper splitSfcMapper;

    @Autowired
    private ActiveMQUtil activeMQUtil;

    @Autowired
    private SfcMapper sfcMapper;

    @Autowired
    private LogisticsTurnoverMapper logisticsTurnoverMapper;


    private static Logger logger = LoggerFactory.getLogger(InspectionTaskManageServiceImpl.class);



    @Override
    public List<InspectionTaskManage> findInspectionTask(InspectionTaskManageRequest inspectionTaskManageRequest) {
        return inspectionTaskManageMapper.findInspectionTask(inspectionTaskManageRequest);
    }

    @Override
    public InspectionTaskManage findInspectionTaskAndDetail(InspectionTaskManageRequest inspectionTaskManageRequest) {
        List<InspectionTaskManage> manageList = this.findInspectionTask(inspectionTaskManageRequest);
        if (manageList == null || manageList.size() <= 0) {
            throw BusinessException.build("产品条码没有可执行的检验任务！");
        }
        String site = inspectionTaskManageRequest.getSite();
        InspectionTaskManage inspectionTaskManage = manageList.get(0);
        String taskBo = inspectionTaskManage.getHandle();
        String inspectionItemBo = inspectionTaskManage.getInspectionItemBo();

        //查询检验项目
        List<InspectionTaskManageDetail> inspectionItemDetailList = this.getInspectionTaskDetail(taskBo);
        if (!Constants.INSPECTION_TASK_STATUS_NEW.equals(inspectionTaskManage.getStatus())) {
            inspectionTaskManage.setInspectionTaskDetailList(inspectionItemDetailList);
            return inspectionTaskManage;
        }

        if (inspectionItemDetailList == null || inspectionItemDetailList.size() <= 0) {
            inspectionItemDetailList = this.getInspectionItemDetail(inspectionItemBo);
            if (inspectionItemDetailList == null || inspectionItemDetailList.size() <= 0) {
                return inspectionTaskManage;
            }

            List<InspectionTaskDetail> taskDetailList = new ArrayList<>();
            for (InspectionTaskManageDetail itemDetail : inspectionItemDetailList) {
                InspectionTaskDetail inspectionTaskDetail = new InspectionTaskDetail();
                inspectionTaskDetail.setHandle(UUID.randomUUID().toString());
                inspectionTaskDetail.setSite(site);
                inspectionTaskDetail.setTaskBo(taskBo);
                inspectionTaskDetail.setInspectionItemDetailBo(itemDetail.getHandle());
                taskDetailList.add(inspectionTaskDetail);

                BigDecimal checkNum = itemDetail.getCheckNum();
                if (checkNum.compareTo(BigDecimal.ONE) == 1) {
                    for (int i = 0; i < checkNum.intValue()-1; i++) {
                        InspectionTaskDetail taskDetail = new InspectionTaskDetail();
                        BeanUtils.copyProperties(inspectionTaskDetail, taskDetail);
                        taskDetail.setHandle(UUID.randomUUID().toString());
                        taskDetailList.add(taskDetail);
                    }
                }
            }

            if (taskDetailList != null && taskDetailList.size() > 0) {
                inspectionTaskDetailService.saveBatch(taskDetailList);
            }
            inspectionItemDetailList = this.getInspectionTaskDetail(taskBo);
        }

        inspectionTaskManage.setInspectionTaskDetailList(inspectionItemDetailList);
        return inspectionTaskManage;
    }

    @Override
    public List<InspectionTaskManageDetail> getInspectionItemDetail(String inspectionItemBo) {
        return inspectionTaskManageMapper.selectInspectionItemDetail(inspectionItemBo);
    }

    @Override
    public List<InspectionTaskManageDetail> getInspectionTaskDetail(String inspectionTaskBo) {
        return inspectionTaskManageMapper.selectInspectionTaskDetail(inspectionTaskBo);
    }

    @Override
    public void completeInspectionTask(String site, String user, List<InspectionTaskManageRequest> requestList) {
        if (requestList == null || requestList.size() <= 0) {
            return;
        }

        LocalDateTime dateTime = LocalDateTime.now();
        List<InspectionTask> inspectionTaskList = new ArrayList<>();
        for (InspectionTaskManageRequest request : requestList) {
            String status = request.getStatus();
            String taskNo = request.getTaskNo();
            if (!Constants.INSPECTION_TASK_STATUS_NEW.equals(status)) {
                throw BusinessException.build("任务【" +taskNo+ "】不是新建状态，不允许执行合格操作");
            }

            List<InspectionTaskManageDetail> inspectionItemDetailList = this.getInspectionTaskDetail(HandleEnum.INSPECTION_TASK.getHandle(taskNo));
            if (inspectionItemDetailList != null && inspectionItemDetailList.size() > 0) {
                    throw BusinessException.build("检验任务【"+taskNo+"】有检验项目，不允许执行合格操作");
            }
            //验证检验状态

            InspectionTask inspectionTaskcheck = new InspectionTask();
            inspectionTaskcheck = inspectionTaskService.getById(request.getHandle());
            if (Constants.INSPECTION_TASK_STATUS_COMPLETE.equals(inspectionTaskcheck.getStatus())){
                throw BusinessException.build("任务【" +inspectionTaskcheck.getTaskNo()+ "】不是新建状态，不允许执行合格操作,请重新检索数据！");
            }

            //修改数据
            InspectionTask inspectionTask = new InspectionTask();
            inspectionTask.setHandle(request.getHandle());
            inspectionTask.setResult("OK");
            inspectionTask.setStatus(Constants.INSPECTION_TASK_STATUS_COMPLETE);
            inspectionTask.setModifyUser(user);
            inspectionTask.setModifiedDateTime(dateTime);
            inspectionTaskList.add(inspectionTask);

            //质检合格后判断下道工序是否需要跨车间转运，如果是则生成转运待办任务

                /**
                 * 1、先查询派工表（Z_SFC_DISPATCH）里的派工数据
                 * 2、根据派工表里的派工序号查找下道工序
                 * 3、如果两个工作中心不一样，就会生成转运待办任务
                 */

                this.CreateLogisticsTask(request);



            }


        if (inspectionTaskList != null && inspectionTaskList.size() > 0) {
            inspectionTaskService.updateBatchById(inspectionTaskList);
        }


    }

    @Override
    public void saveInspectionTaskDetail(String site, String user, InspectionTaskManage inspectionTaskManage) {
        LocalDateTime dateTime = LocalDateTime.now();
        String type = inspectionTaskManage.getOperationType();

        List<InspectionTaskManageDetail> taskDetailList = inspectionTaskManage.getInspectionTaskDetailList();
        if (taskDetailList != null && taskDetailList.size() > 0) {
            List<InspectionTaskDetail> inspectionTaskDetailList = new ArrayList<>();

            //修改明细数据
            for (InspectionTaskManageDetail manageDetail : taskDetailList) {
                String isRequired = manageDetail.getIsRequired();
                String paramNo = manageDetail.getParamNo();
                String checkValues = manageDetail.getCheckValues();
                if ("Y".equals(isRequired) && StringUtil.isBlank(checkValues) && "SUBMIT".equals(type)) {
                    throw BusinessException.build("检验项目【" + paramNo + "】值必输!");
                }

                InspectionTaskDetail taskDetail = new InspectionTaskDetail();
                taskDetail.setHandle(manageDetail.getHandle());
                taskDetail.setCheckValues(checkValues);
                taskDetail.setResult(manageDetail.getResult());
                taskDetail.setRemark(manageDetail.getRemark());
                taskDetail.setCheckUser(user);
                taskDetail.setCheckDate(dateTime);
                inspectionTaskDetailList.add(taskDetail);
            }

            if (inspectionTaskDetailList != null && inspectionTaskDetailList.size() > 0) {
                inspectionTaskDetailService.updateBatchById(inspectionTaskDetailList);
            }
        }

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        String result = inspectionTaskManage.getResult();
        String taskNo = inspectionTaskManage.getTaskNo();
        //修改主表数据
        InspectionTask inspectionTaskModel = inspectionTaskService.getInspectionTaskByTaskNo(site, taskNo);
        if (inspectionTaskModel == null) {
            throw BusinessException.build("检验任务【" +taskNo+ "】不存在！");
        }
        String status = inspectionTaskModel.getStatus();
        String category = inspectionTaskModel.getCategory();
        if (!Constants.INSPECTION_TASK_STATUS_NEW.equals(status) && !Constants.INSPECTION_TASK_STATUS_DOING.equals(status)) {
            throw BusinessException.build("检验任务【" +taskNo+ "】不是【新建|检验中】，不允许操作！");
        }

        //质检合格后判断下道工序是否需要跨车间转运，如果是则生成转运待办任务
        if("OK".equals(result) && inspectionTaskManage.getOperationType().equals("SUBMIT")){
            /**
             * 1、先查询派工表（Z_SFC_DISPATCH）里的派工数据
             * 2、根据派工表里的派工序号查找下道工序
             * 3、如果两个工作中心不一样，就会生成转运待办任务
             */
            InspectionTaskManageRequest request=new InspectionTaskManageRequest();
            request.setSfcdispatchBo(inspectionTaskManage.getSfcdispatchBo());
            request.setSfc(inspectionTaskManage.getSfc());
            request.setStepId(inspectionTaskManage.getStepId());
            request.setItem(inspectionTaskManage.getItem());
            this.CreateLogisticsTask(request);
        }
        inspectionTaskModel.setStatus("SAVE".equals(type) ? Constants.INSPECTION_TASK_STATUS_DOING : Constants.INSPECTION_TASK_STATUS_COMPLETE);
        inspectionTaskModel.setResult(result);
        inspectionTaskModel.setComments(inspectionTaskManage.getComments());
        inspectionTaskModel.setModifyUser(user);
        inspectionTaskModel.setModifiedDateTime(dateTime);
        inspectionTaskModel.setQuestionC(inspectionTaskManage.getQuestionC());
        inspectionTaskService.updateById(inspectionTaskModel);




        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        //如果专检不合格，生产新的检验任务
        /*if (!Constants.INSPECTION_TYPE_P.equals(category) || !"NG".equals(result) || !"SUBMIT".equals(type)) {
            return;
        }

        //任务主表
        InspectionTask newInspectionTask = new InspectionTask();
        BeanUtils.copyProperties(inspectionTaskModel, newInspectionTask);
        String newTaskNo = "P_" + DateUtil.getTimeString();
        String handle = HandleEnum.INSPECTION_TASK.getHandle(site, newTaskNo);
        newInspectionTask.setHandle(handle);
        newInspectionTask.setTaskNo(newTaskNo);
        newInspectionTask.setStatus(Constants.INSPECTION_TASK_STATUS_NEW);
        newInspectionTask.setResult(null);
        newInspectionTask.setComments(null);
        newInspectionTask.setCreatedDateTime(dateTime);
        newInspectionTask.setModifiedDateTime(dateTime);
        inspectionTaskService.save(newInspectionTask);*/
    }

    public void CreateLogisticsTask(InspectionTaskManageRequest request) {
        // SfcDispatch sfcDispatchById = sfcDispatchService.getById(request.getSfcdispatchBo());
        String site = CommonMethods.getSite();
        SfcDispatch sfcDispatchById = sfcDispatchService.getById(HandleEnum.SFC_DISPATCH.getHandle(site, request.getSfc() + "-" + request.getStepId()));
        Router router = routerService.getById(sfcDispatchById.getRouterBo());
        List<String> stringList=new ArrayList<>();
        stringList.add("HJ_6106JYGX");
        stringList.add("HJ_6106JRGX");
        String dispatchNo = request.getSfc() + "-" + request.getStepId();
        SfcDispatch nextsfcDispatch=null;
        boolean flag=false;
        /**
         * 检验工序为返修工序
         */
        if(sfcDispatchById.getStepId().startsWith("9")){
            //找下道工序
            for (int i = 1; i < 50; i++) {
                QueryWrapper<SfcDispatch> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq(SfcDispatch.SFC, sfcDispatchById.getSfc());
                queryWrapper.eq(SfcDispatch.SHOP_ORDER, sfcDispatchById.getShopOrder());
                queryWrapper.eq(SfcDispatch.DISPATCH_SEQ, String.valueOf(Integer.parseInt(sfcDispatchById.getDispatchSeq()) + i));
               // queryWrapper.like(SfcDispatch.ROUTER_BO, sfcDispatchById.getRouterBo().substring(0,sfcDispatchById.getRouterBo().lastIndexOf(",")));
                queryWrapper.eq(SfcDispatch.ROUTER_BO,sfcDispatchById.getRouterBo());
                nextsfcDispatch = sfcDispatchService.getOne(queryWrapper);
                if (nextsfcDispatch != null && !stringList.contains(nextsfcDispatch.getOperation())
                        && !nextsfcDispatch.getWorkCenter().equals(sfcDispatchById.getWorkCenter())) {
                    flag=true;
                    break;
                }
                else if (nextsfcDispatch != null && !stringList.contains(nextsfcDispatch.getOperation())
                        && nextsfcDispatch.getWorkCenter().equals(sfcDispatchById.getWorkCenter())) {
                    flag=false;
                    break;
                }
                else if (nextsfcDispatch != null && stringList.contains(nextsfcDispatch.getOperation())){
                    continue;
                }
                else if (nextsfcDispatch == null){
                    /**
                     * 找正在排队的工序
                     */
                    Map map = sfcMapper.findOperationBySfc(site, sfcDispatchById.getSfc());
                    if (map !=null){
                        SfcDispatch  sfc_inwork = sfcDispatchService.getById(HandleEnum.SFC_DISPATCH.getHandle(site, sfcDispatchById.getSfc() + "-" + map.get("STEP_ID")));
                        if (sfc_inwork !=null &&!sfcDispatchById.getWorkCenter().equals(sfc_inwork.getWorkCenter())
                                && !stringList.contains(sfc_inwork.getWorkCenter())){
                            nextsfcDispatch=new SfcDispatch();
                            nextsfcDispatch.setOperation(sfc_inwork.getOperation());
                            nextsfcDispatch.setStepId(sfc_inwork.getStepId());
                            nextsfcDispatch.setWorkCenter(sfc_inwork.getWorkCenter());
                            flag=true;
                            break;
                        }
                    }
                }

            }
        }else {
            //找下道工序
            for (int i = 1; i < 50; i++) {
                QueryWrapper<SfcDispatch> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq(SfcDispatch.SFC, sfcDispatchById.getSfc());
                queryWrapper.eq(SfcDispatch.SHOP_ORDER, sfcDispatchById.getShopOrder());
                queryWrapper.eq(SfcDispatch.DISPATCH_SEQ, String.valueOf(Integer.parseInt(sfcDispatchById.getDispatchSeq()) + i));
                queryWrapper.like(SfcDispatch.ROUTER_BO, sfcDispatchById.getRouterBo().substring(0,sfcDispatchById.getRouterBo().lastIndexOf(",")));
               // queryWrapper.eq(SfcDispatch.ROUTER_BO,sfcDispatchById.getRouterBo());
                nextsfcDispatch = sfcDispatchService.getOne(queryWrapper);
                if (nextsfcDispatch != null && !stringList.contains(nextsfcDispatch.getOperation())
                        && !nextsfcDispatch.getWorkCenter().equals(sfcDispatchById.getWorkCenter())) {
                    flag=true;
                    break;
                }
                else if (nextsfcDispatch != null && !stringList.contains(nextsfcDispatch.getOperation())
                        && nextsfcDispatch.getWorkCenter().equals(sfcDispatchById.getWorkCenter())) {
                    flag=false;
                    break;
                }
                else if (nextsfcDispatch != null && stringList.contains(nextsfcDispatch.getOperation())){
                    continue;
                }
                else if (nextsfcDispatch == null){
                    QueryWrapper<SfcDispatch> queryWrappers = new QueryWrapper<>();
                    queryWrappers.eq(SfcDispatch.SFC, sfcDispatchById.getSfc());
                    queryWrappers.eq(SfcDispatch.SHOP_ORDER, sfcDispatchById.getShopOrder());
                    queryWrappers.eq(SfcDispatch.DISPATCH_SEQ, String.valueOf(Integer.parseInt(sfcDispatchById.getDispatchSeq()) + (i-1)));
                    queryWrappers.like(SfcDispatch.ROUTER_BO, sfcDispatchById.getRouterBo().substring(0,sfcDispatchById.getRouterBo().lastIndexOf(",")));
                    // queryWrappers.eq(SfcDispatch.ROUTER_BO,sfcDispatchById.getRouterBo());
                    nextsfcDispatch = sfcDispatchService.getOne(queryWrappers);
                    if (nextsfcDispatch.getOther2() != null
                            &&  Constants.RK.equals(nextsfcDispatch.getOther2()) && stringList.contains(nextsfcDispatch.getOperation())){
                        nextsfcDispatch.setOperation(nextsfcDispatch.getOperation());
                        nextsfcDispatch.setStepId(nextsfcDispatch.getStepId());
                        nextsfcDispatch.setWorkCenter("BCP");
                    } else {
                        return;
                    }
                    flag=true;
                    break;
                }
            }
        }


        if (flag){
            LogisticsTurnover logisticsTurnover = new LogisticsTurnover();
            logisticsTurnover.setHandle("LogisticsBO:1000" + "," + UUID.randomUUID().toString());
            logisticsTurnover.setSfcDispatchBo(HandleEnum.SFC_DISPATCH.getHandle(site, dispatchNo));
            logisticsTurnover.setStatus(Constants.LOGISTICS_NEW);
            logisticsTurnover.setCreatedDateTime(LocalDateTime.now());
            logisticsTurnover.setOther1(nextsfcDispatch.getOperation());
            logisticsTurnover.setOther2(sfcDispatchById.getStepId());
            logisticsTurnover.setNextStep(nextsfcDispatch.getStepId());
            logisticsTurnover.setOther3(request.getHandle());
            logisticsTurnover.setIsReceive("N");
            logisticsTurnover.setTurnoverWorkCenter(nextsfcDispatch.getWorkCenter());
            logisticsTurnover.setTurnoutItem(request.getItem());
            logisticsTurnover.setTurnoutWorkcenter(sfcDispatchById.getWorkCenter());
            try {
                logisticsTurnoverService.save(logisticsTurnover);
                /**
                 * 发送MQ消息
                 */
               //消息内容:XX车间操作者xxx加工并质检完的物料
               // Map messageMap=logisticsTurnoverMapper.messageMap();
                //   activeMQUtil.SendMessage("SendMessageHJ02");
            } catch (Exception e) {
                throw new BaseException("周转任务生成失败");
            }
        }

    }


}
